@@ -19,6 +19,10 @@
  * 02111-1307, USA.
  */
 
+/*!
+ * \todo build fails when abiword built without printing support.
+ */
+
 #ifdef ABI_PLUGIN_BUILTIN
 #define abi_plugin_register abipgn_abicommand_register
 #define abi_plugin_unregister abipgn_abicommand_unregister
@@ -34,6 +38,7 @@
 #include <errno.h>
 
 #include <glib.h>
+#include <libgnomeprint/gnome-print-job.h>
 
 #include "xap_Module.h"
 #include "xap_App.h"
@@ -45,10 +50,11 @@
 #include "ie_types.h"
 #include "ap_Convert.h"
 #include "ap_UnixApp.h"
-#include "xap_UnixNullGraphics.h"
-#include "xap_UnixPSGraphics.h"
+#include "gr_UnixPangoGraphics.h"
+#include "gr_UnixNullGraphics.h"
 #include "ap_UnixFrame.h"
 #include "gr_DrawArgs.h"
+#include "ap_EditMethods.h"
 
 #include "AbiCommand.h"
 
@@ -170,9 +176,32 @@ AbiCommand::AbiCommand (void) :
 	m_pCurView (NULL),
 	m_pG (NULL),
 	m_pLayout (NULL),
-	m_bViewDoc (false), m_bRunAsServer (false), m_iPID (0), m_sErrorFile ("")
+	m_bViewDoc (false), 
+	m_bRunAsServer (false), 
+	m_iPID (0), 
+	m_bRunAsAbiCollab(false),
+	m_sErrorFile ("")
 {
 	m_pApp = XAP_App::getApp ();
+	m_pApp->getGraphicsFactory()->registerAsDefault(GRID_UNIX_NULL,true);
+}
+
+
+AbiCommand::AbiCommand (bool bAbiCollab) :
+	m_pCurDoc (NULL),
+	m_pCurFile (new UT_UTF8String),
+	m_pCurFrame (NULL),
+	m_pCurView (NULL),
+	m_pG (NULL),
+	m_pLayout (NULL),
+	m_bViewDoc (false), 
+	m_bRunAsServer (false), 
+	m_iPID (0), 
+	m_bRunAsAbiCollab(bAbiCollab),
+	m_sErrorFile ("")
+{
+	m_pApp = XAP_App::getApp ();
+	m_pApp->getGraphicsFactory()->registerAsDefault(GRID_UNIX_NULL,true);
 }
 
 AbiCommand::~AbiCommand (void)
@@ -235,8 +264,8 @@ AbiCommand::doCommands (void)
 			UT_UTF8String *pTok =
 				const_cast < UT_UTF8String * >(static_cast < const UT_UTF8String * >(toks.getNthItem (0)));
 
-			if (pTok && UT_strcmp (pTok->utf8_str (), "quit") == 0
-				|| UT_strcmp (pTok->utf8_str (), "q") == 0)
+			if (pTok && strcmp (pTok->utf8_str (), "quit") == 0
+				|| strcmp (pTok->utf8_str (), "q") == 0)
 				bQuit = true;
 			else
 			{
@@ -330,7 +359,7 @@ AbiCommand::parseTokens (UT_Vector * pToks)
 	//
 	// New document
 	//
-	if (UT_strcmp (pCom0->utf8_str (), "new") == 0)
+	if (strcmp (pCom0->utf8_str (), "new") == 0)
 	{
 		printf ("Attempting to create a new document \n");
 		PD_Document *pDoc = new PD_Document (m_pApp);
@@ -353,7 +382,7 @@ AbiCommand::parseTokens (UT_Vector * pToks)
 	//
 	// Load in a document
 	//
-	if (UT_strcmp (pCom0->utf8_str (), "load") == 0)
+	if (strcmp (pCom0->utf8_str (), "load") == 0)
 	{
 		printf ("Attempting to load a document \n");
 		if (count >= 2)
@@ -371,10 +400,8 @@ AbiCommand::parseTokens (UT_Vector * pToks)
 					error);
 				return static_cast < UT_sint32 > (error);
 			}
-
 			replaceDocument (pDoc);
 			m_pCurFile->assign (pCom1->utf8_str ());
-
 			return 0;
 		}
 	}
@@ -382,7 +409,7 @@ AbiCommand::parseTokens (UT_Vector * pToks)
 	//
 	// printfile
 	//
-	else if (UT_strcmp (pCom0->utf8_str (), "printfile") == 0)
+	else if (strcmp (pCom0->utf8_str (), "printfile") == 0)
 	{
 		if (count >= 2)
 		{
@@ -396,7 +423,7 @@ AbiCommand::parseTokens (UT_Vector * pToks)
 	//
 	// inserttext
 	//
-	else if (UT_strcmp (pCom0->utf8_str (), "inserttext") == 0)
+	else if (strcmp (pCom0->utf8_str (), "inserttext") == 0)
 	{
 		if (count >= 2)
 		{
@@ -410,7 +437,7 @@ AbiCommand::parseTokens (UT_Vector * pToks)
 	//
 	// delete
 	//
-	else if (UT_strcmp (pCom0->utf8_str (), "delete") == 0)
+	else if (strcmp (pCom0->utf8_str (), "delete") == 0)
 	{
 		if (deleteText (pToks))
 			return 0;
@@ -421,7 +448,7 @@ AbiCommand::parseTokens (UT_Vector * pToks)
 	//
 	// Replace Next
 	//
-	else if (UT_strcmp (pCom0->utf8_str (), "replacenext") == 0)
+	else if (strcmp (pCom0->utf8_str (), "replacenext") == 0)
 	{
 		if (count > 2)
 			return RES_TO_STATUS (replaceNext (pToks));
@@ -430,7 +457,7 @@ AbiCommand::parseTokens (UT_Vector * pToks)
 	//
 	// replaceAll
 	//
-	else if (UT_strcmp (pCom0->utf8_str (), "replaceall") == 0)
+	else if (strcmp (pCom0->utf8_str (), "replaceall") == 0)
 	{
 		if (count > 2)
 			return RES_TO_STATUS (replaceAll (pToks));
@@ -439,7 +466,7 @@ AbiCommand::parseTokens (UT_Vector * pToks)
 	//
 	// Move point to somewhere
 	//
-	else if (UT_strcmp (pCom0->utf8_str (), "movept") == 0)
+	else if (strcmp (pCom0->utf8_str (), "movept") == 0)
 	{
 		if (count > 1)
 			return RES_TO_STATUS (movePoint (pToks));
@@ -448,7 +475,7 @@ AbiCommand::parseTokens (UT_Vector * pToks)
 	//
 	// Open a graphical window on the document
 	//
-	else if (UT_strcmp (pCom0->utf8_str (), "visualedit") == 0)
+	else if (strcmp (pCom0->utf8_str (), "visualedit") == 0)
 	{
 		return RES_TO_STATUS (viewDoc ());
 	}
@@ -456,7 +483,7 @@ AbiCommand::parseTokens (UT_Vector * pToks)
 	//
 	// Start selection
 	//
-	else if (UT_strcmp (pCom0->utf8_str (), "selectstart") == 0)
+	else if (strcmp (pCom0->utf8_str (), "selectstart") == 0)
 	{
 		if (m_pCurView)
 		{
@@ -473,7 +500,7 @@ AbiCommand::parseTokens (UT_Vector * pToks)
 	//
 	// Clear selection
 	//
-	else if (UT_strcmp (pCom0->utf8_str (), "selectclear") == 0)
+	else if (strcmp (pCom0->utf8_str (), "selectclear") == 0)
 	{
 		if (m_pCurView)
 		{
@@ -488,7 +515,7 @@ AbiCommand::parseTokens (UT_Vector * pToks)
 	//
 	// findnext
 	//
-	else if (UT_strcmp (pCom0->utf8_str (), "findnext") == 0)
+	else if (strcmp (pCom0->utf8_str (), "findnext") == 0)
 	{
 		if (m_pCurView && (pToks->getItemCount () > 1))
 		{
@@ -513,7 +540,7 @@ AbiCommand::parseTokens (UT_Vector * pToks)
 	//
 	// Save
 	//
-	else if (UT_strcmp (pCom0->utf8_str (), "save") == 0)
+	else if (strcmp (pCom0->utf8_str (), "save") == 0)
 	{
 		if (m_pCurDoc)
 		{
@@ -540,13 +567,13 @@ AbiCommand::parseTokens (UT_Vector * pToks)
 						(m_pCurDoc->getLastOpenedType ());
 				}
 
-				m_pCurDoc->saveAs (pCom1->utf8_str (), ieft);
+				static_cast<AD_Document*>(m_pCurDoc)->saveAs (pCom1->utf8_str (), ieft);
 
 				return 0;
 			}
 
 			ieft = static_cast < IEFileType > (m_pCurDoc->getLastOpenedType ());
-			m_pCurDoc->saveAs (m_pCurFile->utf8_str (), ieft);
+			static_cast<AD_Document*>(m_pCurDoc)->saveAs (m_pCurFile->utf8_str (), ieft);
 
 			return 0;
 		}
@@ -557,8 +584,8 @@ AbiCommand::parseTokens (UT_Vector * pToks)
 	//
 	// Convert to Text
 	//
-	else if (UT_strcmp (pCom0->utf8_str (), "converttotext") == 0 ||
-			 UT_strcmp (pCom0->utf8_str (), "convert") == 0)
+	else if (strcmp (pCom0->utf8_str (), "converttotext") == 0 ||
+			 strcmp (pCom0->utf8_str (), "convert") == 0)
 	{
 		AP_Convert APConvert;
 		UT_UTF8String src;
@@ -606,7 +633,7 @@ AbiCommand::parseTokens (UT_Vector * pToks)
 	//
 	// Write PID to file
 	//
-	else if (UT_strcmp (pCom0->utf8_str (), "writepid") == 0)
+	else if (strcmp (pCom0->utf8_str (), "writepid") == 0)
 	{
 		if (pToks->getItemCount () < 2)
 			return -1;
@@ -627,7 +654,7 @@ AbiCommand::parseTokens (UT_Vector * pToks)
 	//
 	// server
 	//      
-	else if (UT_strcmp (pCom0->utf8_str (), "server") == 0)
+	else if (strcmp (pCom0->utf8_str (), "server") == 0)
 	{
 		if (pToks->getItemCount () < 2)
 			return -1;
@@ -642,7 +669,7 @@ AbiCommand::parseTokens (UT_Vector * pToks)
 	//
 	// PNG preview
 	//
-	else if (UT_strcmp (pCom0->utf8_str (), "previewpng") == 0)
+	else if (strcmp (pCom0->utf8_str (), "previewpng") == 0)
 	{
 		UT_UTF8String src, destPNG, sWidth, sHeight;
 
@@ -670,7 +697,7 @@ AbiCommand::parseTokens (UT_Vector * pToks)
 	//
 	// Help
 	//
-	else if (UT_strcmp (pCom0->utf8_str (), "help") == 0)
+	else if (strcmp (pCom0->utf8_str (), "help") == 0)
 	{
 		printf ("Currently implemented commands are:\n");
 		printf ("help                        - Prints this message\n");
@@ -746,6 +773,65 @@ AbiCommand::parseTokens (UT_Vector * pToks)
 	return -1;
 }
 
+/*!
+ * Load the document identified by the path sPathToDoc into Abiword
+ */
+bool AbiCommand::loadDocument(UT_UTF8String & sPathToDoc)
+{
+  //
+  // Load in a document
+  //
+
+      PD_Document *pDoc = new PD_Document (m_pApp);
+  
+      UT_Error error = pDoc->readFromFile (sPathToDoc.utf8_str (), IEFT_Unknown);
+
+      if (error != UT_OK)
+      {
+           UNREFP (pDoc);
+	   printf ("Error loading %s error %d \n", sPathToDoc.utf8_str (),error);
+	   return false;
+      }
+      replaceDocument (pDoc);
+      m_pCurFile->assign (sPathToDoc.utf8_str ());
+      return true;
+}
+
+/*!
+ * Create a new Document
+ */
+bool AbiCommand::newDocument(void)
+{
+	//
+	// New document
+	//
+
+  PD_Document *pDoc = new PD_Document (m_pApp);
+
+  UT_Error error = pDoc->newDocument ();
+  
+  if (error != UT_OK)
+  {
+      UNREFP (pDoc);
+      printf ("Error creating new document error %d \n", error);
+      return false;;
+  }
+
+  replaceDocument (pDoc);
+  m_pCurFile->assign ("");
+
+  return true;
+}
+
+
+/*!
+ * Return a pointer to the current document
+ */
+PD_Document * AbiCommand::getCurrentDocument(void)
+{
+  return m_pCurDoc;
+}
+
 //
 // This method calls the method defined in ap_EditMethod.cpp via it's
 // name with the current nullgraphics view as the controlling view.
@@ -791,27 +877,27 @@ AbiCommand::movePoint (UT_Vector * pToks)
 		bool bAbsMove = false;
 		UT_sint32 amt = 0;
 
-		if (UT_stricmp (pTarget->utf8_str (), "BOD") == 0)
+		if (g_ascii_strcasecmp (pTarget->utf8_str (), "BOD") == 0)
 			docpos = FV_DOCPOS_BOD;
-		else if (UT_stricmp (pTarget->utf8_str (), "EOD") == 0)
+		else if (g_ascii_strcasecmp (pTarget->utf8_str (), "EOD") == 0)
 			docpos = FV_DOCPOS_EOD;
-		else if (UT_stricmp (pTarget->utf8_str (), "BOB") == 0)
+		else if (g_ascii_strcasecmp (pTarget->utf8_str (), "BOB") == 0)
 			docpos = FV_DOCPOS_BOB;
-		else if (UT_stricmp (pTarget->utf8_str (), "EOB") == 0)
+		else if (g_ascii_strcasecmp (pTarget->utf8_str (), "EOB") == 0)
 			docpos = FV_DOCPOS_EOB;
-		else if (UT_stricmp (pTarget->utf8_str (), "BOP") == 0)
+		else if (g_ascii_strcasecmp (pTarget->utf8_str (), "BOP") == 0)
 			docpos = FV_DOCPOS_BOP;
-		else if (UT_stricmp (pTarget->utf8_str (), "EOP") == 0)
+		else if (g_ascii_strcasecmp (pTarget->utf8_str (), "EOP") == 0)
 			docpos = FV_DOCPOS_EOP;
-		else if (UT_stricmp (pTarget->utf8_str (), "BOL") == 0)
+		else if (g_ascii_strcasecmp (pTarget->utf8_str (), "BOL") == 0)
  			docpos = FV_DOCPOS_BOL;
-		else if (UT_stricmp (pTarget->utf8_str (), "EOL") == 0)
+		else if (g_ascii_strcasecmp (pTarget->utf8_str (), "EOL") == 0)
 			docpos = FV_DOCPOS_EOL;
-		else if (UT_stricmp (pTarget->utf8_str (), "BOS") == 0)
+		else if (g_ascii_strcasecmp (pTarget->utf8_str (), "BOS") == 0)
 			docpos = FV_DOCPOS_BOS;
-		else if (UT_stricmp (pTarget->utf8_str (), "EOS") == 0)
+		else if (g_ascii_strcasecmp (pTarget->utf8_str (), "EOS") == 0)
 			docpos = FV_DOCPOS_EOS;
-		else if (UT_stricmp (pTarget->utf8_str (), "BOW") == 0)
+		else if (g_ascii_strcasecmp (pTarget->utf8_str (), "BOW") == 0)
 			docpos = FV_DOCPOS_BOW;
 		else if (*(pTarget->utf8_str ()) == '+'
 				 || *(pTarget->utf8_str ()) == '-')
@@ -978,60 +1064,58 @@ bool
 AbiCommand::printFiles (UT_Vector * pToks)
 {
 	UT_return_val_if_fail(m_pCurDoc, false);
-
+	bool portrait = true;
 	for (UT_uint32 i = 1; i < pToks->getItemCount (); i++)
 	{
-		UT_UTF8String *pFile =
-			const_cast < UT_UTF8String * >(static_cast < const UT_UTF8String * >(pToks->getNthItem (i)));
-		PS_GraphicsAllocInfo
-			ai (pFile->utf8_str (), m_pCurFile->utf8_str (),
-				m_pApp->getApplicationName (),
-				static_cast < XAP_UnixApp * >(m_pApp)->getFontManager (), true, m_pApp);
-
-		PS_Graphics *pG = (PS_Graphics *) m_pApp->newGraphics (ai);
-		UT_return_val_if_fail(pG, false);
+		UT_UTF8String *pPrinter =
+		  const_cast < UT_UTF8String * >(static_cast < const UT_UTF8String * >(pToks->getNthItem (i)));
 
+		GR_GraphicsFactory * pGF;
+		
+		pGF = XAP_App::getApp()->getGraphicsFactory();
+		UT_return_val_if_fail(pGF, false);
+		
+		UT_uint32 iDefaultPrintClass = pGF->getDefaultClass(false);		   
+		
+		GnomePrintJob *job = gnome_print_job_new (NULL);
+		UT_return_val_if_fail(job, false);
+		
+		GnomePrintConfig *config = gnome_print_job_get_config (job);
+		UT_return_val_if_fail(config, false);
+		
+		// pPrinter is a printer name, and "-" is our special name for the default printer.
+		if(strcmp(pPrinter->utf8_str(), "-") != 0) {
+		  // should we set 'Settings.Transport.Backend.Printer''? It looks deprecated, but maybe GnomePrint's lpr backend uses it...
+		  gnome_print_config_set(config, reinterpret_cast<const guchar*>("Settings.Transport.Backend.Printer"), 
+					 reinterpret_cast<const guchar*>(pPrinter->utf8_str()));
+		  gnome_print_config_set(config, reinterpret_cast<const guchar*>("Printer"), reinterpret_cast<const guchar*>(pPrinter->utf8_str()));
+		}
+		GR_Graphics *print_graphics;
+		print_graphics = new GR_UnixPangoPrintGraphics(job);
+		
 		// create a new layout and view object for the doc
-		FL_DocLayout *pDocLayout = new FL_DocLayout (m_pCurDoc, pG);
+		FL_DocLayout *pDocLayout = new FL_DocLayout (m_pCurDoc, print_graphics);
 		FV_View *printView = new FV_View (m_pApp, 0, pDocLayout);
 
-		pDocLayout->fillLayouts ();
-		pDocLayout->formatAll ();
-
-		// get the width, height, orient
-		UT_sint32 iWidth = pDocLayout->getWidth ();
-		UT_sint32 iHeight = pDocLayout->getHeight () / pDocLayout->countPages ();
-
-		bool orient = printView->getPageSize ().isPortrait ();
-
-		pG->setPortrait (orient);
-
-		// setup the drawing args
-		dg_DrawArgs da;
-
-		memset (&da, 0, sizeof (da));
-		da.pG = NULL;
-
-		pG->setColorSpace (GR_Graphics::GR_COLORSPACE_COLOR);
-		pG->setPageSize (printView->getPageSize ().getPredefinedName ());
-
-		if (pG->startPrint ())
+		printView->getLayout()->fillLayouts();
+		printView->getLayout()->formatAll();
+		printView->getLayout()->recalculateTOCFields();
+		portrait = printView->getPageSize().isPortrait();
+		if(!portrait)
 		{
-			// iterate over the pages, printing each one
-			for (UT_uint32 k = 1; (k <= pDocLayout->countPages ()); k++)
-			{
-				pG->m_iRasterPosition = (k - 1) * iHeight;
-				pG->startPage (pFile->utf8_str (), k, orient, iWidth,
-							   iHeight);
-				printView->draw (k - 1, &da);
-			}
+		    gnome_print_config_set (config, reinterpret_cast<const guchar *>(GNOME_PRINT_KEY_PAPER_ORIENTATION) , reinterpret_cast<const guchar *>("R90"));
 
-			pG->endPrint ();
 		}
+        
+		s_actuallyPrint (m_pCurDoc, print_graphics,
+				 printView, pPrinter->utf8_str(), 
+				 1, true, 
+				 pDocLayout->getWidth(), pDocLayout->getHeight() / pDocLayout->countPages(), 
+				 pDocLayout->countPages(), 1);
 
 		DELETEP (pDocLayout);
 		DELETEP (printView);
-		DELETEP (pG);
+		DELETEP (print_graphics);
 	}
 	return true;
 }
@@ -1065,11 +1149,11 @@ AbiCommand::replaceDocument (PD_Document * pDoc)
 	m_pCurDoc = pDoc;
 	XAP_UnixApp *pUnixApp = static_cast < XAP_UnixApp * >(m_pApp);
 
-	m_pCurFrame = new AP_UnixFrame (pUnixApp);
+	m_pCurFrame = new AP_UnixFrame();
 	UT_UTF8String extension (".bak~");
 
 	m_pCurFrame->setAutoSaveFileExt (extension.utf8_str ());
-	XAP_UnixNullGraphicsAllocInfo ai (pUnixApp->getFontManager (), m_pApp);
+	GR_UnixNullGraphicsAllocInfo ai;
 
 	m_pG = (UnixNull_Graphics *) m_pApp->newGraphics (ai);
 
