@@ -51,7 +51,8 @@
 LinkGrammarWrap::LinkGrammarWrap(void) 
 {
   m_Opts = parse_options_create();
-  m_Dict = dictionary_create_lang("en");
+  m_Dict = dictionary_create((char*)"en/4.0.dict", (char*)"en/4.0.knowledge",
+	(char*)"en/4.0.constituent-knowledge", (char*)"en/4.0.affix");
   parse_options_set_max_parse_time(m_Opts, 1); // 1 second max parse time
 }
 
@@ -227,9 +228,9 @@ bool LinkGrammarWrap::parseSentence(PieceOfText * pT)
 	  //if(szViolation != NULL)
 	  {
 	    UT_sint32 iLword = linkage_get_link_lword(linkage,j);
-	    char * szLword =  linkage_get_word(linkage,iLword);
+	    const char * szLword =  linkage_get_word(linkage,iLword);
 	    UT_sint32 iRword = linkage_get_link_rword(linkage,j);
-	    char * szRword =  linkage_get_word(linkage,iRword);
+	    const char * szRword =  linkage_get_word(linkage,iRword);
 	    //UT_DEBUGMSG(("Link %d |%s| and |%s|\n",i,szLword,szRword));
 	    //UT_DEBUGMSG(("Left Word num %d Right Word num %d\n",iLword,iRword));
 	  }
