@@ -105,7 +105,7 @@ inline static char * _ucs4ToLatin1(const UT_UCS4Char * text)
 static bool _getTranslationCode(FV_View * pView, UT_String & langCode)
 {
 	XAP_Frame * pFrame = static_cast <XAP_Frame *>(pView->getParentData());
-	UT_ASSERT(pFrame);
+	UT_return_val_if_fail(pFrame,false);
 
 	bool bRet = false;
 
@@ -116,14 +116,11 @@ static bool _getTranslationCode(FV_View * pView, UT_String & langCode)
 	XAP_DialogFactory * pDialogFactory = static_cast<XAP_DialogFactory *>(pFrame->getDialogFactory());
 
 	XAP_Dialog_Language * pDialog = static_cast<XAP_Dialog_Language *>(pDialogFactory->requestDialog(id));
-	UT_ASSERT(pDialog);
-
-	if (!pDialog)
-	return false;
+	UT_return_val_if_fail(pDialog, false);
 
 	UT_String code;
 
-	const XML_Char ** props_in = NULL;
+	const gchar ** props_in = NULL;
 	if (pView->getCharFormat(&props_in))
    	{
 		code = UT_getAttribute("lang", props_in);
@@ -144,7 +141,7 @@ static bool _getTranslationCode(FV_View * pView, UT_String & langCode)
 
 	if (bOK)
    	{
-		const XML_Char * s;
+		const gchar * s;
 		if (pDialog->getChangedLangProperty(&s))
 	   	{
 			UT_String changedLang = s;
@@ -369,6 +366,8 @@ static void FreeTranslation_addToMenus()
 // 
 // -----------------------------------------------------------------------
 
+ABI_PLUGIN_DECLARE ("AbiFreeTranslation")
+
 ABI_FAR_CALL int abi_plugin_register(XAP_ModuleInfo * mi)
 {
 	mi->name = "FreeTranslation plugin";
