@@ -78,7 +78,7 @@ _ucsToAscii(const UT_UCSChar* text)
 //   This is the function that we actually call to invoke the on-line dictionary.
 //   It should be called when the user selects from the context menu
 //
-bool 
+static bool 
 URLDict_invoke(AV_View* v, EV_EditMethodCallData *d)
 {
   // Get the current view that the user is in.
@@ -225,6 +225,8 @@ URLDict_addToMenus()
 //
 // -----------------------------------------------------------------------
 
+ABI_PLUGIN_DECLARE ("AbiURLDict")
+
 ABI_FAR_CALL
 int abi_plugin_register (XAP_ModuleInfo * mi)
 {
