@@ -38,7 +38,7 @@
  */
 void ODe_Style_PageLayout::fetchAttributesFromAbiDoc(PD_Document* pAbiDoc) {
     
-    const XML_Char* pDimensionName;
+    const gchar* pDimensionName;
     UT_Dimension docUnit;
     UT_LocaleTransactor t(LC_NUMERIC, "C");    
 
@@ -81,12 +81,12 @@ void ODe_Style_PageLayout::fetchAttributesFromAbiDoc(PD_Document* pAbiDoc) {
  * Fetch attributes from an AbiWord <section> tag. Mostly page margins.
  */
 void ODe_Style_PageLayout::fetchAttributesFromAbiSection(const PP_AttrProp* pAP) {
-    const XML_Char* pValue;
+    const gchar* pValue;
     bool ok;
     bool hasAbiHeaderMargin = false;
     bool hasAbiFooterMargin = false;
-    double abiHeaderMarginCM;
-    double abiFooterMarginCM;
+    double abiHeaderMarginCM = 0.0;
+    double abiFooterMarginCM = 0.0;
     double abiTopMarginCM;
     double abiBottomMarginCM;
     UT_LocaleTransactor t(LC_NUMERIC, "C");    
@@ -102,6 +102,23 @@ void ODe_Style_PageLayout::fetchAttributesFromAbiSection(const PP_AttrProp* pAP)
      * OpenDocument's header height == AbiWord's top margin -
      *                                 AbiWord's header margin
      */
+
+    ok = pAP->getProperty("page-margin-top", pValue);
+    if (ok && pValue != NULL) {
+        
+        if (hasAbiHeaderMargin) {
+            abiTopMarginCM = UT_convertToDimension(pValue, DIM_CM);
+            
+            UT_UTF8String_sprintf(m_headerHeight, "%fcm",
+                                  abiTopMarginCM - abiHeaderMarginCM);
+        } else {
+            m_marginTop = pValue;
+        }
+    }
+    if(m_marginTop.size() == 0)
+    {
+	m_marginTop = "1.0in";
+    }
     
     ok = pAP->getProperty("page-margin-header", pValue);
     if (ok && pValue != NULL) {
@@ -115,35 +132,7 @@ void ODe_Style_PageLayout::fetchAttributesFromAbiSection(const PP_AttrProp* pAP)
         
         // Redefine the top margin
         hasAbiHeaderMargin = true;
-        m_marginTop = pValue;
-    }
-
-    ok = pAP->getProperty("page-margin-footer", pValue);
-    if (ok && pValue != NULL) {
-        
-        abiFooterMarginCM = UT_convertToDimension(pValue, DIM_CM);
-        
-        // Set the footer height
-        abiBottomMarginCM = UT_convertToDimension(m_marginBottom.utf8_str(), DIM_CM);
-        UT_UTF8String_sprintf(m_footerHeight, "%fcm",
-                              abiBottomMarginCM - abiFooterMarginCM);
-        
-        // Redefine the bottom margin
-        hasAbiFooterMargin = true;
-        m_marginBottom = pValue;
-    }
-
-    ok = pAP->getProperty("page-margin-top", pValue);
-    if (ok && pValue != NULL) {
-        
-        if (hasAbiHeaderMargin) {
-            abiTopMarginCM = UT_convertToDimension(pValue, DIM_CM);
-            
-            UT_UTF8String_sprintf(m_headerHeight, "%fcm",
-                                  abiTopMarginCM - abiHeaderMarginCM);
-        } else {
-            m_marginTop = pValue;
-        }
+        UT_UTF8String_sprintf(m_marginTop, "%fcm",abiHeaderMarginCM);
     }
 
     ok = pAP->getProperty("page-margin-bottom", pValue);
@@ -158,16 +147,61 @@ void ODe_Style_PageLayout::fetchAttributesFromAbiSection(const PP_AttrProp* pAP)
             m_marginBottom = pValue;
         }
     }
+    if(m_marginBottom.size() == 0)
+    {
+	m_marginBottom = "1.0in";
+    }
+
+    ok = pAP->getProperty("page-margin-footer", pValue);
+    if (ok && pValue != NULL) {
+        
+        abiFooterMarginCM = UT_convertToDimension(pValue, DIM_CM);
+        
+        // Set the footer height
+        abiBottomMarginCM = UT_convertToDimension(m_marginBottom.utf8_str(), DIM_CM);
+        UT_UTF8String_sprintf(m_footerHeight, "%fcm",
+                              abiBottomMarginCM - abiFooterMarginCM);
+        
+        // Redefine the bottom margin
+        hasAbiFooterMargin = true;
+        UT_UTF8String_sprintf(m_marginBottom, "%fcm",abiFooterMarginCM);
+    }
 
     ok = pAP->getProperty("page-margin-left", pValue);
     if (ok && pValue != NULL) {
         m_marginLeft = pValue;
+	UT_DEBUGMSG(("Found MarginLeft of section %s \n",pValue));
+    }
+    if(m_marginLeft.size() == 0)
+    {
+	m_marginLeft = "1.0in";
     }
 
     ok = pAP->getProperty("page-margin-right", pValue);
     if (ok && pValue != NULL) {
         m_marginRight = pValue;
     }
+    if(m_marginRight.size() == 0)
+    {
+	m_marginRight = "1.0in";
+    }
+
+    ok = pAP->getProperty("background-color", pValue);
+    if (ok && pValue != NULL) {
+        int len = strlen(pValue);
+        if(len == 6) {
+            m_backgroundColor = UT_UTF8String_sprintf("%s", pValue);
+        } else if(len == 7) {
+            m_backgroundColor = pValue;
+        } else {
+            UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
+        }
+    }
+
+    ok = pAP->getAttribute("strux-image-dataid", pValue);
+    if (ok && pValue != NULL) {
+        m_backgroundImage = pValue;
+    }
 }
 
 
@@ -175,7 +209,7 @@ void ODe_Style_PageLayout::fetchAttributesFromAbiSection(const PP_AttrProp* pAP)
  * 
  */
 bool ODe_Style_PageLayout::hasPageLayoutInfo(const PP_AttrProp* pAP) {
-    const XML_Char* pValue;
+    const gchar* pValue;
     bool ok;
     
     ok = pAP->getProperty("page-margin-header", pValue);
@@ -208,6 +242,16 @@ bool ODe_Style_PageLayout::hasPageLayoutInfo(const PP_AttrProp* pAP) {
         return true;
     }
 
+    ok = pAP->getProperty("background-color", pValue);
+    if (ok && pValue != NULL) {
+        return true;
+    }
+
+    ok = pAP->getAttribute("strux-image-dataid", pValue);
+    if (ok && pValue != NULL) {
+        return true;
+    }
+
     return false;
 }
 
@@ -239,8 +283,18 @@ bool ODe_Style_PageLayout::write(GsfOutput* pODT,
     ODe_writeAttribute(output, "fo:margin-bottom", m_marginBottom);
     ODe_writeAttribute(output, "fo:margin-left", m_marginLeft);
     ODe_writeAttribute(output, "fo:margin-right", m_marginRight);
-    
-    output += "/>\n";
+    ODe_writeAttribute(output, "fo:background-color", m_backgroundColor);
+
+    if(m_backgroundImage.length()) {
+        output += ">\n";
+        output += UT_UTF8String_sprintf("%s  <style:background-image ",rSpacesOffset.utf8_str());
+        output += "xlink:href=\"Pictures/";
+        output += m_backgroundImage;
+        output += ".png\" xlink:type=\"simple\" xlink:show=\"embed\" xlink:actuate=\"onLoad\" style:repeat=\"stretch\"/>\n";
+        output += UT_UTF8String_sprintf("%s </style:page-layout-properties>\n",rSpacesOffset.utf8_str());
+    } else {
+        output += "/>\n";
+    }
 
     ODe_writeUTF8String(pODT, output);
     
