@@ -85,7 +85,7 @@ bool ODe_Style_Style::write(GsfOutput* pODT, const UT_UTF8String& rSpacesOffset)
     output += "<style:style";
     
     if (m_name.empty()) {
-        UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+        UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
         return false;
     }
     escape = m_name;
@@ -94,7 +94,7 @@ bool ODe_Style_Style::write(GsfOutput* pODT, const UT_UTF8String& rSpacesOffset)
     output += "\"";
     
     if (m_family.empty()) {
-        UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+        UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
         return false;
     }
     output += " style:family=\"";
@@ -154,7 +154,7 @@ bool ODe_Style_Style::write(GsfOutput* pODT, const UT_UTF8String& rSpacesOffset)
  */
 bool ODe_Style_Style::hasTextStyleProps(const PP_AttrProp* pAP) {
     
-    const XML_Char* pValue;
+    const gchar* pValue;
     bool ok;
     
     ok = pAP->getProperty("color", pValue);
@@ -196,6 +196,11 @@ bool ODe_Style_Style::hasTextStyleProps(const PP_AttrProp* pAP) {
     if (ok && pValue != NULL) {
         return true;
     }
+
+    ok = pAP->getProperty("display", pValue);
+    if (ok && pValue != NULL) {
+        return true;
+    }
     
     return false;
 }
@@ -207,7 +212,7 @@ bool ODe_Style_Style::hasTextStyleProps(const PP_AttrProp* pAP) {
  */
 bool ODe_Style_Style::hasParagraphStyleProps(const PP_AttrProp* pAP) {
     
-    const XML_Char* pValue;
+    const gchar* pValue;
     bool ok;
     
     ok = pAP->getProperty("bgcolor", pValue);
@@ -224,6 +229,11 @@ bool ODe_Style_Style::hasParagraphStyleProps(const PP_AttrProp* pAP) {
     if (ok && pValue != NULL) {
         return true;
     }
+
+    ok = pAP->getProperty("text-indent", pValue);
+    if (ok && pValue != NULL) {
+        return true;
+    }
     
     ok = pAP->getProperty("widows", pValue);
     if (ok && pValue != NULL) {
@@ -275,7 +285,7 @@ bool ODe_Style_Style::hasParagraphStyleProps(const PP_AttrProp* pAP) {
  * 
  */
 bool ODe_Style_Style::hasSectionInfo(const PP_AttrProp* pAP) {
-    const XML_Char* pValue;
+    const gchar* pValue;
     bool ok;
     
     ok = pAP->getProperty("columns", pValue);
@@ -287,7 +297,35 @@ bool ODe_Style_Style::hasSectionInfo(const PP_AttrProp* pAP) {
     if (ok && pValue != NULL) {
         return true;
     }
+    ok= pAP->getProperty("page-margin-top",pValue);
+    if (ok && pValue != NULL) {
+        return true;
+    }
 
+    ok= pAP->getProperty("page-margin-left",pValue);
+    if (ok && pValue != NULL) {
+        return true;
+    }
+
+    ok= pAP->getProperty("page-margin-right",pValue);
+    if (ok && pValue != NULL) {
+        return true;
+    }
+
+    ok= pAP->getProperty("page-margin-bottom",pValue);
+    if (ok && pValue != NULL) {
+        return true;
+    }
+    
+    ok = pAP->getProperty("page-margin-header", pValue);
+    if (ok && pValue != NULL) {
+        return true;
+    }
+    
+    ok = pAP->getProperty("page-margin-footer", pValue);
+    if (ok && pValue != NULL) {
+        return true;
+    }
     return false;
 }
 
@@ -296,7 +334,7 @@ bool ODe_Style_Style::hasSectionInfo(const PP_AttrProp* pAP) {
  * 
  */
 bool ODe_Style_Style::hasTableCellStyleProps(const PP_AttrProp* pAP) {
-    const XML_Char* pValue;
+    const gchar* pValue;
     bool ok;
     
     // Left border
@@ -368,7 +406,7 @@ bool ODe_Style_Style::hasTableCellStyleProps(const PP_AttrProp* pAP) {
  * 
  */
 bool ODe_Style_Style::hasTableStyleProps(const PP_AttrProp* pAP) {
-    const XML_Char* pValue;
+    const gchar* pValue;
     bool ok;
     
     ok = pAP->getProperty("background-color", pValue);
@@ -494,7 +532,7 @@ ODe_Style_Style& ODe_Style_Style::operator=(const ODe_Style_Style& rStyle) {
  */
 bool ODe_Style_Style::fetchAttributesFromAbiStyle(const PP_AttrProp* pAP) {
 
-    const XML_Char* pValue;
+    const gchar* pValue;
     bool ok;
     
     ok = pAP->getAttribute(PT_NAME_ATTRIBUTE_NAME, pValue);
@@ -504,14 +542,14 @@ bool ODe_Style_Style::fetchAttributesFromAbiStyle(const PP_AttrProp* pAP) {
     ok = pAP->getAttribute(PT_FOLLOWEDBY_ATTRIBUTE_NAME, pValue);
     if (ok && pValue != NULL) {
         
-        if (UT_strcmp("Current Settings", pValue) != 0) {
+        if (strcmp("Current Settings", pValue) != 0) {
             m_nextStyleName = pValue;
         }
     }
     
     ok = pAP->getAttribute(PT_BASEDON_ATTRIBUTE_NAME, pValue);
     if (ok && pValue != NULL) {
-        if (UT_strcmp(pValue, "None") != 0) {
+        if (strcmp(pValue, "None") != 0) {
             // OpenDocument don't use a "None" to say that is has no parent.
             // To say that it simply doesn't define this attribute.
             m_parentStyleName = pValue;
@@ -537,7 +575,7 @@ bool ODe_Style_Style::fetchAttributesFromAbiStyle(const PP_AttrProp* pAP) {
  * Defines the style from attributes and properties of an AbiWord <span>.
  */
 void ODe_Style_Style::fetchAttributesFromAbiSpan(const PP_AttrProp* pAP) {
-    const XML_Char* pValue;
+    const gchar* pValue;
     bool ok;
     
     ok = pAP->getAttribute("style", pValue);
@@ -557,7 +595,7 @@ void ODe_Style_Style::fetchAttributesFromAbiSpan(const PP_AttrProp* pAP) {
  * Fetch attributes from an AbiWord <p> tag. Usually paragraph style attributes.
  */
 void ODe_Style_Style::fetchAttributesFromAbiBlock(const PP_AttrProp* pAP) {
-    const XML_Char* pValue;
+    const gchar* pValue;
     bool ok;
     
     ok = pAP->getAttribute("style", pValue);
@@ -632,7 +670,7 @@ void ODe_Style_Style::fetchAttributesFromAbiFrame(const PP_AttrProp& rAP) {
 /**
  * 
  */
-void ODe_Style_Style::setBreakBefore(const XML_Char* pBreakBefore) {
+void ODe_Style_Style::setBreakBefore(const gchar* pBreakBefore) {
     if (m_pParagraphProps == NULL) {
         m_pParagraphProps = new ParagraphProps();
     }
@@ -654,7 +692,7 @@ const UT_UTF8String& ODe_Style_Style::getFontName() {
 /**
  * 
  */
-void ODe_Style_Style::setColumnWidth(const XML_Char* pColumnWidth) {
+void ODe_Style_Style::setColumnWidth(const gchar* pColumnWidth) {
     if (m_pColumnProps == NULL) {
         m_pColumnProps = new ColumnProps();
     }
@@ -665,7 +703,7 @@ void ODe_Style_Style::setColumnWidth(const XML_Char* pColumnWidth) {
 /**
  * 
  */
-void ODe_Style_Style::setRowHeight(const XML_Char* pRowHeight) {
+void ODe_Style_Style::setRowHeight(const gchar* pRowHeight) {
     if (m_pRowProps == NULL) {
         m_pRowProps = new RowProps();
     }
@@ -763,7 +801,7 @@ bool ODe_Style_Style::SectionProps::isEmpty() const {
  */
 void ODe_Style_Style::SectionProps::
 fetchAttributesFromAbiProps(const PP_AttrProp& rAP) {
-    const XML_Char* pValue;
+    const gchar* pValue;
     bool ok;
     
     ok = rAP.getProperty("columns", pValue);
@@ -837,7 +875,9 @@ bool ODe_Style_Style::SectionProps::operator==(
  */
 bool ODe_Style_Style::ParagraphProps::isEmpty() const {
     return ( m_textAlign.empty() &&
+             m_textIndent.empty() &&
              m_lineHeight.empty() &&
+             m_lineHeightAtLeast.empty() &&
              m_backgroundColor.empty() &&
              m_widows.empty() &&
              m_orphans.empty() &&
@@ -846,7 +886,8 @@ bool ODe_Style_Style::ParagraphProps::isEmpty() const {
              m_marginTop.empty() &&
              m_marginBottom.empty() &&
              m_keepWithNext.empty() &&
-             m_breakBefore.empty());
+             m_breakBefore.empty() &&
+             m_writingMode.empty());
 }
 
 /**
@@ -854,29 +895,66 @@ bool ODe_Style_Style::ParagraphProps::isEmpty() const {
  */
 void ODe_Style_Style::ParagraphProps::
 fetchAttributesFromAbiProps(const PP_AttrProp& rAP) {
-    const XML_Char* pValue;
+    const gchar* pValue;
     bool ok;
     
     ok = rAP.getProperty("bgcolor", pValue);
     if (ok && pValue != NULL) {
-        
-        if (UT_XML_strlen(pValue)==6) {
+        int len = strlen(pValue);
+        if (len==6) {
             // It should be a RGB color.
             UT_UTF8String_sprintf(m_backgroundColor, "#%s", pValue);
-        } else {
-            UT_ASSERT(UT_strcmp("transparent", pValue)==0);
+        } else if((len==7) || (strcmp("transparent", pValue)==0)) {
             m_backgroundColor = pValue;
+        } else {
+            UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
         }
     }
     
     ok = rAP.getProperty("line-height", pValue);
     if (ok && pValue != NULL) {
-        m_lineHeight = pValue;
+        UT_LocaleTransactor t(LC_NUMERIC, "C");
+        if(strstr(pValue, "+")) { // "at least" spacing
+            int len = strlen(pValue);
+
+            if((len > 1) && (pValue[len - 1] == '+')) {
+               gchar* temp = (gchar*)pValue;
+               temp[len-1] = '\0';
+               m_lineHeightAtLeast = UT_UTF8String_sprintf("%fin", UT_convertToDimension(temp, DIM_IN));
+               m_lineHeight.clear(); // make sure this is empty
+            } else {
+                UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
+            }
+        } else if(strstr(pValue, "pt")) { // "exactly" spacing
+            m_lineHeight = UT_UTF8String_sprintf("%fin", UT_convertToDimension(pValue, DIM_IN));
+            m_lineHeightAtLeast.clear(); // make sure this is empty
+        } else { //"single", "double", "1.5 lines", or "multiple" spacing
+            m_lineHeight = UT_UTF8String_sprintf("%.0f%%", atof(pValue) * 100);
+            m_lineHeightAtLeast.clear(); // make sure this is empty
+        }
     }
     
     ok = rAP.getProperty("text-align", pValue);
     if (ok && pValue != NULL) {
-        m_textAlign = pValue;
+        if(!strcmp(pValue, "right")) {
+            m_textAlign = "end"; //see Bug 10719
+        } else {
+            m_textAlign = pValue;
+        }
+    }
+
+    ok = rAP.getProperty("text-indent", pValue);
+    if (ok && pValue != NULL) {
+        m_textIndent = pValue;
+    }
+
+    ok = rAP.getProperty("dom-dir", pValue);
+    if (ok && pValue != NULL) {
+        if(!strcmp(pValue, "rtl")) {
+            m_writingMode = "rl";
+        } else {
+            m_writingMode = "lr";
+        }
     }
     
     ok = rAP.getProperty("widows", pValue);
@@ -911,7 +989,7 @@ fetchAttributesFromAbiProps(const PP_AttrProp& rAP) {
     
     ok = rAP.getProperty("keep-with-next", pValue);
     if (ok && pValue != NULL) {
-        if (!UT_strcmp(pValue, "yes")) {
+        if (!strcmp(pValue, "yes")) {
             m_keepWithNext = "always";
         } else {
             m_keepWithNext = "auto";
@@ -934,9 +1012,9 @@ write(UT_UTF8String& rOutput, const UT_UTF8String& rSpacesOffset) const {
     rOutput += "<style:paragraph-properties";
     
     ODe_writeAttribute(rOutput, "fo:text-align", m_textAlign);
-    // Bugged, must realize how to translate it correctly form AbiWord to
-    // OpenDocument
-    //ODe_writeAttribute(rOutput, "fo:line-height", m_lineHeight);
+    ODe_writeAttribute(rOutput, "fo:text-indent", m_textIndent);
+    ODe_writeAttribute(rOutput, "fo:line-height", m_lineHeight);
+    ODe_writeAttribute(rOutput, "style:line-height-at-least", m_lineHeightAtLeast);
     ODe_writeAttribute(rOutput, "fo:background-color", m_backgroundColor);
     ODe_writeAttribute(rOutput, "fo:widows", m_widows);
     ODe_writeAttribute(rOutput, "fo:orphans", m_orphans);
@@ -946,6 +1024,7 @@ write(UT_UTF8String& rOutput, const UT_UTF8String& rSpacesOffset) const {
     ODe_writeAttribute(rOutput, "fo:margin-bottom", m_marginBottom);
     ODe_writeAttribute(rOutput, "fo:keep-with-next", m_keepWithNext);
     ODe_writeAttribute(rOutput, "fo:break-before", m_breakBefore);
+    ODe_writeAttribute(rOutput, "style:writing-mode", m_writingMode);
 
     rOutput += "/>\n";
 }
@@ -958,7 +1037,9 @@ ODe_Style_Style::ParagraphProps& ODe_Style_Style::ParagraphProps::operator=(
                                        const ParagraphProps& rParagraphProps) {
                                         
     m_textAlign = rParagraphProps.m_textAlign;
+    m_textIndent = rParagraphProps.m_textIndent;
     m_lineHeight = rParagraphProps.m_lineHeight;
+    m_lineHeightAtLeast = rParagraphProps.m_lineHeightAtLeast;
     m_backgroundColor = rParagraphProps.m_backgroundColor;
     m_widows = rParagraphProps.m_widows;
     m_orphans = rParagraphProps.m_orphans;
@@ -968,6 +1049,7 @@ ODe_Style_Style::ParagraphProps& ODe_Style_Style::ParagraphProps::operator=(
     m_marginBottom = rParagraphProps.m_marginBottom;
     m_keepWithNext = rParagraphProps.m_keepWithNext;
     m_breakBefore = rParagraphProps.m_breakBefore;
+    m_writingMode = rParagraphProps.m_writingMode;
     
     return *this;
 }
@@ -981,7 +1063,9 @@ bool ODe_Style_Style::ParagraphProps::operator==(
 
     return                    
         m_textAlign       == rParagraphProps.m_textAlign &&
+        m_textIndent       == rParagraphProps.m_textIndent &&
         m_lineHeight      == rParagraphProps.m_lineHeight &&
+        m_lineHeightAtLeast == rParagraphProps.m_lineHeightAtLeast &&
         m_backgroundColor == rParagraphProps.m_backgroundColor &&
         m_widows          == rParagraphProps.m_widows &&
         m_orphans         == rParagraphProps.m_orphans &&
@@ -990,7 +1074,8 @@ bool ODe_Style_Style::ParagraphProps::operator==(
         m_marginTop       == rParagraphProps.m_marginTop &&
         m_marginBottom    == rParagraphProps.m_marginBottom &&
         m_keepWithNext    == rParagraphProps.m_keepWithNext &&
-        m_breakBefore     == rParagraphProps.m_breakBefore;
+        m_breakBefore     == rParagraphProps.m_breakBefore &&
+        m_writingMode     == rParagraphProps.m_writingMode;
 }
 
 
@@ -1012,7 +1097,9 @@ bool ODe_Style_Style::TextProps::isEmpty() const {
            m_language.empty() &&
            m_country.empty() &&
            m_fontStyle.empty() &&
-           m_fontWeight.empty();
+           m_fontWeight.empty() &&
+           m_backgroundColor.empty() &&
+           m_display.empty();
 }
 
 
@@ -1021,72 +1108,41 @@ bool ODe_Style_Style::TextProps::isEmpty() const {
  */
 void ODe_Style_Style::TextProps::
 fetchAttributesFromAbiProps(const PP_AttrProp& rAP) {
-    const XML_Char* pValue;
+    const gchar* pValue;
     bool ok;
     
     ok = rAP.getProperty("color", pValue);
     if (ok && pValue != NULL) {
-        if (UT_XML_strlen(pValue)==6) {
+        int len = strlen(pValue);
+        if (len == 6) {
             UT_UTF8String_sprintf(m_color, "#%s", pValue);
-        } else {
-            UT_ASSERT(UT_XML_strlen(pValue)==7);
-            UT_ASSERT(pValue[0]=='#');
+        } else if ((len == 7) && (pValue[0] == '#')){
             m_color = pValue;
+        } else {
+            UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
         }
     }
     
     ok = rAP.getProperty("text-decoration", pValue);
     if (ok && pValue != NULL) {
-        XML_Char buffer[500];
-        UT_uint32 i,j;
-        bool keepGoing = true;
-        bool finishedWord = false;
-        
-        buffer[0] = 0;
-        i = 0;
-        
-        j = 0;        
-        while (keepGoing) {
-            buffer[i] = pValue[j];
-            
-            if (pValue[j] == 0) {
-                keepGoing = false;
-                finishedWord = true;
-            } else if (isspace(pValue[j])) {
-                finishedWord = true;
-                buffer[i] = 0; // Make a null terminated string
-            }
-            
-            if (finishedWord) {
-                if (!UT_strcmp(buffer, "underline")) {
-                    m_underlineType = "single";
-                    
-                } else if (!UT_strcmp(buffer, "line-through")) {
-                    m_lineThroughType = "single";
-                }
-                finishedWord = false;
-                i = 0;
-                buffer[0] = 0;
-            } else {
-                i++;
-            }
-            
-            j++;
-        }
-        
-    } // if (ok)
+        if (strstr(pValue, "underline"))
+            m_underlineType = "single";
+
+        if (strstr(pValue, "line-through"))
+            m_lineThroughType = "single";
+    }
     
     
     ok = rAP.getProperty("text-position", pValue);
     if (ok && pValue != NULL) {
-        if (!UT_strcmp("subscript", pValue)) {
+        if (!strcmp("subscript", pValue)) {
             // Hard coded, it's ugly but I can't do otherwise.
             m_textPosition = "-33%";
-        } else if (!UT_strcmp("superscript", pValue)) {
+        } else if (!strcmp("superscript", pValue)) {
             // Hard coded, it's ugly but I can't do otherwise.
             m_textPosition = "33%";
         } else {
-            UT_ASSERT( !UT_strcmp("normal", pValue) );
+            UT_ASSERT( !strcmp("normal", pValue) );
             m_textPosition.clear();
         }
     }
@@ -1106,29 +1162,39 @@ fetchAttributesFromAbiProps(const PP_AttrProp& rAP) {
     
     ok = rAP.getProperty("lang", pValue);
     if (ok && pValue != NULL) {
-        if (!UT_strcmp(pValue, "-none-")) {
+        if (!strcmp(pValue, "-none-")) {
             m_language = "none";
             m_country = "none";
         } else {
-            XML_Char strLanguage[3];
-            XML_Char strCountry[3];
+            gchar strLanguage[4];
+            gchar strCountry[3];
 
-            // All valid strings should have 5 chars. e.g.: "en-US"
-            UT_ASSERT( UT_XML_strlen(pValue) == 5); 
-            
-            if (UT_XML_strlen(pValue) == 5) {
+            int len = strlen(pValue);
+            bool bLong = (len == 6);
+            // Note: not all language codes are 5 characters (e.g. cop-EG)
+            if ((len == 5) || bLong) {
                 strLanguage[0] = pValue[0];
                 strLanguage[1] = pValue[1];
-                strLanguage[2] = 0;
+
+                if(bLong) {
+                    strLanguage[2] = pValue[2];
+                    // pValue[3] == '-'
+                    strCountry[0] = pValue[4];
+                    strCountry[1] = pValue[5];
+                } else {
+                    strLanguage[2] = 0;
+                    // pValue[2] == '-'
+                    strCountry[0] = pValue[3];
+                    strCountry[1] = pValue[4];
+                }
                 
-                // pValue[2] == '-'
-                
-                strCountry[0] = pValue[3];
-                strCountry[1] = pValue[4];
+                strLanguage[3] = 0;
                 strCountry[2] = 0;
                
                 m_language = strLanguage;
                 m_country = strCountry;
+            } else {
+                UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
             }
         }
     }
@@ -1136,7 +1202,7 @@ fetchAttributesFromAbiProps(const PP_AttrProp& rAP) {
     
     ok = rAP.getProperty("font-style", pValue);
     if (ok && pValue != NULL) {
-        if (!UT_strcmp(pValue, "italic")) {
+        if (!strcmp(pValue, "italic")) {
             m_fontStyle = "italic";
         }
     }
@@ -1144,8 +1210,34 @@ fetchAttributesFromAbiProps(const PP_AttrProp& rAP) {
     
     ok = rAP.getProperty("font-weight", pValue);
     if (ok && pValue != NULL) {
-        if (!UT_strcmp(pValue, "bold")) {
+        if (!strcmp(pValue, "bold")) {
             m_fontWeight = "bold";
+        } else if (!strcmp(pValue, "normal")) {
+            m_fontWeight = "normal";
+        }
+    }
+
+
+    ok = rAP.getProperty("bgcolor", pValue);
+    if (ok && pValue != NULL) {
+        int len = strlen(pValue);
+        if (len==6) {
+            // It should be an RGB color.
+            UT_UTF8String_sprintf(m_backgroundColor, "#%s", pValue);
+        } else if ((len == 7) || (strcmp("transparent", pValue) == 0)) {
+            m_backgroundColor = pValue;
+        } else {
+            UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
+        }
+    }
+
+
+    ok = rAP.getProperty("display", pValue);
+    if (ok && pValue != NULL) {
+        if (!strcmp(pValue, "none")) {
+            m_display = "none";
+        } else {
+            m_display = "true";
         }
     }
 }
@@ -1174,6 +1266,8 @@ write(UT_UTF8String& rOutput, const UT_UTF8String& rSpacesOffset) const {
     ODe_writeAttribute(rOutput, "fo:country", m_country);
     ODe_writeAttribute(rOutput, "fo:font-style", m_fontStyle);
     ODe_writeAttribute(rOutput, "fo:font-weight", m_fontWeight);
+    ODe_writeAttribute(rOutput, "fo:background-color", m_backgroundColor);
+    ODe_writeAttribute(rOutput, "text:display", m_display);
     
     rOutput += "/>\n";
 }
@@ -1195,6 +1289,8 @@ ODe_Style_Style::TextProps& ODe_Style_Style::TextProps::operator=(
     m_country = rTextProps.m_country;
     m_fontStyle = rTextProps.m_fontStyle;
     m_fontWeight = rTextProps.m_fontWeight;
+    m_backgroundColor = rTextProps.m_backgroundColor;
+    m_display = rTextProps.m_display;
     
     return *this;
 }
@@ -1216,7 +1312,9 @@ bool ODe_Style_Style::TextProps::operator==(
         m_language        == rTextProps.m_language &&
         m_country         == rTextProps.m_country &&
         m_fontStyle       == rTextProps.m_fontStyle &&
-        m_fontWeight      == rTextProps.m_fontWeight;
+        m_fontWeight      == rTextProps.m_fontWeight &&
+        m_backgroundColor == rTextProps.m_backgroundColor &&
+        m_display         == rTextProps.m_display;
 }
 
 
@@ -1242,49 +1340,48 @@ bool ODe_Style_Style::TableProps::isEmpty() const {
  */
 void ODe_Style_Style::TableProps::
 fetchAttributesFromAbiProps(const PP_AttrProp& rAP) {
-    const XML_Char* pValue;
+    const gchar* pValue;
     bool ok;
     
     ok = rAP.getProperty("background-color", pValue);
     if (ok && pValue != NULL) {
-        UT_UTF8String_sprintf(m_backgroundColor, "#%s", pValue);
+        int len = strlen(pValue);
+        if(len == 6) {
+            UT_UTF8String_sprintf(m_backgroundColor, "#%s", pValue);
+        } else if (len == 7) {
+            m_backgroundColor = pValue;
+        } else {
+            UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
+        }
     }
     
     ok = rAP.getProperty("table-column-props", pValue);
     if (ok && pValue != NULL) {
-        XML_Char buffer[100];
-        UT_uint32 i;
+        std::string buffer;
         double tableWidth = 0.0;
-        UT_Dimension dimension;
+        UT_Dimension dimension = DIM_none;
         bool gotDimension = false;
         
         // The table width is the sum of all column widths.
         
-        buffer[0] = 0;
-        i = 0;
         while (*pValue != 0) {
             if (*pValue == '/') {
                 // We've reached the end of a column width
-                
-                buffer[i] = 0; // NULL terminate the string
-                
                 if (!gotDimension) {
-                    dimension = UT_determineDimension(buffer, DIM_none);
+                    dimension = UT_determineDimension(buffer.c_str(), DIM_none);
                     gotDimension = true;
                 }
                 
-                tableWidth += UT_convertDimensionless(buffer);
-                
-                i = 0; // Restart the buffer
+                tableWidth += UT_convertDimensionless(buffer.c_str());
+                buffer.clear();
             } else {
                 // Store the character in the buffer
-                buffer[i] = *pValue;
-                i++;
+                buffer += *pValue;
             }
             pValue++;
         }
         
-	UT_LocaleTransactor t(LC_NUMERIC, "C");
+        UT_LocaleTransactor t(LC_NUMERIC, "C");
         UT_UTF8String_sprintf(m_width, "%f%s",
                               tableWidth, UT_dimensionName(dimension) );
     }
@@ -1473,7 +1570,7 @@ bool ODe_Style_Style::CellProps::isEmpty() const {
  */
 void ODe_Style_Style::CellProps::
 fetchAttributesFromAbiProps(const PP_AttrProp& rAP) {
-    const XML_Char* pValue;
+    const gchar* pValue;
     bool ok;
     
     // Left border
@@ -1549,7 +1646,14 @@ fetchAttributesFromAbiProps(const PP_AttrProp& rAP) {
     
     ok = rAP.getProperty("background-color", pValue);
     if (ok && pValue != NULL) {
-        UT_UTF8String_sprintf(m_backgroundColor, "#%s", pValue);
+        int len = strlen(pValue);
+        if(len == 6) {
+            UT_UTF8String_sprintf(m_backgroundColor, "#%s", pValue);
+        } else if (len == 7) {
+            m_backgroundColor = pValue;
+        } else {
+            UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
+        }
     }
 }
 
@@ -1637,7 +1741,7 @@ bool ODe_Style_Style::GraphicProps::isEmpty() const {
  */
 void ODe_Style_Style::GraphicProps::
 fetchAttributesFromAbiProps(const PP_AttrProp& rAP) {
-    const XML_Char* pValue = NULL;
+    const gchar* pValue = NULL;
     bool ok;
     
     // Left border
@@ -1742,7 +1846,14 @@ fetchAttributesFromAbiProps(const PP_AttrProp& rAP) {
     
     ok = rAP.getProperty("background-color", pValue);
     if (ok && pValue != NULL) {
-        UT_UTF8String_sprintf(m_backgroundColor, "#%s", pValue);
+        int len = strlen(pValue);
+        if(len == 6) {
+            UT_UTF8String_sprintf(m_backgroundColor, "#%s", pValue);
+        } else if (len == 7) {
+            m_backgroundColor = pValue;
+        } else {
+            UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
+        }
     }
     
     
@@ -1750,10 +1861,10 @@ fetchAttributesFromAbiProps(const PP_AttrProp& rAP) {
     
     ok = rAP.getProperty("wrap-mode", pValue);
     if (ok && pValue != NULL) {
-        if (!UT_strcmp(pValue, "above-text")) {
+        if (!strcmp(pValue, "above-text")) {
             m_wrap = "run-through";
             m_runThrough = "foreground";
-        } else if (!UT_strcmp(pValue, "wrapped-both")) {
+        } else if (!strcmp(pValue, "wrapped-both")) {
             m_wrap = "parallel";
         } else {
             UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
@@ -1764,7 +1875,7 @@ fetchAttributesFromAbiProps(const PP_AttrProp& rAP) {
     ok = rAP.getProperty("position-to", pValue);
     UT_ASSERT (ok && pValue != NULL);
 
-    if (!UT_strcmp(pValue, "block-above-text")) {
+    if (!strcmp(pValue, "block-above-text")) {
         m_horizontalRel = "paragraph";
         m_verticalRel = "paragraph";
     } else {
