@@ -45,13 +45,13 @@ ODi_ContentStream_ListenerState::ODi_ContentStream_ListenerState (
                 ODi_ElementStack& rElementStack)
                 : ODi_ListenerState("ContentStream", rElementStack),
                   m_pAbiDocument (pDocument),
-                  m_rFontFaceDecls(rFontFaceDecls),
                   m_pGsfInfile(pGsfInfile),
-                  m_pStyles(pStyles)
+                  m_pStyles(pStyles),
+                  m_rFontFaceDecls(rFontFaceDecls)
 {
-    UT_ASSERT(m_pAbiDocument);
-    UT_ASSERT(m_pStyles);
-    UT_ASSERT(m_pGsfInfile);
+    UT_ASSERT_HARMLESS(m_pAbiDocument);
+    UT_ASSERT_HARMLESS(m_pStyles);
+    UT_ASSERT_HARMLESS(m_pGsfInfile);
 }
 
 
@@ -69,21 +69,21 @@ ODi_ContentStream_ListenerState::~ODi_ContentStream_ListenerState()
  * @param pName The name of the element.
  * @param ppAtts The attributes of the parsed start tag.
  */
-void ODi_ContentStream_ListenerState::startElement (const XML_Char* pName,
-                                               const XML_Char** ppAtts,
+void ODi_ContentStream_ListenerState::startElement (const gchar* pName,
+                                               const gchar** ppAtts,
                                                ODi_ListenerStateAction& rAction)
 {
-    if (!UT_strcmp(pName, "office:font-face-decls")) {
+    if (!strcmp(pName, "office:font-face-decls")) {
 
         rAction.pushState("FontFaceDecls");
 
-    } else if (!UT_strcmp(pName, "office:body")) {
+    } else if (!strcmp(pName, "office:body")) {
 
         // Here comes the document itself. So, at this point, all styles have
         // already been defined.
         m_pStyles->addedAllStyles(m_pAbiDocument,  m_rFontFaceDecls);
 
-    } else if (!UT_strcmp(pName, "style:style")) {
+    } else if (!strcmp(pName, "style:style")) {
         ODi_ListenerState* pStyle = NULL;
         
         pStyle = m_pStyles->addStyle(ppAtts, m_rElementStack);
@@ -100,7 +100,7 @@ void ODi_ContentStream_ListenerState::startElement (const XML_Char* pName,
         pStyle = m_pStyles->addList(ppAtts, m_rElementStack);
         rAction.pushState(pStyle, false);
         
-    } else if (!UT_strcmp(pName, "office:text")) {
+    } else if (!strcmp(pName, "office:text")) {
         rAction.pushState("TextContent");
     }
 }
@@ -109,10 +109,10 @@ void ODi_ContentStream_ListenerState::startElement (const XML_Char* pName,
 /**
  *
  */
-void ODi_ContentStream_ListenerState::endElement (const XML_Char* pName,
+void ODi_ContentStream_ListenerState::endElement (const gchar* pName,
                                                 ODi_ListenerStateAction& rAction)
 {
-    if (!UT_strcmp(pName, "office:document-content")) {
+    if (!strcmp(pName, "office:document-content")) {
         // We're done.
         rAction.popState();
     }
@@ -122,7 +122,7 @@ void ODi_ContentStream_ListenerState::endElement (const XML_Char* pName,
 /**
  * 
  */
-void ODi_ContentStream_ListenerState::charData (
-                            const XML_Char* pBuffer, int length)
+void ODi_ContentStream_ListenerState::charData (const gchar* /*pBuffer*/, 
+												int /*length*/)
 {
 }
