@@ -91,12 +91,12 @@ ODi_Office_Styles::~ODi_Office_Styles() {
  * @return The address of the newly created ODi_Style_Style or NULL, if the
  *         specified style is not currently supported (like graphic styles).
  */
-ODi_Style_Style* ODi_Office_Styles::addStyle(const XML_Char** ppAtts,
+ODi_Style_Style* ODi_Office_Styles::addStyle(const gchar** ppAtts,
                                            ODi_ElementStack& rElementStack) {
 
-    const XML_Char* pFamily;
-    const XML_Char* pName;
-    const XML_Char* pDisplayName;
+    const gchar* pFamily;
+    const gchar* pName;
+    const gchar* pDisplayName;
     ODi_Style_Style* pStyle = NULL;
     const ODi_Style_Style* pConstStyle;
     UT_UTF8String replacementName;
@@ -109,7 +109,7 @@ ODi_Style_Style* ODi_Office_Styles::addStyle(const XML_Char** ppAtts,
 
     UT_return_val_if_fail(pFamily, pStyle);
     
-    if(!UT_strcmp(pFamily, "text")) {
+    if(!strcmp(pFamily, "text")) {
         // AbiWord doesn't support two styles with the same name, differing only
         // on its type (eg: A "Example" character style and a "Example"
         // paragraph style).
@@ -132,7 +132,7 @@ ODi_Style_Style* ODi_Office_Styles::addStyle(const XML_Char** ppAtts,
             pStyle = m_textStyleStyles.addStyle(ppAtts, rElementStack);
         }
         
-    } else if(!UT_strcmp(pFamily, "paragraph")) {
+    } else if(!strcmp(pFamily, "paragraph")) {
         // AbiWord doesn't support two styles with the same name, differing only
         // on its type (eg: A "Example" character style and a "Example"
         // paragraph style).
@@ -155,22 +155,22 @@ ODi_Style_Style* ODi_Office_Styles::addStyle(const XML_Char** ppAtts,
             pStyle = m_paragraphStyleStyles.addStyle(ppAtts, rElementStack);
         }
         
-    } else if(!UT_strcmp(pFamily, "section")) {
+    } else if(!strcmp(pFamily, "section")) {
         pStyle = m_sectionStyleStyles.addStyle(ppAtts, rElementStack);
         
-    } else if(!UT_strcmp(pFamily, "graphic")) {
+    } else if(!strcmp(pFamily, "graphic")) {
         pStyle = m_graphicStyleStyles.addStyle(ppAtts, rElementStack);
         
-    } else if(!UT_strcmp(pFamily, "table")) {
+    } else if(!strcmp(pFamily, "table")) {
         pStyle = m_tableStyleStyles.addStyle(ppAtts, rElementStack);
         
-    } else if(!UT_strcmp(pFamily, "table-column")) {
+    } else if(!strcmp(pFamily, "table-column")) {
         pStyle = m_tableColumnStyleStyles.addStyle(ppAtts, rElementStack);
         
-    } else if(!UT_strcmp(pFamily, "table-row")) {
+    } else if(!strcmp(pFamily, "table-row")) {
         pStyle = m_tableRowStyleStyles.addStyle(ppAtts, rElementStack);
         
-    } else if(!UT_strcmp(pFamily, "table-cell")) {
+    } else if(!strcmp(pFamily, "table-cell")) {
         pStyle = m_tableCellStyleStyles.addStyle(ppAtts, rElementStack);
     }
     
@@ -183,14 +183,15 @@ ODi_Style_Style* ODi_Office_Styles::addStyle(const XML_Char** ppAtts,
  * 
  * @return The address of the newly created ODi_Style_PageLayout.
  */
-ODi_Style_PageLayout* ODi_Office_Styles::addPageLayout(const XML_Char** ppAtts,
-                                                     ODi_ElementStack& rElementStack) {
+ODi_Style_PageLayout* ODi_Office_Styles::addPageLayout(const gchar** ppAtts,
+                                                     ODi_ElementStack& rElementStack,
+                                                     ODi_Abi_Data& rAbiData) {
                                
-    const XML_Char* pAttrValue;
+    const gchar* pAttrValue;
     ODi_Style_PageLayout* pStyle;
     bool ok;
                              
-    pStyle = new ODi_Style_PageLayout(rElementStack);
+    pStyle = new ODi_Style_PageLayout(rElementStack, rAbiData);
     pAttrValue = UT_getAttribute("style:name", ppAtts);
     ok = m_pageLayoutStyles.insert(pAttrValue, pStyle);
     
@@ -205,11 +206,11 @@ ODi_Style_PageLayout* ODi_Office_Styles::addPageLayout(const XML_Char** ppAtts,
  * 
  * @return The address of the newly created ODi_Style_MasterPage.
  */
-ODi_Style_MasterPage* ODi_Office_Styles::addMasterPage(const XML_Char** ppAtts,
+ODi_Style_MasterPage* ODi_Office_Styles::addMasterPage(const gchar** ppAtts,
                                                      PD_Document* pDocument,
                                                      ODi_ElementStack& rElementStack) {
                                                         
-    const XML_Char* pAttrValue;
+    const gchar* pAttrValue;
     ODi_Style_MasterPage* pStyle;
     bool ok;
                              
@@ -226,19 +227,18 @@ ODi_Style_MasterPage* ODi_Office_Styles::addMasterPage(const XML_Char** ppAtts,
 /**
  * 
  */
-ODi_Style_Style* ODi_Office_Styles::addDefaultStyle(const XML_Char** ppAtts,
+ODi_Style_Style* ODi_Office_Styles::addDefaultStyle(const gchar** ppAtts,
                                                   ODi_ElementStack& rElementStack) {
     
-    const XML_Char* pAttr;
+    const gchar* pAttr = NULL;
     
     pAttr = UT_getAttribute("style:family", ppAtts);
-    UT_ASSERT(pAttr);
 
-    if (!UT_strcmp("paragraph", pAttr)) {
+    if (pAttr && !strcmp("paragraph", pAttr)) {
         
         return m_paragraphStyleStyles.addDefaultStyle(rElementStack);
         
-    } else if (!UT_strcmp("table", pAttr)) {
+    } else if (pAttr && !strcmp("table", pAttr)) {
         
         return m_tableStyleStyles.addDefaultStyle(rElementStack);        
 
@@ -279,7 +279,7 @@ void ODi_Office_Styles::_linkMasterStyles() {
     ODi_Style_PageLayout* pLayout;
     
     pMasterStylesVec = m_masterPageStyles.enumerate();
-    UT_ASSERT(pMasterStylesVec);
+    UT_return_if_fail(pMasterStylesVec);
     
     count = pMasterStylesVec->getItemCount();
     for (i=0; i<count; i++) {
@@ -306,13 +306,13 @@ void ODi_Office_Styles::_linkListStyles() {
     const ODi_Style_Style* pStyle;
     
     pStylesVec = m_listStyles.enumerate();
-    UT_ASSERT(pStylesVec);
+    UT_return_if_fail(pStylesVec);
     
     count = pStylesVec->getItemCount();
     for (i=0; i<count; i++) {
 
         pListStyle = (*pStylesVec)[i];
-        UT_ASSERT(pListStyle);
+        UT_continue_if_fail(pListStyle);
         
         count2 = pListStyle->getLevelCount();
 
@@ -348,7 +348,7 @@ void ODi_Office_Styles::_defineAbiStyles(PD_Document* pDocument) const {
 
 
     pListVec = m_listStyles.enumerate();
-    UT_ASSERT(pListVec);
+    UT_return_if_fail(pListVec);
     
     count = pListVec->getItemCount();
     for (i=0; i<count; i++) {
@@ -395,7 +395,7 @@ void ODi_Office_Styles::_buildAbiPropsAttrString(
     
 
     pListVec = m_listStyles.enumerate();
-    UT_ASSERT(pListVec);
+    UT_return_if_fail(pListVec);
     
     count = pListVec->getItemCount();
     for (i=0; i<count; i++) {
@@ -420,7 +420,7 @@ void ODi_Office_Styles::_fixStyles() {
  * @param pStyleName The name of the style wanted.
  */
 const ODi_Style_Style* ODi_Office_Styles::getParagraphStyle(
-                                             const XML_Char* pStyleName,
+                                             const gchar* pStyleName,
                                              bool bOnContentStream) {
                                                 
     return m_paragraphStyleStyles.getStyle(pStyleName, bOnContentStream);
@@ -430,7 +430,7 @@ const ODi_Style_Style* ODi_Office_Styles::getParagraphStyle(
 /**
  * 
  */
-const ODi_Style_Style* ODi_Office_Styles::getTextStyle(const XML_Char* pStyleName,
+const ODi_Style_Style* ODi_Office_Styles::getTextStyle(const gchar* pStyleName,
                                               bool bOnContentStream) {
     return m_textStyleStyles.getStyle(pStyleName, bOnContentStream);
 }
@@ -439,7 +439,7 @@ const ODi_Style_Style* ODi_Office_Styles::getTextStyle(const XML_Char* pStyleNam
 /**
  * 
  */ 
-const ODi_Style_Style* ODi_Office_Styles::getSectionStyle(const XML_Char* pStyleName,
+const ODi_Style_Style* ODi_Office_Styles::getSectionStyle(const gchar* pStyleName,
                                                  bool bOnContentStream) {
     return m_sectionStyleStyles.getStyle(pStyleName, bOnContentStream);
 }
@@ -448,7 +448,7 @@ const ODi_Style_Style* ODi_Office_Styles::getSectionStyle(const XML_Char* pStyle
 /**
  * 
  */
-const ODi_Style_Style* ODi_Office_Styles::getGraphicStyle(const XML_Char* pStyleName,
+const ODi_Style_Style* ODi_Office_Styles::getGraphicStyle(const gchar* pStyleName,
                                           bool bOnContentStream) {
     return m_graphicStyleStyles.getStyle(pStyleName, bOnContentStream);
 }
@@ -457,7 +457,7 @@ const ODi_Style_Style* ODi_Office_Styles::getGraphicStyle(const XML_Char* pStyle
 /**
  * 
  */
-const ODi_Style_Style* ODi_Office_Styles::getTableStyle(const XML_Char* pStyleName,
+const ODi_Style_Style* ODi_Office_Styles::getTableStyle(const gchar* pStyleName,
                                                   bool bOnContentStream) {
                                                     
     return m_tableStyleStyles.getStyle(pStyleName, bOnContentStream);
@@ -468,7 +468,7 @@ const ODi_Style_Style* ODi_Office_Styles::getTableStyle(const XML_Char* pStyleNa
  * 
  */
 const ODi_Style_Style* ODi_Office_Styles::getTableColumnStyle(
-                                                  const XML_Char* pStyleName,
+                                                  const gchar* pStyleName,
                                                   bool bOnContentStream) {
                                                     
     return m_tableColumnStyleStyles.getStyle(pStyleName, bOnContentStream);
@@ -479,7 +479,7 @@ const ODi_Style_Style* ODi_Office_Styles::getTableColumnStyle(
  * 
  */
 const ODi_Style_Style* ODi_Office_Styles::getTableRowStyle(
-                                                  const XML_Char* pStyleName,
+                                                  const gchar* pStyleName,
                                                   bool bOnContentStream) {
                                                     
     return m_tableRowStyleStyles.getStyle(pStyleName, bOnContentStream);
@@ -490,7 +490,7 @@ const ODi_Style_Style* ODi_Office_Styles::getTableRowStyle(
  * 
  */
 const ODi_Style_Style* ODi_Office_Styles::getTableCellStyle(
-                                                  const XML_Char* pStyleName,
+                                                  const gchar* pStyleName,
                                                   bool bOnContentStream) {
                                                     
     return m_tableCellStyleStyles.getStyle(pStyleName, bOnContentStream);
@@ -500,10 +500,10 @@ const ODi_Style_Style* ODi_Office_Styles::getTableCellStyle(
 /**
  * Adds a list style (<text:list-style>)
  */
-ODi_Style_List* ODi_Office_Styles::addList(const XML_Char** ppAtts,
+ODi_Style_List* ODi_Office_Styles::addList(const gchar** ppAtts,
                                          ODi_ElementStack& rElementStack) {
     bool ok;
-    const XML_Char* pAttrValue = NULL;
+    const gchar* pAttrValue = NULL;
     ODi_Style_List* pStyle = NULL;
 
     pStyle = new ODi_Style_List(rElementStack);
@@ -521,10 +521,10 @@ ODi_Style_List* ODi_Office_Styles::addList(const XML_Char** ppAtts,
  * Adds a notes configuration (<text:notes-configuration>)
  */
 ODi_NotesConfiguration* ODi_Office_Styles::addNotesConfiguration(
-                                               const XML_Char** ppAtts,
+                                               const gchar** ppAtts,
                                                ODi_ElementStack& rElementStack) {
     bool ok;
-    const XML_Char* pAttrValue = NULL;
+    const gchar* pAttrValue = NULL;
     ODi_NotesConfiguration* pNotesConfig = NULL;
 
     pNotesConfig = new ODi_NotesConfiguration(rElementStack);
