@@ -62,49 +62,50 @@ ODi_Style_Style::ODi_Style_Style(ODi_ElementStack& rElementStack) :
 /**
  * 
  */
-void ODi_Style_Style::startElement(const XML_Char* pName,
-                                  const XML_Char** ppAtts,
-                                  ODi_ListenerStateAction& rAction) {
+void ODi_Style_Style::startElement(const gchar* pName,
+                                  const gchar** ppAtts,
+								   ODi_ListenerStateAction& /*rAction*/) 
+{
 
-    if (!UT_strcmp("style:style", pName)) {
+    if (!strcmp("style:style", pName)) {
         
         _parse_style_style(ppAtts);
         
-    } else if (!UT_strcmp("style:paragraph-properties", pName)) {
+    } else if (!strcmp("style:paragraph-properties", pName)) {
         
         _parse_style_paragraphProperties(ppAtts);
         
-    } else if (!UT_strcmp("style:text-properties", pName)) {
+    } else if (!strcmp("style:text-properties", pName)) {
         
         _parse_style_textProperties(ppAtts);
         
-    } else if (!UT_strcmp("style:section-properties", pName)) {
+    } else if (!strcmp("style:section-properties", pName)) {
         
         _parse_style_sectionProperties(ppAtts);
         
-    } else if (!UT_strcmp("style:graphic-properties", pName)) {
+    } else if (!strcmp("style:graphic-properties", pName)) {
         
         _parse_style_graphicProperties(ppAtts);
         
-    } else if (!UT_strcmp("style:table-properties", pName)) {
+    } else if (!strcmp("style:table-properties", pName)) {
         
         _parse_style_tableProperties(ppAtts);
         
-    } else if (!UT_strcmp("style:table-column-properties", pName)) {
+    } else if (!strcmp("style:table-column-properties", pName)) {
         
         _parse_style_tableColumnProperties(ppAtts);
         
-    } else if (!UT_strcmp("style:table-row-properties", pName)) {
+    } else if (!strcmp("style:table-row-properties", pName)) {
         
         _parse_style_tableRowProperties(ppAtts);
         
-    } else if (!UT_strcmp("style:table-cell-properties", pName)) {
+    } else if (!strcmp("style:table-cell-properties", pName)) {
         
         _parse_style_tableCellProperties(ppAtts);
         
-    } else if (!UT_strcmp("style:default-style", pName)) {
+    } else if (!strcmp("style:default-style", pName)) {
         
-        const XML_Char* pAttr;
+        const gchar* pAttr;
         
         pAttr = UT_getAttribute("style:family", ppAtts);
         UT_ASSERT(pAttr);
@@ -114,9 +115,9 @@ void ODi_Style_Style::startElement(const XML_Char* pName,
         m_displayName = m_name = "Normal";
         m_parentStyleName = "None";
         
-    } else if (!UT_strcmp("style:columns", pName)) {
+    } else if (!strcmp("style:columns", pName)) {
         
-        const XML_Char* pVal;
+        const gchar* pVal;
         
         pVal = UT_getAttribute("fo:column-count", ppAtts);
         if (pVal) {
@@ -142,13 +143,13 @@ void ODi_Style_Style::startElement(const XML_Char* pName,
 /**
  * 
  */
-void ODi_Style_Style::endElement(const XML_Char* pName,
+void ODi_Style_Style::endElement(const gchar* pName,
                                 ODi_ListenerStateAction& rAction) {
                                     
-    if (!UT_strcmp("style:style", pName)) {
+    if (!strcmp("style:style", pName)) {
         rAction.popState();
         
-    } else if (!UT_strcmp("style:default-style", pName)) {
+    } else if (!strcmp("style:default-style", pName)) {
         // I'm a default style.
         rAction.popState();
     }
@@ -161,9 +162,9 @@ void ODi_Style_Style::endElement(const XML_Char* pName,
 /**
  * 
  */
-void ODi_Style_Style::_parse_style_style(const XML_Char** ppAtts) {
+void ODi_Style_Style::_parse_style_style(const gchar** ppAtts) {
     
-    const XML_Char* pAttr;
+    const gchar* pAttr;
     
     if (m_name.empty()) {
         pAttr = UT_getAttribute("style:name", ppAtts);
@@ -220,9 +221,9 @@ void ODi_Style_Style::_parse_style_style(const XML_Char** ppAtts) {
 /**
  * 
  */
-void ODi_Style_Style::_parse_style_paragraphProperties(const XML_Char** ppProps) {
+void ODi_Style_Style::_parse_style_paragraphProperties(const gchar** ppProps) {
     
-    const XML_Char* pVal;
+    const gchar* pVal;
     
     pVal = UT_getAttribute ("style:line-height-at-least", ppProps);
     if (pVal) {
@@ -245,11 +246,11 @@ void ODi_Style_Style::_parse_style_paragraphProperties(const XML_Char** ppProps)
     
     pVal = UT_getAttribute ("fo:text-align", ppProps);
     if (pVal) {
-        if (!UT_strcmp(pVal, "end")) {
+        if (!strcmp(pVal, "end")) {
             m_align = "right";
-        } else if (!UT_strcmp(pVal, "center")) {
+        } else if (!strcmp(pVal, "center")) {
             m_align = "center";
-        } else if (!UT_strcmp(pVal, "justify")) {
+        } else if (!strcmp(pVal, "justify")) {
             m_align = "justify";
         } else {
             m_align = "left";
@@ -307,7 +308,7 @@ void ODi_Style_Style::_parse_style_paragraphProperties(const XML_Char** ppProps)
     
     pVal = UT_getAttribute("fo:keep-with-next", ppProps);
     if (pVal) {
-        if ( !UT_strcmp(pVal, "always") ) {
+        if ( !strcmp(pVal, "always") ) {
             m_keepWithNext = "yes";
         } else {
             m_keepWithNext.clear();
@@ -318,6 +319,15 @@ void ODi_Style_Style::_parse_style_paragraphProperties(const XML_Char** ppProps)
     if (pVal) {
         m_textIndent = pVal;
     }
+
+    pVal = UT_getAttribute("style:writing-mode", ppProps);
+    if (pVal) {
+        if(!strcmp(pVal,"rl") || !strcmp(pVal,"rl-tb") || !strcmp(pVal,"tb-rl")) {
+            m_direction = "rtl";
+        } else {
+            m_direction = "ltr";
+        }
+    }
 }
 
 
@@ -327,10 +337,10 @@ void ODi_Style_Style::_parse_style_paragraphProperties(const XML_Char** ppProps)
 /**
  * <style:text-properties />
  */
-void ODi_Style_Style::_parse_style_textProperties(const XML_Char** ppProps) {
+void ODi_Style_Style::_parse_style_textProperties(const gchar** ppProps) {
     
-    const XML_Char* pVal;
-    const XML_Char* pVal2;
+    const gchar* pVal = NULL;
+    const gchar* pVal2 = NULL;
     
     pVal = UT_getAttribute("fo:color", ppProps);
     if (pVal) {
@@ -338,36 +348,63 @@ void ODi_Style_Style::_parse_style_textProperties(const XML_Char** ppProps) {
     }
     
     
-    const XML_Char* undr = NULL;
-    const XML_Char* strk = NULL;
+    const gchar* undrStyle = NULL;
+    const gchar* undrType = NULL;
+    const gchar* strkStyle = NULL;
+    const gchar* strkType = NULL;
 
-    undr = const_cast<const XML_Char *>(
+    undrStyle = const_cast<const gchar *>(
         UT_getAttribute("style:text-underline-style", ppProps));
-    strk = const_cast<const XML_Char *>(
+    undrType = const_cast<const gchar *>(
+        UT_getAttribute("style:text-underline-type", ppProps));
+    strkStyle = const_cast<const gchar *>(
         UT_getAttribute("style:text-line-through-style", ppProps));
+    strkType = const_cast<const gchar *>(
+        UT_getAttribute("style:text-line-through-type", ppProps));
 
-    if (undr || strk) {
+    if (undrStyle || undrType || strkStyle || strkType) {
         // I'm not sure if "+=" works for an uninitialized string.
         m_textDecoration = "";
+        bool bUnderline = false, bStrike = false;
 
-        if(undr)
-            if (UT_strcmp(undr, "none") != 0)
+        if (undrStyle) {
+            if (strcmp(undrStyle, "none") != 0) {
                 m_textDecoration += "underline";
+                bUnderline = true;
+            }
+        }
 
-        if (undr && strk)
-            m_textDecoration += ",";
+        if (!bUnderline && undrType) { //don't append "underline" twice
+            if (strcmp(undrType, "none") != 0) {
+                m_textDecoration += "underline";
+            }
+        }
 
-        if(strk)
-            if (UT_strcmp(strk, "none") != 0)
+        if (strkStyle) {
+            if (strcmp(strkStyle, "none") != 0) {
+                if(m_textDecoration.length())
+                    m_textDecoration += " "; //separate the props with a space, not a comma
                 m_textDecoration += "line-through";
+                bStrike = true;
+            }
+        }
+
+        if (!bStrike && strkType) { //don't append "line-through" twice
+            if (strcmp(strkType, "none") != 0) {
+                if(m_textDecoration.length())
+                    m_textDecoration += " "; //separate the props with a space, not a comma
+                m_textDecoration += "line-through";
+            }
+        }
     }
-    
-    
+
     pVal = UT_getAttribute("style:text-position", ppProps);
     if(pVal) {
+        int position = 0;
+
         if (strstr(pVal, "sub") || strstr(pVal, "-"))
             m_textPos = "subscript";
-        else if (strstr(pVal, "super"))
+        else if (strstr(pVal, "super") || ((sscanf(pVal, "%d%%", &position) == 1) && (position > 0)))
             m_textPos = "superscript";
         else 
             m_textPos = "normal";
@@ -388,7 +425,7 @@ void ODi_Style_Style::_parse_style_textProperties(const XML_Char** ppProps) {
     pVal2 = UT_getAttribute("fo:country", ppProps);
     if ( pVal && pVal2 ) {
             
-        if (!UT_strcmp(pVal, "none") && !UT_strcmp(pVal2, "none")) {
+        if (!strcmp(pVal, "none") && !strcmp(pVal2, "none")) {
             // AbiWord uses "-none-" instead of "none-none";
             m_lang = "-none-";
         } else {
@@ -399,7 +436,7 @@ void ODi_Style_Style::_parse_style_textProperties(const XML_Char** ppProps) {
     
     pVal = UT_getAttribute("fo:font-style", ppProps);
     if (pVal) {
-        if (!UT_strcmp(pVal, "italic") || !UT_strcmp(pVal, "normal")) {
+        if (!strcmp(pVal, "italic") || !strcmp(pVal, "normal")) {
             m_fontStyle = pVal;
         }
     }
@@ -407,12 +444,27 @@ void ODi_Style_Style::_parse_style_textProperties(const XML_Char** ppProps) {
     
     pVal = UT_getAttribute ("fo:font-weight", ppProps);
     if(pVal) {
-        if (!UT_strcmp(pVal, "bold")) {
+        if (!strcmp(pVal, "bold")) {
             m_fontWeight = "bold";
         } else {
             m_fontWeight = "normal";
         }
     }
+
+    // Note (for testing interoperability): OO.org doesn't correctly support hidden text:
+    // http://qa.openoffice.org/issues/show_bug.cgi?id=64237
+
+    pVal = UT_getAttribute ("text:display", ppProps);
+    if(pVal) {
+        if (!strcmp(pVal, "none")) {
+            m_display = pVal;
+        }
+    }
+
+    pVal = UT_getAttribute ("fo:background-color", ppProps);
+    if(pVal) {
+        m_bgcolor.assign(pVal);
+    }
 }
 
 
@@ -422,9 +474,9 @@ void ODi_Style_Style::_parse_style_textProperties(const XML_Char** ppProps) {
 /**
  * 
  */
-void ODi_Style_Style::_parse_style_sectionProperties(const XML_Char** ppProps) {
+void ODi_Style_Style::_parse_style_sectionProperties(const gchar** ppProps) {
     
-    const XML_Char* pVal;
+    const gchar* pVal;
     
     pVal = UT_getAttribute("fo:column-count", ppProps);
     if (pVal) {
@@ -439,22 +491,34 @@ void ODi_Style_Style::_parse_style_sectionProperties(const XML_Char** ppProps) {
 /**
  * <style:graphic-properties />
  */
-void ODi_Style_Style::_parse_style_graphicProperties(const XML_Char** ppProps) {
-    const XML_Char* pVal;
+void ODi_Style_Style::_parse_style_graphicProperties(const gchar** ppProps) {
+    const gchar* pVal;
     
     pVal = UT_getAttribute("style:wrap", ppProps);
     if (pVal) {
         m_wrap = pVal;
     }
-}
 
+    pVal = UT_getAttribute("fo:border-top", ppProps);
+    if (pVal) {
+        _stripColorLength(m_borderTop_color, m_borderTop_thickness, m_haveTopBorder, pVal);
+    }
+
+    pVal = UT_getAttribute("fo:border-bottom", ppProps);
+    if (pVal) {
+        _stripColorLength(m_borderBottom_color, m_borderBottom_thickness, m_haveBottomBorder, pVal);
+    }
+
+    pVal = UT_getAttribute("fo:border-left", ppProps);
+    if (pVal) {
+        _stripColorLength(m_borderLeft_color, m_borderLeft_thickness, m_haveLeftBorder, pVal);
+    }
+
+    pVal = UT_getAttribute("fo:border-right", ppProps);
+    if (pVal) {
+        _stripColorLength(m_borderRight_color, m_borderRight_thickness, m_haveRightBorder, pVal);
+    }
 
-/**
- * <style:table-properties />
- */
-void ODi_Style_Style::_parse_style_tableProperties(const XML_Char** ppProps) {
-    const XML_Char* pVal;
-    
     pVal = UT_getAttribute("fo:background-color", ppProps);
     if (pVal) {
         m_backgroundColor = pVal;
@@ -462,11 +526,40 @@ void ODi_Style_Style::_parse_style_tableProperties(const XML_Char** ppProps) {
 }
 
 
+/**
+ * <style:table-properties />
+ */
+void ODi_Style_Style::_parse_style_tableProperties(const gchar** ppProps) {
+    const gchar* pVal;
+    
+    pVal = UT_getAttribute("fo:background-color", ppProps);
+    if (pVal) {
+        m_backgroundColor = pVal;
+    }
+
+    pVal = UT_getAttribute("fo:margin-left",ppProps);
+    if (pVal) {
+        m_TableMarginLeft = pVal;
+    }
+
+    pVal = UT_getAttribute("fo:margin-right",ppProps);
+    if (pVal) {
+        m_TableMarginRight = pVal;
+    }
+
+
+    pVal = UT_getAttribute("style:width",ppProps);
+    if (pVal) {
+        m_TableWidth = pVal;
+    }
+}
+
+
 /**
  * <style:table-column-properties />
  */
-void ODi_Style_Style::_parse_style_tableColumnProperties(const XML_Char** ppProps) {
-    const XML_Char* pVal;
+void ODi_Style_Style::_parse_style_tableColumnProperties(const gchar** ppProps) {
+    const gchar* pVal;
     
     pVal = UT_getAttribute("style:column-width", ppProps);
     if (pVal) {
@@ -478,8 +571,8 @@ void ODi_Style_Style::_parse_style_tableColumnProperties(const XML_Char** ppProp
 /**
  * <style:table-row-properties />
  */
-void ODi_Style_Style::_parse_style_tableRowProperties(const XML_Char** ppProps) {
-    const XML_Char* pVal;
+void ODi_Style_Style::_parse_style_tableRowProperties(const gchar** ppProps) {
+    const gchar* pVal;
     
     pVal = UT_getAttribute("style:min-row-height", ppProps);
     if (pVal) {
@@ -496,8 +589,8 @@ void ODi_Style_Style::_parse_style_tableRowProperties(const XML_Char** ppProps)
 /**
  * <style:table-cell-properties />
  */
-void ODi_Style_Style::_parse_style_tableCellProperties(const XML_Char** ppProps) {
-    const XML_Char* pVal;
+void ODi_Style_Style::_parse_style_tableCellProperties(const gchar** ppProps) {
+    const gchar* pVal;
     
     // If "fo:border" is defined, its value will fill all "fo:border-*"
     pVal = UT_getAttribute("fo:border", ppProps);
@@ -591,18 +684,18 @@ void ODi_Style_Style::defineAbiStyle(PD_Document* pDocument) {
      *           ...
      *           a[n] = 0 (NULL character)
      */
-    const XML_Char* pAttr[11];
+    const gchar* pAttr[11];
     UT_uint32 i = 0;
     bool ok;
     
     pAttr[i++] = "type";
-    if (!UT_strcmp("paragraph", m_family.utf8_str())) {
+    if (!strcmp("paragraph", m_family.utf8_str())) {
         pAttr[i++] = "P";
-    } else if (!UT_strcmp("text", m_family.utf8_str())) {
+    } else if (!strcmp("text", m_family.utf8_str())) {
         pAttr[i++] = "C";
     } else {
         // Really shouldn't happen
-        UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+        UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
     }
     
     // AbiWord uses the display name
@@ -626,7 +719,7 @@ void ODi_Style_Style::defineAbiStyle(PD_Document* pDocument) {
     pAttr[i] = 0; // Signal the end of the array 
     
     ok = pDocument->appendStyle(pAttr);
-    UT_ASSERT(ok);
+    UT_ASSERT_HARMLESS(ok);
 }
 
 
@@ -647,7 +740,7 @@ void ODi_Style_Style::buildAbiPropsAttrString(ODi_FontFaceDecls& rFontFaceDecls)
             UT_DEBUGMSG(("*** [OpenDocument] no parent style to resolve '%s'\n",
                 m_fontSize.utf8_str()));
                 
-            UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+            UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
             
             // Ignore its value then
             m_fontSize.clear();
@@ -691,6 +784,7 @@ void ODi_Style_Style::buildAbiPropsAttrString(ODi_FontFaceDecls& rFontFaceDecls)
     APPEND_STYLE("bgcolor: ", m_bgcolor);
     APPEND_STYLE("keep-with-next: ", m_keepWithNext);
     APPEND_STYLE("text-indent: ", m_textIndent);
+    APPEND_STYLE("dom-dir: ", m_direction);
     
     // <style:text-properties />
     APPEND_STYLE("color: ", m_color);
@@ -698,13 +792,11 @@ void ODi_Style_Style::buildAbiPropsAttrString(ODi_FontFaceDecls& rFontFaceDecls)
     APPEND_STYLE("text-position: ", m_textPos);
     
     if (!m_fontName.empty()) {
-        const UT_UTF8String* pFontFamily;
+        const std::string & fontFamily = rFontFaceDecls.getFontFamily(m_fontName);
         
-        pFontFamily = rFontFaceDecls.getFontFamily(m_fontName);
-        
-        UT_ASSERT(pFontFamily);
-        if (pFontFamily!=NULL) {
-            APPEND_STYLE("font-family: ", (*pFontFamily));
+        UT_ASSERT_HARMLESS(!fontFamily.empty());
+        if (!fontFamily.empty()) {
+            APPEND_STYLE("font-family: ", fontFamily);
         }
     }
     
@@ -712,6 +804,11 @@ void ODi_Style_Style::buildAbiPropsAttrString(ODi_FontFaceDecls& rFontFaceDecls)
     APPEND_STYLE("lang: ", m_lang);
     APPEND_STYLE("font-style: ", m_fontStyle);
     APPEND_STYLE("font-weight: ", m_fontWeight);
+
+    // AbiWord hangs when a paragraph has a "display:none" property
+    if (m_family.length() && !strcmp("text", m_family.utf8_str())) {
+        APPEND_STYLE("display: ", m_display);
+    }
     
     // <style:section-properties />
     APPEND_STYLE("columns: ", m_columns);
@@ -771,7 +868,7 @@ const UT_UTF8String* ODi_Style_Style::getWrap(bool local) const {
 void ODi_Style_Style::_stripColorLength(UT_UTF8String& rColor,
                                   UT_UTF8String& rLength,
                                   ODi_Style_Style::HAVE_BORDER& rHaveBorder,
-                                  const XML_Char* pString) const {
+                                  const gchar* pString) const {
                                         
     UT_uint16 i, start;
     bool hasWord;
@@ -779,7 +876,7 @@ void ODi_Style_Style::_stripColorLength(UT_UTF8String& rColor,
     rColor.clear();
     rLength.clear();
     
-    if (!UT_strcmp(pString, "none")) {
+    if (!strcmp(pString, "none")) {
         // Color and length remain empty.
         rHaveBorder = HAVE_BORDER_NO;
         return;
@@ -830,7 +927,7 @@ void ODi_Style_Style::_stripColorLength(UT_UTF8String& rColor,
  * 
  * @param length 0 for NULL terminated strings.
  */
-bool ODi_Style_Style::_isValidDimensionString(const XML_Char* pString,
+bool ODi_Style_Style::_isValidDimensionString(const gchar* pString,
                                              UT_uint32 length) const {
     UT_uint32 i;
     bool gotDecimalSeparator;
@@ -865,7 +962,7 @@ bool ODi_Style_Style::_isValidDimensionString(const XML_Char* pString,
     }
     
     
-    XML_Char dimStr[100];
+    gchar dimStr[100];
     UT_Dimension dim;
     UT_uint32 j;
     
