@@ -59,7 +59,7 @@ ODi_Style_Style_Family::~ODi_Style_Style_Family() {
  * @param pReplacementName Name to replace the actual name of the style that is
  *                         being added. NULL if the name shouldn't be replaced.
  */
-ODi_Style_Style* ODi_Style_Style_Family::addStyle(const XML_Char** ppAtts,
+ODi_Style_Style* ODi_Style_Style_Family::addStyle(const gchar** ppAtts,
                              ODi_ElementStack& rElementStack,
                              UT_UTF8String* pReplacementName,
                              UT_UTF8String* pReplacementDisplayName) {
@@ -67,7 +67,7 @@ ODi_Style_Style* ODi_Style_Style_Family::addStyle(const XML_Char** ppAtts,
     ODi_Style_Style* pStyle;
     bool bOnContentStream;
     bool ok = true;
-    const XML_Char* pName;
+    const gchar* pName;
     
     bOnContentStream = rElementStack.hasElement("office:document-content");
     
@@ -133,15 +133,12 @@ ODi_Style_Style* ODi_Style_Style_Family::addStyle(const XML_Char** ppAtts,
         UT_UTF8String originalName = pName;
         
         if (bOnContentStream) {
-            ok = m_removedStyleStyles_contentStream.ins(originalName,
-                                                        *pReplacementName);
+            m_removedStyleStyles_contentStream[pName]
+				= pReplacementName->utf8_str();
         } else {
-            ok = m_removedStyleStyles.ins(originalName,
-                                          *pReplacementName);
+            m_removedStyleStyles[pName] = pReplacementName->utf8_str();
         }
-        
-        UT_ASSERT(ok);
-    }
+	}
     
     return pStyle;
 }
@@ -158,12 +155,15 @@ void ODi_Style_Style_Family::fixStyles() {
     
     UT_uint32 i, count;
     UT_GenericVector<ODi_Style_Style*>* pStylesVec;
-    ODi_Style_Style* pStyle;
+    ODi_Style_Style* pStyle = NULL;
     bool noneFound;
     
     do {
         pStylesVec = m_styles.enumerate();
-        UT_ASSERT(pStylesVec);
+        if(!pStylesVec) {
+           UT_ASSERT_HARMLESS(pStylesVec);
+           break; //using continue here seems like it could cause an infinite loop?
+        }
         
         noneFound = true;
         count = pStylesVec->getItemCount();
@@ -186,7 +186,10 @@ void ODi_Style_Style_Family::fixStyles() {
     
     do {
         pStylesVec = m_styles_contentStream.enumerate();
-        UT_ASSERT(pStylesVec);
+        if(!pStylesVec) {
+           UT_ASSERT_HARMLESS(pStylesVec);
+           break; //using continue here seems like it could cause an infinite loop?
+        }
         
         noneFound = true;
         count = pStylesVec->getItemCount();
@@ -219,7 +222,7 @@ void ODi_Style_Style_Family::buildAbiPropsAttrString(
     }
     
     pStylesVec = m_styles.enumerate();
-    UT_ASSERT(pStylesVec);
+    UT_return_if_fail(pStylesVec);
     count = pStylesVec->getItemCount();
     for (i=0; i<count; i++) {
         (*pStylesVec)[i]->buildAbiPropsAttrString(rFontFaceDecls);
@@ -227,7 +230,7 @@ void ODi_Style_Style_Family::buildAbiPropsAttrString(
 	DELETEP(pStylesVec);
     
     pStylesVec = m_styles_contentStream.enumerate();
-    UT_ASSERT(pStylesVec);
+    UT_return_if_fail(pStylesVec);
     count = pStylesVec->getItemCount();
     for (i=0; i<count; i++) {
         (*pStylesVec)[i]->buildAbiPropsAttrString(rFontFaceDecls);
@@ -246,7 +249,6 @@ void ODi_Style_Style_Family::removeStyleStyle(ODi_Style_Style* pRemovedStyle,
     UT_GenericVector<ODi_Style_Style*>* pStylesVec;
     UT_UTF8String styleName;
     UT_UTF8String replacementName;
-    bool ok;
 
 
     _findSuitableReplacement(replacementName, pRemovedStyle, bOnContentStream);
@@ -256,15 +258,13 @@ void ODi_Style_Style_Family::removeStyleStyle(ODi_Style_Style* pRemovedStyle,
         m_styles_contentStream.remove(
             pRemovedStyle->getName().utf8_str(), NULL);
             
-        ok = m_removedStyleStyles_contentStream.ins(pRemovedStyle->getName(),
-                                                   replacementName);
+        m_removedStyleStyles_contentStream[pRemovedStyle->getName().utf8_str()]
+			= replacementName.utf8_str();
     } else {
         m_styles.remove(pRemovedStyle->getName().utf8_str(), NULL);
-        ok = m_removedStyleStyles.ins(pRemovedStyle->getName(), replacementName);
+        m_removedStyleStyles[pRemovedStyle->getName().utf8_str()] = replacementName.utf8_str();
     }
     
-    UT_ASSERT(ok);
-
 
 
     // Fix all references to it.
@@ -275,7 +275,7 @@ void ODi_Style_Style_Family::removeStyleStyle(ODi_Style_Style* pRemovedStyle,
         return;
     }
     
-    if (!UT_strcmp(replacementName.utf8_str(), "<NULL>")) {
+    if (!strcmp(replacementName.utf8_str(), "<NULL>")) {
         replacementName.clear();
     }
     
@@ -283,7 +283,7 @@ void ODi_Style_Style_Family::removeStyleStyle(ODi_Style_Style* pRemovedStyle,
     // references to this common style.
     pStylesVec = m_styles_contentStream.enumerate();
 
-    UT_ASSERT(pStylesVec);
+    UT_return_if_fail(pStylesVec);
     
     count = pStylesVec->getItemCount();
     for (i=0; i<count; i++) {
@@ -300,7 +300,7 @@ void ODi_Style_Style_Family::removeStyleStyle(ODi_Style_Style* pRemovedStyle,
     // Now fix references from the styles defined on the styles stream.
     pStylesVec = m_styles.enumerate();
 
-    UT_ASSERT(pStylesVec);
+    UT_return_if_fail(pStylesVec);
     
     count = pStylesVec->getItemCount();
     for (i=0; i<count; i++) {
@@ -328,7 +328,7 @@ void ODi_Style_Style_Family::defineAbiStyles(PD_Document* pDocument) const {
     }
     
     pStylesVec = m_styles.enumerate();
-    UT_ASSERT(pStylesVec);
+    UT_return_if_fail(pStylesVec);
     
     count = pStylesVec->getItemCount();
     for (i=0; i<count; i++) {
@@ -397,19 +397,20 @@ void ODi_Style_Style_Family::_findSuitableReplacement(
             _findSuitableReplacement(rReplacementName, pStyle, bOnContentStream);
         }
     } else {
-        const UT_UTF8String* pString;
+        std::string aString;
         // Was this parent already removed?
         if (bOnContentStream) {
-            pString = m_removedStyleStyles_contentStream[
-                                    pRemovedStyle->getParentName()];
+            aString = m_removedStyleStyles_contentStream[
+                                    pRemovedStyle->getParentName().utf8_str()];
         }
         
         if (!pStyle) {
-            pString = m_removedStyleStyles[pRemovedStyle->getParentName()];
+            aString = m_removedStyleStyles[pRemovedStyle->getParentName()
+										   .utf8_str()];
         }
         
-        if(pString) {
-            rReplacementName = *pString;
+        if(!aString.empty()) {
+            rReplacementName = aString.c_str();
         } else {
             // I give up...
             if (m_pDefaultStyle) {
@@ -444,13 +445,16 @@ void ODi_Style_Style_Family::linkStyles() {
 /**
  * 
  */
-const ODi_Style_Style* ODi_Style_Style_Family::getStyle(const XML_Char* pStyleName,
+const ODi_Style_Style* ODi_Style_Style_Family::getStyle(const gchar* pStyleName,
                                                       bool bOnContentStream) {
+
+    UT_return_val_if_fail(pStyleName, NULL);
+
     const ODi_Style_Style* pStyle = NULL;
     
     // Is it the default style?
     if (m_pDefaultStyle != NULL) {
-        if (!UT_strcmp(m_pDefaultStyle->getName().utf8_str(), pStyleName)) {
+        if (!strcmp(m_pDefaultStyle->getName().utf8_str(), pStyleName)) {
             pStyle = m_pDefaultStyle;
         }
     }
@@ -472,21 +476,21 @@ const ODi_Style_Style* ODi_Style_Style_Family::getStyle(const XML_Char* pStyleNa
     if (!pStyle) {
         // We haven't found it. Have we removed it (done on _fixStyles())?
         
-        const UT_UTF8String* pReplacementName;
+        std::string replacementName;
         
         if (bOnContentStream) {
-            pReplacementName = m_removedStyleStyles_contentStream[pStyleName];
+            replacementName = m_removedStyleStyles_contentStream[pStyleName];
             
-            if (pReplacementName == NULL) {
-                pReplacementName = m_removedStyleStyles[pStyleName];
+            if (replacementName.empty()) {
+                replacementName = m_removedStyleStyles[pStyleName];
             }
         } else {
-            pReplacementName = m_removedStyleStyles[pStyleName];
+            replacementName = m_removedStyleStyles[pStyleName];
         }
         
-        if (pReplacementName) {
+        if (!replacementName.empty()) {
             // We will send back its replacement.
-            return this->getStyle(pReplacementName->utf8_str(),
+            return this->getStyle(replacementName.c_str(),
                                            bOnContentStream);
         } else {
             // This style never existed.
@@ -518,7 +522,7 @@ void ODi_Style_Style_Family::_linkStyles(bool onContentStream) {
     } else {
         pStylesVec = m_styles.enumerate();
     }
-    UT_ASSERT(pStylesVec);
+    UT_return_if_fail(pStylesVec);
     
     count = pStylesVec->getItemCount();
     for (i=0; i<count; i++) {
@@ -530,7 +534,7 @@ void ODi_Style_Style_Family::_linkStyles(bool onContentStream) {
             pOtherStyle = this->getStyle(pStyle->getParentName().utf8_str(),
                                          onContentStream);
             
-            UT_ASSERT(pOtherStyle);
+            UT_ASSERT_HARMLESS(pOtherStyle);
             
             if (pOtherStyle) {
                 pStyle->setParentStylePointer(pOtherStyle);
@@ -547,7 +551,7 @@ void ODi_Style_Style_Family::_linkStyles(bool onContentStream) {
             pOtherStyle = this->getStyle(pStyle->getNextStyleName().utf8_str(),
                                          onContentStream);
                 
-            UT_ASSERT(pOtherStyle);
+            UT_ASSERT_HARMLESS(pOtherStyle);
                 
             if (pOtherStyle) {
                 pStyle->setNextStylePointer(pOtherStyle);
