@@ -59,22 +59,22 @@ public:
                             
     virtual ~ODi_Table_ListenerState() {}
 
-    void startElement (const XML_Char* pName, const XML_Char** ppAtts,
+    void startElement (const gchar* pName, const gchar** ppAtts,
                        ODi_ListenerStateAction& rAction);
                        
-    void endElement (const XML_Char* pName, ODi_ListenerStateAction& rAction);
+    void endElement (const gchar* pName, ODi_ListenerStateAction& rAction);
     
-    void charData (const XML_Char* pBuffer, int length) {}
+    void charData (const gchar* /*pBuffer*/, int /*length*/) {}
     
 private:
 
-    void _parseTableStart(const XML_Char** ppAtts,
+    void _parseTableStart(const gchar** ppAtts,
                           ODi_ListenerStateAction& rAction);
 
-    void _parseColumnStart(const XML_Char** ppAtts,
+    void _parseColumnStart(const gchar** ppAtts,
                            ODi_ListenerStateAction& rAction);
 
-    void _parseCellStart(const XML_Char** ppAtts,
+    void _parseCellStart(const gchar** ppAtts,
                          ODi_ListenerStateAction& rAction);
     
     bool m_onContentStream;
