@@ -65,16 +65,9 @@ IE_Imp_OpenDocument::~IE_Imp_OpenDocument ()
 /**
  * Import the given file
  */
-UT_Error IE_Imp_OpenDocument::importFile(const char * szFilename)
+UT_Error IE_Imp_OpenDocument::_loadFile (GsfInput * oo_src)
 {
-    GsfInput* oo_src = GSF_INPUT (gsf_input_stdio_new (szFilename, NULL));
-      
-    if (oo_src == NULL) {
-        return UT_ERROR;
-    }
-
     m_pGsfInfile = GSF_INFILE (gsf_infile_zip_new (oo_src, NULL));
-    g_object_unref (G_OBJECT (oo_src));
     
     if (m_pGsfInfile == NULL) {
         return UT_ERROR;
@@ -147,7 +140,7 @@ UT_Error IE_Imp_OpenDocument::_handleMimetype ()
     if (!pInput) {
         UT_DEBUGMSG(("Error: didn't get a mimetype. Assuming that it's a"
         " application/vnd.oasis.opendocument.text document\n"));
-        UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+        UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
         return UT_OK;
     }
 
@@ -159,7 +152,9 @@ UT_Error IE_Imp_OpenDocument::_handleMimetype ()
             gsf_input_size (pInput));
     }
 
-    if (UT_strcmp("application/vnd.oasis.opendocument.text", mimetype.utf8_str()) != 0) {
+    if ((strcmp("application/vnd.oasis.opendocument.text", mimetype.utf8_str()) != 0) &&
+        (strcmp("application/vnd.oasis.opendocument.text-template", mimetype.utf8_str()) != 0))
+    {
         UT_DEBUGMSG(("*** Unknown mimetype '%s'\n", mimetype.utf8_str()));
         return UT_IE_BOGUSDOCUMENT;
     }
@@ -168,20 +163,36 @@ UT_Error IE_Imp_OpenDocument::_handleMimetype ()
     return UT_OK;
 }
 
+gboolean gsf_infile_child_exists(GsfInfile *infile, char const *name)
+{
+    GsfInput* pInput = gsf_infile_child_by_name(infile, name);
+
+    if (!pInput) {
+      return FALSE;
+    }
+
+    g_object_unref (G_OBJECT (pInput));
+    return TRUE;
+}
 
 /**
  * Handle the meta-stream
  */
 UT_Error IE_Imp_OpenDocument::_handleMetaStream ()
-{
-    UT_Error error;
-    
-    error = m_pStreamListener->setState("MetaStream");
-    if (error != UT_OK) {
-        return error;
+{ 
+    if (gsf_infile_child_exists (m_pGsfInfile, "meta.xml")) {
+	UT_Error error;
+	
+	error = m_pStreamListener->setState("MetaStream");
+	if (error != UT_OK) {
+	  return error;
+	}
+	
+	return _handleStream (m_pGsfInfile, "meta.xml", *m_pStreamListener);
     }
-    
-    return _handleStream (m_pGsfInfile, "meta.xml", *m_pStreamListener);
+
+    // no metadata. let's not get our panties in a bunch over it
+    return UT_OK;
 }
 
 
@@ -213,8 +224,6 @@ UT_Error IE_Imp_OpenDocument::_handleStylesStream ()
         return error;
     }
     
-    m_pStreamListener->clearFontFaceDecls();
-        
     return _handleStream (m_pGsfInfile, "styles.xml", *m_pStreamListener);
 }
 
@@ -231,8 +240,6 @@ UT_Error IE_Imp_OpenDocument::_handleContentStream ()
         return error;
     }
     
-    m_pStreamListener->clearFontFaceDecls();
-    
     return _handleStream (m_pGsfInfile, "content.xml", *m_pStreamListener);
 }
 
@@ -293,7 +300,7 @@ UT_Error IE_Imp_OpenDocument::_parseStream ( GsfInfile* pGsfInfile,
  */
 void IE_Imp_OpenDocument::_setDocumentProperties() {
 
-    const XML_Char* ppProps[5];
+    const gchar* ppProps[5];
     bool ok;
 
     // OpenDocument endnotes are, by definition, placed at the end of the document.    
@@ -305,5 +312,5 @@ void IE_Imp_OpenDocument::_setDocumentProperties() {
     ppProps[4] = 0;
 
     ok = getDoc()->setProperties(ppProps);
-    UT_ASSERT(ok);
+    UT_ASSERT_HARMLESS(ok);
 }
