@@ -32,6 +32,29 @@
 
 #include "ut_debugmsg.h"
 
+static void
+oo_gsf_output_write(GsfOutput *output, size_t num_bytes, guint8 const *data)
+{
+  if (!gsf_output_write(output, num_bytes, data)) {
+    UT_DEBUGMSG(("DOM: gsf_output_write() failed\n"));
+    UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+  }
+}
+
+static void
+oo_gsf_output_close(GsfOutput *output)
+{
+  if (!gsf_output_close(output)) {
+    const GError * err = gsf_output_error(output);
+    UT_DEBUGMSG(("DOM: gsf_output_close() failed\n"));
+    if (err) {
+      UT_DEBUGMSG(("DOM: reason: %s\n", err->message));
+    }
+    UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+  }
+  g_object_unref(output);
+}
+
 /*****************************************************************************/
 /*****************************************************************************/
 
@@ -49,7 +72,7 @@ IE_Exp_OpenWriter_Sniffer::~IE_Exp_OpenWriter_Sniffer()
  */
 bool IE_Exp_OpenWriter_Sniffer::recognizeSuffix(const char * szSuffix)
 {
-  return (!UT_stricmp(szSuffix,".sxw"));
+  return (!g_ascii_strcasecmp(szSuffix,".sxw"));
 }
 
 /*!
@@ -84,17 +107,17 @@ bool IE_Exp_OpenWriter_Sniffer::getDlgLabels(const char ** pszDesc,
 static void writeToStream (GsfOutput * stream, const char * const message [], size_t nElements)
 {
     for(UT_uint32 k = 0; k < nElements; k++)
-      gsf_output_write(stream, strlen(message[k]), reinterpret_cast<const guint8 *>(message[k]));
+      oo_gsf_output_write(stream, strlen(message[k]), reinterpret_cast<const guint8 *>(message[k]));
 }
 
 static void writeString (GsfOutput * output, const UT_String & str)
 {
-   gsf_output_write (output, str.length(), reinterpret_cast<const guint8*>(str.c_str()));
+   oo_gsf_output_write (output, str.length(), reinterpret_cast<const guint8*>(str.c_str()));
 }
 
 static void writeUTF8String (GsfOutput * output, const UT_UTF8String & str)
 {
-  gsf_output_write (output, str.byteLength(), reinterpret_cast<const guint8*>(str.utf8_str()));
+  oo_gsf_output_write (output, str.byteLength(), reinterpret_cast<const guint8*>(str.utf8_str()));
 }
 
 static void outputCharData (GsfOutput * output, const UT_UCSChar * data, UT_uint32 length)
@@ -122,9 +145,14 @@ static void outputCharData (GsfOutput * output, const UT_UCSChar * data, UT_uint
 	 sBuf += "&amp;";
 	 pData++;
 	 break;
-	 
+
+      case UCS_LF: //line breaks
+	 sBuf += "<text:line-break/>";
+	 pData++;
+	 break;
+
       case UCS_TAB:
-	 sBuf += "\t";
+	 sBuf += "<text:tab-stop/>";
 	 pData++;
 	 break;
 	 
@@ -157,16 +185,14 @@ OO_WriterImpl::OO_WriterImpl(GsfOutfile *pOutfile, OO_StylesContainer *pStylesCo
 		"<office:document-content xmlns:office=\"http://openoffice.org/2000/office\" xmlns:style=\"http://openoffice.org/2000/style\" xmlns:text=\"http://openoffice.org/2000/text\" xmlns:table=\"http://openoffice.org/2000/table\" xmlns:draw=\"http://openoffice.org/2000/drawing\" xmlns:fo=\"http://www.w3.org/1999/XSL/Format\" xmlns:xlink=\"http://www.w3.org/1999/xlink\" xmlns:number=\"http://openoffice.org/2000/datastyle\" xmlns:svg=\"http://www.w3.org/2000/svg\" xmlns:chart=\"http://openoffice.org/2000/chart\" xmlns:dr3d=\"http://openoffice.org/2000/dr3d\" xmlns:math=\"http://www.w3.org/1998/Math/MathML\" xmlns:form=\"http://openoffice.org/2000/form\" xmlns:script=\"http://openoffice.org/2000/script\" office:class=\"text\" office:version=\"1.0\">\n",
 		"<office:script/>\n"
 	};
-	writeToStream(m_pContentStream, preamble, NrElements(preamble));
-
+	writeToStream(m_pContentStream, preamble, G_N_ELEMENTS(preamble));
 
 	// in my sample files content.xml, styles.xml font-decl sections are the same
-	UT_UTF8String fontDecls = "<office:font-decls>";
+	UT_UTF8String fontDecls = "<office:font-decls>\n";
 	OO_StylesWriter::addFontDecls(fontDecls, *m_pStylesContainer);
-	fontDecls += "</office:font-decls>";
+	fontDecls += "</office:font-decls>\n";
 	writeUTF8String(m_pContentStream, fontDecls);
 	
-
 	writeUTF8String(m_pContentStream, "<office:automatic-styles>\n");
 	
 	int *styleNum = NULL;
@@ -185,8 +211,8 @@ OO_WriterImpl::OO_WriterImpl(GsfOutfile *pOutfile, OO_StylesContainer *pStylesCo
 	   UT_DEBUGMSG(("%s", styleString.c_str()));
 	   UT_DEBUGMSG(("\"%s\"\n", styleProps->c_str()));
 	}
-	FREEP(tempStylesKeysList);
-	FREEP(tempStylesValuesList);
+	DELETEP(tempStylesKeysList);
+	DELETEP(tempStylesValuesList);
 
 	// block styles
 	UT_GenericVector<const UT_String*> *tempBlockStylesKeysList = m_pStylesContainer->getBlockStylesKeys();
@@ -200,7 +226,7 @@ OO_WriterImpl::OO_WriterImpl(GsfOutfile *pOutfile, OO_StylesContainer *pStylesCo
 		styleString += UT_String_sprintf("</style:style>");
 		writeString(m_pContentStream, styleString);
 	}
-	FREEP(tempBlockStylesKeysList);
+	DELETEP(tempBlockStylesKeysList);
 
     //m_acc.writeStylePreamble(m_contentStream);
     
@@ -216,7 +242,7 @@ OO_WriterImpl::OO_WriterImpl(GsfOutfile *pOutfile, OO_StylesContainer *pStylesCo
 	  "</text:sequence-decls>\n"
        };
 
-    writeToStream(m_pContentStream, midsection, NrElements(midsection));
+    writeToStream(m_pContentStream, midsection, G_N_ELEMENTS(midsection));
 }
 
 OO_WriterImpl::~OO_WriterImpl()
@@ -226,10 +252,9 @@ OO_WriterImpl::~OO_WriterImpl()
 	  "</office:body>\n",
 	  "</office:document-content>\n"
       };
-    writeToStream(m_pContentStream, postamble, NrElements(postamble));
+    writeToStream(m_pContentStream, postamble, G_N_ELEMENTS(postamble));
 
-    gsf_output_close(m_pContentStream);
-    g_object_unref(G_OBJECT(m_pContentStream));
+    oo_gsf_output_close(m_pContentStream);
 }
 
 void OO_WriterImpl::insertText(const UT_UCSChar * data, UT_uint32 length)
@@ -254,12 +279,12 @@ void OO_WriterImpl::openBlock(UT_String & styleAtts, UT_String & styleProps, UT_
 	if (bIsHeading) 
 	{
 		tag = "<text:h " + props + ">";
-		m_blockEnd = "</text:h>";
+		m_blockEnd = "</text:h>\n";
 	}
 	else
 	{
 		tag = "<text:p " + props + ">";
-		m_blockEnd = "</text:p>";
+		m_blockEnd = "</text:p>\n";
 	}	
 
 	writeUTF8String(m_pContentStream, tag);
@@ -285,10 +310,36 @@ void OO_WriterImpl::closeSpan()
    writeUTF8String(m_pContentStream, closeSpan);
 }
 
+void OO_WriterImpl::openHyperlink(const PP_AttrProp* pAP)
+{
+	UT_return_if_fail(pAP);
+
+	UT_UTF8String output = "<text:a ", escape;
+	const gchar* pValue = NULL;
+
+	if(pAP->getAttribute("xlink:href",pValue) && pValue)
+	{
+		escape = pValue;
+		escape.escapeURL();
+
+		if(escape.length())
+		{
+			output+="xlink:href=\"";
+			output+=escape;
+			output+="\">";
+			writeUTF8String(m_pContentStream, output);
+		}
+	}
+}
+
+void OO_WriterImpl::closeHyperlink()
+{
+    UT_UTF8String output = "</text:a>";
+    writeUTF8String(m_pContentStream, output);
+}
 
 void OO_StylesContainer::addSpanStyle(UT_String &key)
 {
-   const void *temp;
    //if (!m_spanStylesHash.contains(key.utf8_str(), temp)) 
    if (!m_spanStylesHash.pick(key.c_str())) 
    {
@@ -349,7 +400,6 @@ const int OO_StylesContainer::getSpanStyleNum(UT_String &key) const
 const int OO_StylesContainer::getBlockStyleNum(UT_String & styleAtts, UT_String & styleProps) const
 {
 	UT_GenericVector<const UT_String*> *keys = m_blockAttsHash.keys();
-	int num = keys->findItem(&styleProps);
 
 	for (int i = 0; i < keys->size(); i++) 
 	{
@@ -409,7 +459,7 @@ void OO_AccumulatorImpl::openBlock(UT_String & styleAtts, UT_String & styleProps
 
 
 OO_Listener::OO_Listener (PD_Document * pDocument, IE_Exp_OpenWriter * pie, OO_ListenerImpl *pListenerImpl)
-   : PL_Listener (), m_pDocument(pDocument), m_pie(pie), m_pListenerImpl(pListenerImpl), m_bInBlock(false), m_bInSpan(false)
+   : PL_Listener (), m_pDocument(pDocument), m_pie(pie), m_pListenerImpl(pListenerImpl), m_bInBlock(false), m_bInSpan(false), m_bInHyperlink(false)
 {
 }
 
@@ -434,6 +484,31 @@ bool OO_Listener::populate(PL_StruxFmtHandle sfh,
 			}
 			return true;
 		}
+		case PX_ChangeRecord::PXT_InsertObject:
+		{
+			const PX_ChangeRecord_Object * pcro = static_cast<const PX_ChangeRecord_Object *> (pcr);
+			PT_AttrPropIndex api = pcr->getIndexAP();
+			switch (pcro->getObjectType())
+			{
+				case PTO_Hyperlink:
+				{
+					_closeSpan();
+					const PP_AttrProp* pAP = NULL;
+					m_pDocument->getAttrProp(api,&pAP);
+					const gchar* pValue = NULL;
+
+					if(pAP && pAP->getAttribute("xlink:href",pValue) && pValue) {
+						_openHyperlink(pAP);
+					} else {
+						_closeHyperlink();
+					}
+
+					return true;
+				}
+				default:
+					return true;
+			}
+        }
 	default:
 	   break;
 	}
@@ -452,8 +527,11 @@ bool OO_Listener::populateStrux(PL_StruxDocHandle sdh,
    case PTX_Block:
    {
       _closeSpan ();
+      _closeHyperlink();
       _openBlock(pcr->getIndexAP());
+      break;
    }
+   default: break;
    }
    
    return true;
@@ -467,12 +545,12 @@ bool OO_Listener::change(PL_StruxFmtHandle sfh,
 }
    
 bool OO_Listener::insertStrux(PL_StruxFmtHandle sfh,
-				      const PX_ChangeRecord * pcr,
-				      PL_StruxDocHandle sdh,
-				      PL_ListenerId lid,
-				      void (* pfnBindHandles)(PL_StruxDocHandle sdhNew,
-							      PL_ListenerId lid,
-							      PL_StruxFmtHandle sfhNew))
+			      const PX_ChangeRecord * pcr,
+			      PL_StruxDocHandle sdh,
+			      PL_ListenerId lid,
+			      void (* pfnBindHandles)(PL_StruxDocHandle sdhNew,
+						      PL_ListenerId lid,
+						      PL_StruxFmtHandle sfhNew))
 {
    UT_ASSERT_NOT_REACHED();
    return true;
@@ -486,6 +564,7 @@ bool OO_Listener::signal(UT_uint32 iSignal)
 
 void OO_Listener::endDocument()
 {
+   _closeHyperlink();
    _closeBlock();
 }
 
@@ -501,19 +580,23 @@ void OO_Listener::_openBlock (PT_AttrPropIndex api)
 
 	if (bHaveProp && pAP)
 	{
-		UT_UTF8String sa, pa, f;
+		UT_UTF8String sa, pa, f, escape;
 		OO_StylesWriter::map(pAP, sa, pa, f);
 
-		const XML_Char * szStyle = NULL;
+		const gchar * szStyle = NULL;
 		pAP->getAttribute("style", szStyle);
 
 		if (szStyle && pa.size())
 		{
 			// custom properties, prepare to derive style
-			sa += UT_UTF8String_sprintf("style:parent-style-name=\"%s\" ", szStyle);
+			escape = szStyle;
+			sa += UT_UTF8String_sprintf("style:parent-style-name=\"%s\" ", escape.escapeXML().utf8_str());
 		}
 		else if (szStyle)
-			sa += UT_UTF8String_sprintf("text:style-name=\"%s\" ", szStyle);
+		{
+			escape = szStyle;
+			sa += UT_UTF8String_sprintf("text:style-name=\"%s\" ", escape.escapeXML().utf8_str());
+		}
 	
 		// FIXME: hacky, but in sxw there is a distinction between heading- and text paragraphs
 		// we should probably also care about text:level here
@@ -572,6 +655,22 @@ void OO_Listener::_closeSpan()
    m_bInSpan = false;
 }
 
+void OO_Listener::_openHyperlink(const PP_AttrProp* pAP)
+{
+   if (m_bInHyperlink || !pAP)
+      return;
+
+   m_pListenerImpl->openHyperlink(pAP);
+   m_bInHyperlink = true;
+}
+
+void OO_Listener::_closeHyperlink()
+{
+   if (m_bInHyperlink)
+      m_pListenerImpl->closeHyperlink();
+   m_bInHyperlink = false;
+}
+
 
 /*****************************************************************************/
 /*****************************************************************************/
@@ -603,23 +702,23 @@ public:
 	"</office:document-meta>\n"
       };
 
-    writeToStream(meta, preamble, NrElements(preamble));
+    writeToStream(meta, preamble, G_N_ELEMENTS(preamble));
 
     UT_UTF8String meta_val, val;
 
     if (pDoc->getMetaDataProp(PD_META_KEY_DATE, meta_val) && meta_val.size()) {
       val = UT_UTF8String_sprintf("<dc:date>%s</dc:date>\n", meta_val.utf8_str());
-      gsf_output_write(meta, val.size(), reinterpret_cast<const guint8*>(val.utf8_str()));
+      oo_gsf_output_write(meta, val.size(), reinterpret_cast<const guint8*>(val.utf8_str()));
     }
     if (pDoc->getMetaDataProp(PD_META_KEY_LANGUAGE, meta_val) && meta_val.size()) {
-      val = UT_UTF8String_sprintf("<dc:language>%s</dc:language>\n", meta_val.utf8_str());
-      gsf_output_write(meta, val.size(), reinterpret_cast<const guint8*>(val.utf8_str()));
+      val = UT_UTF8String_sprintf("<dc:language>%s</dc:language>\n", meta_val.escapeXML().utf8_str());
+      oo_gsf_output_write(meta, val.size(), reinterpret_cast<const guint8*>(val.utf8_str()));
     }
 
-    writeToStream(meta, postamble, NrElements(postamble));
+    writeToStream(meta, postamble, G_N_ELEMENTS(postamble));
+
+    oo_gsf_output_close(meta);
 
-    gsf_output_close(meta);
-    g_object_unref(G_OBJECT(meta));
     return true;
   }
 
@@ -654,10 +753,9 @@ public:
 	"</office:document-settings>"
       };
 
-    writeToStream (settings, preamble, NrElements(preamble));
+    writeToStream (settings, preamble, G_N_ELEMENTS(preamble));
 
-    gsf_output_close(settings);
-    g_object_unref(G_OBJECT(settings));
+    oo_gsf_output_close(settings);
 
     return true;
   }
@@ -691,14 +789,13 @@ public:
       {
 	// create individual pictures
 	UT_String name = UT_String_sprintf("IMG-%d.png", k);
-	GsfOutput * img = gsf_outfile_new_child(GSF_OUTFILE(pictures), name.c_str(), FALSE);
-	gsf_output_write(img, pByteBuf->getLength(),  pByteBuf->getPointer(0));
-	gsf_output_close(img);
-	g_object_unref(G_OBJECT(img));
+	GsfOutput * img = gsf_outfile_new_child(GSF_OUTFILE(pictures), name.c_str(), FALSE);	
+	oo_gsf_output_write(img, pByteBuf->getLength(),  pByteBuf->getPointer(0));
+
+	oo_gsf_output_close(img);
       }
 
-    gsf_output_close(pictures);
-    g_object_unref(G_OBJECT(pictures));
+    oo_gsf_output_close(pictures);
 
     return true;
   }
@@ -744,8 +841,7 @@ public:
 	"</manifest:manifest>\n"
       };
 
-
-    writeToStream (manifest, preamble, NrElements(preamble));
+    writeToStream (manifest, preamble, G_N_ELEMENTS(preamble));
 
     const char * szName;
     const char * szMimeType;
@@ -754,19 +850,17 @@ public:
       {
 	if (k == 0) {
 	  name = "<manifest:file-entry manifest:media-type='' manifest:full-path='Pictures/'/>\n";
-	  gsf_output_write(manifest, name.size(), reinterpret_cast<const guint8 *>(name.c_str()));
+	  oo_gsf_output_write(manifest, name.size(), reinterpret_cast<const guint8 *>(name.c_str()));
 	}
 	  
-	name = UT_String_sprintf("<manifest:file-entry manifest:media-type='%s' manifest:full-path='Pictures/IMG-%d.png'/>\n", k, szMimeType);
-	gsf_output_write (manifest, name.size(), reinterpret_cast<const guint8 *>(name.c_str()));
+	name = UT_String_sprintf("<manifest:file-entry manifest:media-type='%s' manifest:full-path='Pictures/IMG-%d.png'/>\n", szMimeType, k);
+	oo_gsf_output_write (manifest, name.size(), reinterpret_cast<const guint8 *>(name.c_str()));
       }
 
-    writeToStream (manifest, postamble, NrElements(postamble));
+    writeToStream (manifest, postamble, G_N_ELEMENTS(postamble));
 
-    gsf_output_close(manifest);
-    g_object_unref(G_OBJECT(manifest));
-    gsf_output_close(meta_inf);
-    g_object_unref(G_OBJECT(meta_inf));
+    oo_gsf_output_close(manifest);
+    oo_gsf_output_close(meta_inf);
 
     return true;
   }
@@ -780,108 +874,105 @@ private:
 
 bool OO_StylesWriter::writeStyles(PD_Document * pDoc, GsfOutfile * oo, OO_StylesContainer & stylesContainer)
 {
-     GsfOutput * styleStream = gsf_outfile_new_child (oo, "styles.xml", FALSE);
-     
-     static const char * const preamble [] = 
-	{	   
-	   "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n",
-	   "<!DOCTYPE office:document-styles PUBLIC \"-//OpenOffice.org//DTD OfficeDocument 1.0//EN\" \"office.dtd\">\n",
-	   "<office:document-styles xmlns:office=\"http://openoffice.org/2000/office\" xmlns:style=\"http://openoffice.org/2000/style\" xmlns:text=\"http://openoffice.org/2000/text\" xmlns:table=\"http://openoffice.org/2000/table\" xmlns:draw=\"http://openoffice.org/2000/drawing\" xmlns:fo=\"http://www.w3.org/1999/XSL/Format\" xmlns:xlink=\"http://www.w3.org/1999/xlink\" xmlns:number=\"http://openoffice.org/2000/datastyle\" xmlns:svg=\"http://www.w3.org/2000/svg\" xmlns:chart=\"http://openoffice.org/2000/chart\" xmlns:dr3d=\"http://openoffice.org/2000/dr3d\" xmlns:math=\"http://www.w3.org/1998/Math/MathML\" xmlns:form=\"http://openoffice.org/2000/form\" xmlns:script=\"http://openoffice.org/2000/script\" office:version=\"1.0\">\n"
-	};
-
-     static const char * const midsection [] = 
-	{	   
-	   "<office:styles>\n",
-	   "<style:default-style style:family=\"graphics\">\n",
-	   "<style:properties draw:start-line-spacing-horizontal=\"0.283cm\" draw:start-line-spacing-vertical=\"0.283cm\" draw:end-line-spacing-horizontal=\"0.283cm\" draw:end-line-spacing-vertical=\"0.283cm\" fo:color=\"#000000\" style:font-name=\"Nimbus Roman No9 L\" fo:font-size=\"12pt\" fo:language=\"en\" fo:country=\"US\" style:font-name-asian=\"HG Mincho Light J\" style:font-size-asian=\"12pt\" style:language-asian=\"none\" style:country-asian=\"none\" style:font-name-complex=\"Arial Unicode MS\" style:font-size-complex=\"12pt\" style:language-complex=\"none\" style:country-complex=\"none\" style:text-autospace=\"ideograph-alpha\" style:punctuation-wrap=\"simple\" style:line-break=\"strict\">\n",
-	   "<style:tab-stops/>\n",
-	   "</style:properties>\n",
-	   "</style:default-style>\n",
-	   "<style:default-style style:family=\"paragraph\">\n",
-	   "<style:properties fo:color=\"#000000\" style:font-name=\"Nimbus Roman No9 L\" fo:font-size=\"12pt\" fo:language=\"en\" fo:country=\"US\" style:font-name-asian=\"HG Mincho Light J\" style:font-size-asian=\"12pt\" style:language-asian=\"none\" style:country-asian=\"none\" style:font-name-complex=\"Arial Unicode MS\" style:font-size-complex=\"12pt\" style:language-complex=\"none\" style:country-complex=\"none\" fo:hyphenate=\"false\" fo:hyphenation-remain-char-count=\"2\" fo:hyphenation-push-char-count=\"2\" fo:hyphenation-ladder-count=\"no-limit\" style:text-autospace=\"ideograph-alpha\" style:punctuation-wrap=\"hanging\" style:line-break=\"strict\" style:tab-stop-distance=\"2.205cm\"/>\n",
-	   "</style:default-style>\n"
-	};
-
-	UT_UTF8String styles;
-	const PD_Style * pStyle = NULL;
-	UT_GenericVector<PD_Style *> vecStyles;
-	pDoc->getAllUsedStyles(&vecStyles);
-	UT_uint32 k = 0;
-	UT_UTF8String styleAtts, propAtts, font;
-	for (k=0; k < vecStyles.getItemCount(); k++)
+  GsfOutput * styleStream = gsf_outfile_new_child (oo, "styles.xml", FALSE);
+  
+  static const char * const preamble [] = 
+    {	   
+      "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n",
+      "<!DOCTYPE office:document-styles PUBLIC \"-//OpenOffice.org//DTD OfficeDocument 1.0//EN\" \"office.dtd\">\n",
+      "<office:document-styles xmlns:office=\"http://openoffice.org/2000/office\" xmlns:style=\"http://openoffice.org/2000/style\" xmlns:text=\"http://openoffice.org/2000/text\" xmlns:table=\"http://openoffice.org/2000/table\" xmlns:draw=\"http://openoffice.org/2000/drawing\" xmlns:fo=\"http://www.w3.org/1999/XSL/Format\" xmlns:xlink=\"http://www.w3.org/1999/xlink\" xmlns:number=\"http://openoffice.org/2000/datastyle\" xmlns:svg=\"http://www.w3.org/2000/svg\" xmlns:chart=\"http://openoffice.org/2000/chart\" xmlns:dr3d=\"http://openoffice.org/2000/dr3d\" xmlns:math=\"http://www.w3.org/1998/Math/MathML\" xmlns:form=\"http://openoffice.org/2000/form\" xmlns:script=\"http://openoffice.org/2000/script\" office:version=\"1.0\">\n"
+    };
+  
+  static const char * const midsection [] = 
+    {	   
+      "<office:styles>\n",
+      "<style:default-style style:family=\"graphics\">\n",
+      "<style:properties draw:start-line-spacing-horizontal=\"0.283cm\" draw:start-line-spacing-vertical=\"0.283cm\" draw:end-line-spacing-horizontal=\"0.283cm\" draw:end-line-spacing-vertical=\"0.283cm\" fo:color=\"#000000\" style:font-name=\"Nimbus Roman No9 L\" fo:font-size=\"12pt\" fo:language=\"en\" fo:country=\"US\" style:font-name-asian=\"HG Mincho Light J\" style:font-size-asian=\"12pt\" style:language-asian=\"none\" style:country-asian=\"none\" style:font-name-complex=\"Arial Unicode MS\" style:font-size-complex=\"12pt\" style:language-complex=\"none\" style:country-complex=\"none\" style:text-autospace=\"ideograph-alpha\" style:punctuation-wrap=\"simple\" style:line-break=\"strict\">\n",
+      "<style:tab-stops/>\n",
+      "</style:properties>\n",
+      "</style:default-style>\n",
+      "<style:default-style style:family=\"paragraph\">\n",
+      "<style:properties fo:color=\"#000000\" style:font-name=\"Nimbus Roman No9 L\" fo:font-size=\"12pt\" fo:language=\"en\" fo:country=\"US\" style:font-name-asian=\"HG Mincho Light J\" style:font-size-asian=\"12pt\" style:language-asian=\"none\" style:country-asian=\"none\" style:font-name-complex=\"Arial Unicode MS\" style:font-size-complex=\"12pt\" style:language-complex=\"none\" style:country-complex=\"none\" fo:hyphenate=\"false\" fo:hyphenation-remain-char-count=\"2\" fo:hyphenation-push-char-count=\"2\" fo:hyphenation-ladder-count=\"no-limit\" style:text-autospace=\"ideograph-alpha\" style:punctuation-wrap=\"hanging\" style:line-break=\"strict\" style:tab-stop-distance=\"2.205cm\"/>\n",
+      "</style:default-style>\n"
+    };
+  
+  UT_UTF8String styles;
+  const PD_Style * pStyle = NULL;
+  UT_GenericVector<PD_Style *> vecStyles;
+  pDoc->getAllUsedStyles(&vecStyles);
+  UT_uint32 k = 0;
+  UT_UTF8String styleAtts, propAtts, font;
+  for (k=0; k < vecStyles.getItemCount(); k++)
+    {
+      pStyle = vecStyles.getNthItem(k);
+      PT_AttrPropIndex api = pStyle->getIndexAP();
+      const PP_AttrProp * pAP = NULL;
+      bool bHaveProp = pDoc->getAttrProp (api, &pAP);
+      
+      if (bHaveProp && pAP) 
 	{
-		pStyle = vecStyles.getNthItem(k);
-		PT_AttrPropIndex api = pStyle->getIndexAP();
-		const PP_AttrProp * pAP = NULL;
-		bool bHaveProp = pDoc->getAttrProp (api, &pAP);
-		
-		if (bHaveProp && pAP) 
-		{
-			OO_StylesWriter::map(pAP, styleAtts, propAtts, font);
-
-			styles += "<style:style " + styleAtts + ">\n";
-			styles += "<style:properties " + propAtts + "/>\n";
-			styles += "</style:style>\n";
-		}
-
-		if (font.size())
-		{
-			UT_String f = font.utf8_str();
-			stylesContainer.addFont(f);
-			font.clear();
-		}
+	  OO_StylesWriter::map(pAP, styleAtts, propAtts, font);
+	  
+	  styles += "<style:style " + styleAtts + ">\n";
+	  styles += "<style:properties " + propAtts + "/>\n";
+	  styles += "</style:style>\n";
 	}
-
-	static const char * const postamble [] = 
+      
+      if (font.size())
 	{
-	   "<text:outline-style>\n",
-	   "<text:outline-level-style text:level=\"1\" style:num-format=\"\"/>\n",
-	   "<text:outline-level-style text:level=\"2\" style:num-format=\"\"/>\n",
-	   "<text:outline-level-style text:level=\"3\" style:num-format=\"\"/>\n",
-	   "<text:outline-level-style text:level=\"4\" style:num-format=\"\"/>\n",
-	   "<text:outline-level-style text:level=\"5\" style:num-format=\"\"/>\n",
-	   "<text:outline-level-style text:level=\"6\" style:num-format=\"\"/>\n",
-	   "<text:outline-level-style text:level=\"7\" style:num-format=\"\"/>\n",
-	   "<text:outline-level-style text:level=\"8\" style:num-format=\"\"/>\n",
-	   "<text:outline-level-style text:level=\"9\" style:num-format=\"\"/>\n",
-	   "<text:outline-level-style text:level=\"10\" style:num-format=\"\"/>\n",
-	   "</text:outline-style>\n",
-	   "<text:footnotes-configuration style:num-format=\"1\" text:start-value=\"0\" text:footnotes-position=\"page\" text:start-numbering-at=\"document\"/>\n",
-	   "<text:endnotes-configuration style:num-format=\"i\" text:start-value=\"0\"/>\n",
-	   "<text:linenumbering-configuration text:number-lines=\"false\" text:offset=\"0.499cm\" style:num-format=\"1\" text:number-position=\"left\" text:increment=\"5\"/>\n",
-	   "</office:styles>\n",
-	   "<office:automatic-styles>\n",
-	   "<style:page-master style:name=\"pm1\">\n",
-	   "<style:properties fo:page-width=\"21.59cm\" fo:page-height=\"27.94cm\" style:num-format=\"1\" style:print-orientation=\"portrait\" fo:margin-top=\"2.54cm\" fo:margin-bottom=\"2.54cm\" fo:margin-left=\"3.175cm\" fo:margin-right=\"3.175cm\" style:footnote-max-height=\"0cm\">\n",
-	   "<style:footnote-sep style:width=\"0.018cm\" style:distance-before-sep=\"0.101cm\" style:distance-after-sep=\"0.101cm\" style:adjustment=\"left\" style:rel-width=\"25%\" style:color=\"#000000\"/>\n",
-	   "</style:properties>\n",
-	   "<style:header-style/>\n",
-	   "<style:footer-style/>\n",
-	   "</style:page-master>\n",
-	   "</office:automatic-styles>\n",
-	   "<office:master-styles>\n",
-	   "<style:master-page style:name=\"Standard\" style:page-master-name=\"pm1\"/>\n",
-	   "</office:master-styles>\n",
-	   "</office:document-styles>\n"
-      };
-     
-
-	writeToStream(styleStream, preamble, NrElements(preamble));
-
-	UT_UTF8String fontDecls = "<office:font-decls>\n";
-	OO_StylesWriter::addFontDecls(fontDecls, stylesContainer);
-	fontDecls += "</office:font-decls>\n";
-    writeUTF8String(styleStream, fontDecls.utf8_str());
-
-    writeToStream(styleStream, midsection, NrElements(midsection));
-    writeUTF8String(styleStream, styles.utf8_str());
-    writeToStream(styleStream, postamble, NrElements(postamble));
-     
-     
-    gsf_output_close(styleStream);
-    g_object_unref(G_OBJECT(styleStream));
-     
-    return true;
+	  UT_String f = font.utf8_str();
+	  stylesContainer.addFont(f);
+	  font.clear();
+	}
+    }
+  
+  static const char * const postamble [] = 
+    {
+      "<text:outline-style>\n",
+      "<text:outline-level-style text:level=\"1\" style:num-format=\"\"/>\n",
+      "<text:outline-level-style text:level=\"2\" style:num-format=\"\"/>\n",
+      "<text:outline-level-style text:level=\"3\" style:num-format=\"\"/>\n",
+      "<text:outline-level-style text:level=\"4\" style:num-format=\"\"/>\n",
+      "<text:outline-level-style text:level=\"5\" style:num-format=\"\"/>\n",
+      "<text:outline-level-style text:level=\"6\" style:num-format=\"\"/>\n",
+      "<text:outline-level-style text:level=\"7\" style:num-format=\"\"/>\n",
+      "<text:outline-level-style text:level=\"8\" style:num-format=\"\"/>\n",
+      "<text:outline-level-style text:level=\"9\" style:num-format=\"\"/>\n",
+      "<text:outline-level-style text:level=\"10\" style:num-format=\"\"/>\n",
+      "</text:outline-style>\n",
+      "<text:footnotes-configuration style:num-format=\"1\" text:start-value=\"0\" text:footnotes-position=\"page\" text:start-numbering-at=\"document\"/>\n",
+      "<text:endnotes-configuration style:num-format=\"i\" text:start-value=\"0\"/>\n",
+      "<text:linenumbering-configuration text:number-lines=\"false\" text:offset=\"0.499cm\" style:num-format=\"1\" text:number-position=\"left\" text:increment=\"5\"/>\n",
+      "</office:styles>\n",
+      "<office:automatic-styles>\n",
+      "<style:page-master style:name=\"pm1\">\n",
+      "<style:properties fo:page-width=\"21.59cm\" fo:page-height=\"27.94cm\" style:num-format=\"1\" style:print-orientation=\"portrait\" fo:margin-top=\"2.54cm\" fo:margin-bottom=\"2.54cm\" fo:margin-left=\"3.175cm\" fo:margin-right=\"3.175cm\" style:footnote-max-height=\"0cm\">\n",
+      "<style:footnote-sep style:width=\"0.018cm\" style:distance-before-sep=\"0.101cm\" style:distance-after-sep=\"0.101cm\" style:adjustment=\"left\" style:rel-width=\"25%\" style:color=\"#000000\"/>\n",
+      "</style:properties>\n",
+      "<style:header-style/>\n",
+      "<style:footer-style/>\n",
+      "</style:page-master>\n",
+      "</office:automatic-styles>\n",
+      "<office:master-styles>\n",
+      "<style:master-page style:name=\"Standard\" style:page-master-name=\"pm1\"/>\n",
+      "</office:master-styles>\n",
+      "</office:document-styles>\n"
+    };
+  
+  writeToStream(styleStream, preamble, G_N_ELEMENTS(preamble));
+  
+  UT_UTF8String fontDecls = "<office:font-decls>\n";
+  OO_StylesWriter::addFontDecls(fontDecls, stylesContainer);
+  fontDecls += "</office:font-decls>\n";
+  writeUTF8String(styleStream, fontDecls.utf8_str());
+  
+  writeToStream(styleStream, midsection, G_N_ELEMENTS(midsection));
+  writeUTF8String(styleStream, styles.utf8_str());
+  writeToStream(styleStream, postamble, G_N_ELEMENTS(postamble));
+  
+  oo_gsf_output_close(styleStream);
+  
+  return true;
 }
 
 void OO_StylesWriter::addFontDecls(UT_UTF8String & buffer, OO_StylesContainer & stylesContainer)
@@ -890,39 +981,49 @@ void OO_StylesWriter::addFontDecls(UT_UTF8String & buffer, OO_StylesContainer &
 	for (int i = 0; i < vecFonts->size(); i++) {
 		// FIXME ATM only variable width fonts
 		// check here by using pango?
-		const XML_Char * pitch = "variable";
+		const gchar * pitch = "variable";
 		const UT_String * font = vecFonts->getNthItem(i);
-		buffer += UT_UTF8String_sprintf("<style:font-decl style:name=\"%s\" fo:font-family=\"'%s'\" style:font-pitch=\"%s\"/>", 
-										font->c_str(), font->c_str(), pitch);
+		buffer += UT_UTF8String_sprintf("<style:font-decl style:name=\"%s\" fo:font-family=\"'%s'\" style:font-pitch=\"%s\"/>\n", 
+						font->c_str(), font->c_str(), pitch);
 	}
-	FREEP(vecFonts);
+	DELETEP(vecFonts);
 }
 
 void OO_StylesWriter::map(const PP_AttrProp * pAP, UT_UTF8String & styleAtts, UT_UTF8String & propAtts, UT_UTF8String & font) 
 {		
-	const XML_Char * szValue = NULL;
+	UT_UTF8String escape;
+	const gchar * szValue = NULL;
 	styleAtts.clear();
 	propAtts.clear();
 
 	// Attributes
-	if (pAP->getAttribute("name", szValue)) 
-		styleAtts += UT_UTF8String_sprintf("style:name=\"%s\" ", szValue);
+	if (pAP->getAttribute("name", szValue))
+	{ 
+		escape = szValue;
+		styleAtts += UT_UTF8String_sprintf("style:name=\"%s\" ", escape.escapeXML().utf8_str());
+	}
 
 	if (pAP->getAttribute("type", szValue)) 
-		if (!UT_strcmp(szValue, "P"))
+		if (!strcmp(szValue, "P"))
 		{
 			styleAtts += UT_UTF8String_sprintf("style:family=\"paragraph\" ");
 			styleAtts += UT_UTF8String_sprintf("style:class=\"text\" ");
 		}
 
 	if (pAP->getAttribute("basedon", szValue)) 
-		styleAtts += UT_UTF8String_sprintf("style:parent-style-name=\"%s\" ", szValue);
+	{
+		escape = szValue;
+		styleAtts += UT_UTF8String_sprintf("style:parent-style-name=\"%s\" ", escape.escapeXML().utf8_str());
+	}
 	
 	if (pAP->getAttribute("followedby", szValue)) 
 	{
 		// ignore, current style is default
-		if (UT_strcmp(szValue, "Current Settings"))
-			styleAtts += UT_UTF8String_sprintf("style:next-style-name=\"%s\" ", szValue);
+		if (strcmp(szValue, "Current Settings"))
+		{
+			escape = szValue;
+			styleAtts += UT_UTF8String_sprintf("style:next-style-name=\"%s\" ", escape.escapeXML().utf8_str());
+		}
 	}
 
 	// Properties
@@ -938,7 +1039,7 @@ void OO_StylesWriter::map(const PP_AttrProp * pAP, UT_UTF8String & styleAtts, UT
 	}
 
 	if (pAP->getProperty("dom-dir", szValue)) //:ltr; 
-		if (!UT_strcmp(szValue, "rtl"))
+		if (!strcmp(szValue, "rtl"))
 		{
 			// FIXME some of these parameters are mentioned more than once
 			propAtts += UT_UTF8String_sprintf("fo:text-align", "end");
@@ -1015,11 +1116,11 @@ void OO_StylesWriter::map(const PP_AttrProp * pAP, UT_UTF8String & styleAtts, UT
 
 	if (pAP->getProperty("text-align", szValue)) //left center right -> start end left right center justify
 	{
-		if (UT_strcmp(szValue, "left")) // skip default
+		if (strcmp(szValue, "left")) // skip default
 		{
 			propAtts += UT_UTF8String_sprintf("style:justify-single-word=\"false\" ");
 
-			if (!UT_strcmp(szValue, "right"))
+			if (!strcmp(szValue, "right"))
 				propAtts += UT_UTF8String_sprintf("fo:text-align=\"end\" ");
 			else
 				propAtts += UT_UTF8String_sprintf("fo:text-align=\"%s\" ", szValue);
@@ -1046,9 +1147,9 @@ void OO_StylesWriter::map(const PP_AttrProp * pAP, UT_UTF8String & styleAtts, UT
 
 	if (pAP->getProperty("text-position", szValue))
 	{
-		if (!UT_strcmp(szValue, "superscript"))
+		if (!strcmp(szValue, "superscript"))
 			propAtts += "style:text-position=\"super 58%\" ";
-		else if (!UT_strcmp(szValue, "subscript"))
+		else if (!strcmp(szValue, "subscript"))
 			propAtts += "style:text-position=\"sub 58%\" ";
 	}
 
@@ -1070,74 +1171,84 @@ IE_Exp_OpenWriter::~IE_Exp_OpenWriter()
 {
 }
 
+#define SXW_MIMETYPE "application/vnd.sun.xml.writer"
+
 /*!
  * This writes out our AbiWord file as an OpenOffice
  * compound document
  */
 UT_Error IE_Exp_OpenWriter::_writeDocument(void)
 {
+  UT_return_val_if_fail (getFp(), UT_ERROR);
+  m_oo = GSF_OUTFILE (gsf_outfile_zip_new (getFp(), NULL));
   UT_return_val_if_fail(m_oo, UT_ERROR);
-  
+
+  {
+    GsfOutput * mimetype = gsf_outfile_new_child (m_oo, "mimetype", FALSE);
+    if (!mimetype)
+      {
+	oo_gsf_output_close(GSF_OUTPUT(m_oo));
+	return UT_ERROR;
+      }
+
+    oo_gsf_output_write(mimetype, strlen(SXW_MIMETYPE), (const guint8 *)SXW_MIMETYPE);
+    oo_gsf_output_close(mimetype);
+  }
+
   if (!OO_MetaDataWriter::writeMetaData(getDoc(), m_oo))
-    return UT_ERROR;
+    {
+      oo_gsf_output_close(GSF_OUTPUT(m_oo));
+      return UT_ERROR;
+    }
 
   if (!OO_SettingsWriter::writeSettings(getDoc(), m_oo))
-    return UT_ERROR;
+    {
+      oo_gsf_output_close(GSF_OUTPUT(m_oo));
+      return UT_ERROR;
+    }
 
   if (!OO_PicturesWriter::writePictures(getDoc(), m_oo))
-    return UT_ERROR;
+    {
+      oo_gsf_output_close(GSF_OUTPUT(m_oo));
+      return UT_ERROR;
+    }
 
   if (!OO_ManifestWriter::writeManifest(getDoc(), m_oo))
-    return UT_ERROR;
+    {
+      oo_gsf_output_close(GSF_OUTPUT(m_oo));
+      return UT_ERROR;
+    }
 
   OO_StylesContainer stylesContainer;
   OO_AccumulatorImpl accumulatorImpl(&stylesContainer);
   OO_Listener listener1(getDoc(), this, &accumulatorImpl);
   if (!getDoc()->tellListener(static_cast<PL_Listener *>(&listener1)))
-    return UT_ERROR;
+    {
+      oo_gsf_output_close(GSF_OUTPUT(m_oo));
+      return UT_ERROR;
+    }
 
   if (!OO_StylesWriter::writeStyles(getDoc(), m_oo, stylesContainer))
-    return UT_ERROR;
+    {
+      oo_gsf_output_close(GSF_OUTPUT(m_oo));
+      return UT_ERROR;
+    }
 
-  OO_WriterImpl writerImpl(m_oo, &stylesContainer);
-  OO_Listener listener2(getDoc(), this, &writerImpl);
-  if (!getDoc()->tellListener(static_cast<PL_Listener *>(&listener2)))
-    return UT_ERROR;
-  listener2.endDocument();
+  {
+    OO_WriterImpl writerImpl(m_oo, &stylesContainer);
+    OO_Listener listener2(getDoc(), this, &writerImpl);
+    if (!getDoc()->tellListener(static_cast<PL_Listener *>(&listener2)))
+      {
+	oo_gsf_output_close(GSF_OUTPUT(m_oo));
+	return UT_ERROR;
+      }
 
+    listener2.endDocument();
+  }
+
+  oo_gsf_output_close(GSF_OUTPUT(m_oo));
   return UT_OK;
 }
 
-/*!
- * Open the underlying ZIP file for writing
- */
-bool IE_Exp_OpenWriter::_openFile(const char * szFilename)
-{
-  UT_return_val_if_fail(!m_oo, false);
-  
-  GsfOutput * sink = GSF_OUTPUT (gsf_output_stdio_new (szFilename, NULL));
-  if (!sink)
-    return false;
-
-  m_oo = GSF_OUTFILE (gsf_outfile_zip_new (sink, NULL));
-  g_object_unref(G_OBJECT(sink));
-
-  return (m_oo != 0);
-}
-
-/*!
- * Close our underlying ZIP file
- */
-bool IE_Exp_OpenWriter::_closeFile(void)
-{
-  if(m_oo) {
-    gsf_output_close(GSF_OUTPUT(m_oo));
-    g_object_unref(G_OBJECT(m_oo));
-    m_oo = 0;
-  }
-
-  return true;
-}
-
 /*****************************************************************************/
 /*****************************************************************************/
