@@ -55,7 +55,7 @@ IE_Exp_DocBook_Sniffer::IE_Exp_DocBook_Sniffer (const char * name) :
 
 bool IE_Exp_DocBook_Sniffer::recognizeSuffix(const char * szSuffix)
 {
-	return (!UT_stricmp(szSuffix,".dbk") ||!UT_stricmp(szSuffix,".xml"));
+	return (!g_ascii_strcasecmp(szSuffix,".dbk") ||!g_ascii_strcasecmp(szSuffix,".xml"));
 }
 
 UT_Error IE_Exp_DocBook_Sniffer::constructExporter(PD_Document * pDocument,
@@ -126,6 +126,40 @@ void IE_Exp_DocBook :: writeln (const char *txt)
 	iwrite (txt);
 	IE_Exp :: write ("\n");
 }
+
+/*****************************************************************/
+/*****************************************************************/
+
+/*!
+   removes the suffix from a string by searching backwards for the specified 
+   character delimiter. If the delimiter is not found, a copy of the original 
+   string is returned
+   
+   eg. _stripSuffix("/home/user/file.png, '.') returns "/home/user/file" 
+	   _stripSuffix("/home/user/foo_bar, '_') returns /home/user/foo 
+	   _stripSuffix("/home/user/file.png, '_') returns /home/user/file.png"
+   TODO: put this in UT_String somehow, it came from ie_exp_HTML.
+*/
+static char *_stripSuffix(const char* from, char delimiter)
+{
+	char * fremove_s = (char *)malloc(strlen(from)+1);
+	strcpy(fremove_s, from);   
+
+	char * p = fremove_s + strlen(fremove_s);
+	while ((p >= fremove_s) && (*p != delimiter))
+		p--;
+	
+	if (p >= fremove_s)
+	*p = '\0';
+	
+	return fremove_s;
+}
+
+static char * _stripSuffix(const UT_UTF8String & from, char delimiter)
+{
+  return _stripSuffix(from.utf8_str(), delimiter);
+}
+
 /*****************************************************************/
 /*****************************************************************/
 
@@ -333,21 +367,21 @@ void s_DocBook_Listener :: _closeSpan(void)
 	
 	if (pAP)
 	{
-		const XML_Char * szValue = 0;
+		const gchar * szValue = 0;
 
-		if (pAP->getProperty(static_cast<const XML_Char *>("text-position"), szValue))
+		if (pAP->getProperty(static_cast<const gchar *>("text-position"), szValue))
 		{
-			if (!UT_strcmp("superscript", szValue))
+			if (!strcmp("superscript", szValue))
 			{
 				_tagClose(TT_SUPERSCRIPT,"superscript",false,false,false);
 			}
-			else if (!UT_strcmp("subscript", szValue))
+			else if (!strcmp("subscript", szValue))
 			{
 				_tagClose(TT_SUBSCRIPT,"subscript",false,false,false);
 			}
 		}
 
-		if ((pAP->getProperty(static_cast<const XML_Char *>("font-style"), szValue)) && !UT_strcmp(szValue, "italic"))
+		if ((pAP->getProperty(static_cast<const gchar *>("font-style"), szValue)) && !strcmp(szValue, "italic"))
 		{
 			_tagClose(TT_EMPHASIS,"emphasis",false,false,false);
 		}
@@ -481,11 +515,11 @@ void s_DocBook_Listener :: _openParagraph(PT_AttrPropIndex api)
 
 	if (bHaveProp && pAP)
 	{
-		const XML_Char * szValue = 0;
+		const gchar * szValue = 0;
 
-		if (pAP->getAttribute(static_cast<const XML_Char *>(PT_STYLE_ATTRIBUTE_NAME), szValue))
+		if (pAP->getAttribute(static_cast<const gchar *>(PT_STYLE_ATTRIBUTE_NAME), szValue))
 		{	
-			if((!UT_strcmp(szValue, "Heading 1")) || (!UT_strcmp(szValue, "Numbered Heading 1")))
+			if((!strcmp(szValue, "Heading 1")) || (!strcmp(szValue, "Numbered Heading 1")))
 			{
 				// <p style="Heading 1"> ...
 				_closeChapterTitle();
@@ -499,14 +533,14 @@ void s_DocBook_Listener :: _openParagraph(PT_AttrPropIndex api)
 				}
 				return;
 			}
-			else if((!UT_strcmp(szValue, "Heading 2")) || (!UT_strcmp(szValue, "Numbered Heading 2")))
+			else if((!strcmp(szValue, "Heading 2")) || (!strcmp(szValue, "Numbered Heading 2")))
 			{
 				// <p style="Heading 2"> ...
 				_closeChapterTitle();
 
 				if((!_inSectionStrux()) && !m_bInTitle)
 				{
-					_openSection(api, !UT_strcmp(szValue, m_sParentStyle.utf8_str()) ? m_iSectionDepth : 2, szValue);
+					_openSection(api, !strcmp(szValue, m_sParentStyle.utf8_str()) ? m_iSectionDepth : 2, szValue);
 					m_sParentStyle = szValue;  //don't coalesce runs inside of a title
 				}
 				else if(!m_bInTitle)
@@ -517,14 +551,14 @@ void s_DocBook_Listener :: _openParagraph(PT_AttrPropIndex api)
 				}
 				return;
 			}
-			else if((!UT_strcmp(szValue, "Heading 3")) || (!UT_strcmp(szValue, "Numbered Heading 3")))
+			else if((!strcmp(szValue, "Heading 3")) || (!strcmp(szValue, "Numbered Heading 3")))
 			{
 				// <p style="Heading 3"> ...
 				_closeChapterTitle();
 
 				if(!_inSectionStrux() && !m_bInTitle)
 				{
-					_openSection(api, !UT_strcmp(szValue, m_sParentStyle.utf8_str()) ? m_iSectionDepth : 3, szValue);
+					_openSection(api, !strcmp(szValue, m_sParentStyle.utf8_str()) ? m_iSectionDepth : 3, szValue);
 					m_sParentStyle = szValue;
 				}
 				else
@@ -535,14 +569,14 @@ void s_DocBook_Listener :: _openParagraph(PT_AttrPropIndex api)
 				}
 				return;
 			}
-			else if(!UT_strcmp(szValue, "Heading 4")) 
+			else if(!strcmp(szValue, "Heading 4")) 
 			{
 				// <p style="Heading 4"> ...
 				_closeChapterTitle();
 
 				if(!_inSectionStrux() && !m_bInTitle)
 				{
-					_openSection(api, !UT_strcmp(szValue, m_sParentStyle.utf8_str()) ? m_iSectionDepth : 4, szValue);
+					_openSection(api, !strcmp(szValue, m_sParentStyle.utf8_str()) ? m_iSectionDepth : 4, szValue);
 					m_sParentStyle = szValue;
 				}
 				else
@@ -553,7 +587,7 @@ void s_DocBook_Listener :: _openParagraph(PT_AttrPropIndex api)
 				}
 				return;
 			}
-			else if (!UT_strcmp (szValue, "Chapter Heading"))
+			else if (!strcmp (szValue, "Chapter Heading"))
 			{
 				if(!_inSectionStrux() && !m_bInTitle)
 				{
@@ -568,7 +602,7 @@ void s_DocBook_Listener :: _openParagraph(PT_AttrPropIndex api)
 				}
 				return;
 			}
-			else if (!UT_strcmp (szValue, "Section Heading"))
+			else if (!strcmp (szValue, "Section Heading"))
 			{
 				_closeChapterTitle();
 
@@ -585,7 +619,7 @@ void s_DocBook_Listener :: _openParagraph(PT_AttrPropIndex api)
 				}
 				return;
 			}
-			else if (!UT_strcmp (szValue, "Plain Text"))
+			else if (!strcmp (szValue, "Plain Text"))
 			{
 				_closeChapterTitle();
 
@@ -613,7 +647,7 @@ void s_DocBook_Listener :: _openParagraph(PT_AttrPropIndex api)
 				m_sParentStyle = szValue;
 				return;
 			}
-			else if (!UT_strcmp (szValue, "Normal"))
+			else if (!strcmp (szValue, "Normal"))
 			{
 				_closeChapterTitle();
 
@@ -652,7 +686,7 @@ void s_DocBook_Listener :: _openParagraph(PT_AttrPropIndex api)
 					condition = _getProps(api);
 					buf = "para";
 
-					if(UT_strcmp (szValue, m_sLastStyle.utf8_str()))  // not a coalescing run
+					if(strcmp (szValue, m_sLastStyle.utf8_str()))  // not a coalescing run
 					{
 						_openSection(api,m_iSectionDepth,szValue);
 						_closeSectionTitle(); //no title
@@ -750,7 +784,7 @@ void s_DocBook_Listener :: _openSection (PT_AttrPropIndex api, int sub, const UT
 	UT_UTF8String section = "section", escaped = "", condition = "";
 	const PP_AttrProp * pAP = NULL;
 	bool bHaveProp = m_pDocument->getAttrProp(api,&pAP);
-	const XML_Char * szValue = 0;
+	const gchar * szValue = 0;
 
 	if(content.length())
 	{
@@ -760,7 +794,7 @@ void s_DocBook_Listener :: _openSection (PT_AttrPropIndex api, int sub, const UT
 		section += escaped;
 		section += "\"";
 	}
-	if(pAP && bHaveProp && (!UT_strcmp(content.utf8_str(),"abi-frame")))  //we only need this for frames
+	if(pAP && bHaveProp && (!strcmp(content.utf8_str(),"abi-frame")))  //we only need this for frames
 	{
 		condition = _getProps(api);
 		if(condition.length())
@@ -777,9 +811,6 @@ void s_DocBook_Listener :: _openSection (PT_AttrPropIndex api, int sub, const UT
 
 	if(pAP && bHaveProp && (pAP->getAttribute("strux-image-dataid", szValue)))
 	{
-		if(!UT_strcmp("",szValue))  // occurs with word documents
-			return;
-
 		_closeSectionTitle(); // no title
 		_handlePositionedImage(api);
 	}
@@ -808,21 +839,21 @@ void s_DocBook_Listener :: _openSpan(PT_AttrPropIndex api)
 	
 	if (bHaveProp && pAP)
 	{
-		const XML_Char * szValue = 0;
+		const gchar * szValue = 0;
 
-		if ((pAP->getAttribute(static_cast<const XML_Char *>("revision"), szValue)))
+		if ((pAP->getAttribute(static_cast<const gchar *>("revision"), szValue)))
 		{
 			buf += " revision=\"";
 			buf += szValue;
 			buf += "\"";
 		}
-		if ((pAP->getProperty(static_cast<const XML_Char *>("lang"), szValue)))
+		if ((pAP->getProperty(static_cast<const gchar *>("lang"), szValue)))
 		{
 			buf += " lang=\"";
 			buf += szValue;
 			buf += "\"";
 		}
-		if ((pAP->getProperty(static_cast<const XML_Char *>("font-weight"), szValue)) && !UT_strcmp(szValue, "bold"))
+		if ((pAP->getProperty(static_cast<const gchar *>("font-weight"), szValue)) && !strcmp(szValue, "bold"))
 		{
 			buf += " role=\"strong\"";
 		}
@@ -837,17 +868,17 @@ void s_DocBook_Listener :: _openSpan(PT_AttrPropIndex api)
 
 		_tagOpen(TT_PHRASE,buf,false,false,false);
 
-		if ((pAP->getProperty(static_cast<const XML_Char *>("font-style"), szValue)) && !UT_strcmp(szValue, "italic"))
+		if ((pAP->getProperty(static_cast<const gchar *>("font-style"), szValue)) && !strcmp(szValue, "italic"))
 		{
 			_tagOpen(TT_EMPHASIS,"emphasis",false,false,false);
 		}
-		if (pAP->getProperty(static_cast<const XML_Char *>("text-position"), szValue))
+		if (pAP->getProperty(static_cast<const gchar *>("text-position"), szValue))
 		{
-			if (!UT_strcmp("superscript", szValue))
+			if (!strcmp("superscript", szValue))
 			{
 				_tagOpen(TT_SUPERSCRIPT,"superscript",false,false,false);
 			}
-			else if (!UT_strcmp("subscript", szValue))
+			else if (!strcmp("subscript", szValue))
 			{
 				_tagOpen(TT_SUBSCRIPT,"subscript",false,false,false);
 			}
@@ -1267,7 +1298,7 @@ void s_DocBook_Listener::_handleDocument(void)
 	UT_UTF8String buf("book condition=\""), condition("");
 	PT_AttrPropIndex docApi = m_pDocument->getAttrPropIndex();
 	const PP_AttrProp * pDAP = NULL;
-	const XML_Char* szValue = 0;
+	const gchar* szValue = 0;
 
 	m_pDocument->getAttrProp (docApi, &pDAP);
 	UT_return_if_fail(pDAP);
@@ -1420,6 +1451,9 @@ void s_DocBook_Listener::_handleMetaData(void)
 
 		if(buf.length())
 		{
+			if(_tagTop() == TT_BOOKINFO)
+				_tagOpen(TT_KEYWORDSET,"keywordset");
+
 			_tagOpen(TT_KEYWORD,"keyword",false,true,false);
 			buf.escapeXML();
 			m_pie->write(buf.utf8_str());
@@ -1488,7 +1522,7 @@ void s_DocBook_Listener::_handleRevisions(void)
 void s_DocBook_Listener::_handleImage(PT_AttrPropIndex api)
 {
 	UT_UTF8String buf(""), escaped("");
-	const XML_Char* szValue = 0;
+	const gchar* szValue = 0;
 	const PP_AttrProp * pAP = NULL;
 	bool bHaveProp = m_pDocument->getAttrProp(api,&pAP);
 	UT_LocaleTransactor t(LC_NUMERIC, "C");
@@ -1509,7 +1543,7 @@ void s_DocBook_Listener::_handleImage(PT_AttrPropIndex api)
 	if(bHaveProp && pAP && pAP->getAttribute("dataid", szValue))
 	{
 		char* dataid = strdup(const_cast<char*>(szValue));
-		char * temp = _stripSuffix(UT_basename(szValue), '_');
+		char * temp = _stripSuffix(UT_go_basename(szValue), '_');
 		char * fstripped = _stripSuffix(temp, '.');
 		UT_UTF8String_sprintf(buf, "%s.png", fstripped);
 		m_utvDataIDs.push_back(dataid);
@@ -1538,7 +1572,7 @@ void s_DocBook_Listener::_handleImage(PT_AttrPropIndex api)
 
 		escaped.clear();
 		escaped = "imagedata fileref=\"";
-		escaped += UT_basename(m_pie->getFileName());
+		escaped += UT_go_basename(m_pie->getFileName());
 		escaped += "_data/";
 		escaped += buf.escapeXML();
 		escaped += "\" format=\"PNG\"";
@@ -1578,7 +1612,7 @@ void s_DocBook_Listener::_handleImage(PT_AttrPropIndex api)
 void s_DocBook_Listener::_handlePositionedImage(PT_AttrPropIndex api)
 {
 	UT_UTF8String buf(""), escaped("");
-	const XML_Char* szValue = 0;
+	const gchar* szValue = 0;
 	const PP_AttrProp * pAP = NULL;
 	bool bHaveProp = m_pDocument->getAttrProp(api,&pAP);
 	UT_LocaleTransactor t(LC_NUMERIC, "C");
@@ -1586,7 +1620,7 @@ void s_DocBook_Listener::_handlePositionedImage(PT_AttrPropIndex api)
 	if(bHaveProp && pAP && pAP->getAttribute("strux-image-dataid", szValue))
 	{
 		char* dataid = strdup(const_cast<char*>(szValue));
-		char * temp = _stripSuffix(UT_basename(szValue), '_');
+		char * temp = _stripSuffix(UT_go_basename(szValue), '_');
 		char * fstripped = _stripSuffix(temp, '.');
 		UT_UTF8String_sprintf(buf, "%s.png", fstripped);
 		m_utvDataIDs.push_back(dataid);
@@ -1615,7 +1649,7 @@ void s_DocBook_Listener::_handlePositionedImage(PT_AttrPropIndex api)
 
 		escaped.clear();
 		escaped = "imagedata fileref=\"";
-		escaped += UT_basename(m_pie->getFileName());
+		escaped += UT_go_basename(m_pie->getFileName());
 		escaped += "_data/";
 		escaped += buf.escapeXML();
 		escaped += "\" format=\"PNG\"";
@@ -1655,7 +1689,7 @@ void s_DocBook_Listener::_handlePositionedImage(PT_AttrPropIndex api)
 void s_DocBook_Listener::_handleMath(PT_AttrPropIndex api)
 {
 	UT_UTF8String buf(""), escaped(""), condition("");
-	const XML_Char* szValue = 0;
+	const gchar* szValue = 0;
 	const PP_AttrProp * pAP = NULL;
 	bool bHaveProp = m_pDocument->getAttrProp(api,&pAP);
 	UT_LocaleTransactor t(LC_NUMERIC, "C");
@@ -1684,7 +1718,7 @@ void s_DocBook_Listener::_handleMath(PT_AttrPropIndex api)
 		_tagOpen(TT_INLINEEQUATION,"inlineequation",false,false,false);
 
 		escaped = "graphic fileref=\"";
-		escaped += UT_basename(m_pie->getFileName());
+		escaped += UT_go_basename(m_pie->getFileName());
 		escaped += "_data/";
 		escaped += buf.escapeXML();
 		escaped += "\" format=\"PNG\"";
@@ -1743,7 +1777,7 @@ void s_DocBook_Listener::_handleMath(PT_AttrPropIndex api)
 void s_DocBook_Listener::_handleEmbedded(PT_AttrPropIndex api)
 {
 	UT_UTF8String buf(""), escaped(""), condition("");
-	const XML_Char* szValue = 0;
+	const gchar* szValue = 0;
 	const PP_AttrProp * pAP = NULL;
 	bool bHaveProp = m_pDocument->getAttrProp(api,&pAP);
 	UT_LocaleTransactor t(LC_NUMERIC, "C");
@@ -1774,7 +1808,7 @@ void s_DocBook_Listener::_handleEmbedded(PT_AttrPropIndex api)
 		_tagOpen(TT_IMAGEOBJECT,"imageobject",false,false,false);
 
 		escaped = "imagedata fileref=\"";
-		escaped += UT_basename(m_pie->getFileName());
+		escaped += UT_go_basename(m_pie->getFileName());
 		escaped += "_data/";
 		escaped += buf.escapeXML();
 		escaped += "\" format=\"PNG\"";
@@ -1818,14 +1852,16 @@ void s_DocBook_Listener::_handleField(const PX_ChangeRecord_Object * pcro, PT_At
 	UT_UTF8String buf(""), escaped(""), condition("");
 	const PP_AttrProp * pAP = NULL;
 	bool bHaveProp = m_pDocument->getAttrProp(api,&pAP);
-	const XML_Char * szValue = 0, * szStyle = 0;
+	const gchar * szValue = 0, * szStyle = 0;
 
 	if(!m_bInParagraph)
 		_openBlock(api,false);
 
+	m_pie->populateFields ();
+
 	if(bHaveProp && pAP && pAP->getAttribute ("type", szValue))
 	{
-		if (!UT_strcmp (szValue, "list_label"))
+		if (!strcmp (szValue, "list_label"))
 		{
 			_openList(api);
 			return;
@@ -1843,7 +1879,7 @@ void s_DocBook_Listener::_handleField(const PX_ChangeRecord_Object * pcro, PT_At
 			buf += condition.escapeXML();
 			buf += "\"";
 		}
-		if (!UT_strcmp (szValue, "endnote_anchor"))  //give the endnote <para> a unique id
+		if (!strcmp (szValue, "endnote_anchor"))  //give the endnote <para> a unique id
 		{
 			if(pAP->getAttribute("endnote-id", szStyle))
 			{
@@ -1860,7 +1896,7 @@ void s_DocBook_Listener::_handleField(const PX_ChangeRecord_Object * pcro, PT_At
 		_tagOpen(TT_PHRASE,buf,false,false,false);
 		buf.clear();
 
-		if (!UT_strcmp (szValue, "footnote_ref"))
+		if (!strcmp (szValue, "footnote_ref"))
 		{
 			buf = "footnoteref linkend=\"footnote-id-";
 
@@ -1876,7 +1912,7 @@ void s_DocBook_Listener::_handleField(const PX_ChangeRecord_Object * pcro, PT_At
 
 			_tagOpenClose(buf,true,false,false);
 		}
-		else if (!UT_strcmp (szValue, "endnote_ref"))
+		else if (!strcmp (szValue, "endnote_ref"))
 		{
 			buf = "xref linkend=\"endnote-id-";
 
@@ -1912,7 +1948,7 @@ void s_DocBook_Listener::_handleField(const PX_ChangeRecord_Object * pcro, PT_At
 void s_DocBook_Listener::_handleTOC(PT_AttrPropIndex api)
 {
 	UT_UTF8String buf(""), condition(""), content("toc");
-	const XML_Char* szValue = 0;
+	const gchar* szValue = 0;
 	const PP_AttrProp * pAP = NULL;
 	bool bHaveProp = m_pDocument->getAttrProp(api,&pAP);
 
@@ -1955,7 +1991,7 @@ void s_DocBook_Listener::_handleTOC(PT_AttrPropIndex api)
 void s_DocBook_Listener::_handleHyperlink(PT_AttrPropIndex api)
 {
 	UT_UTF8String buf(""), escaped("");
-	const XML_Char* szValue = 0;
+	const gchar* szValue = 0;
 	const PP_AttrProp * pAP = NULL;
 	bool bHaveProp = m_pDocument->getAttrProp(api,&pAP);
 
@@ -1996,13 +2032,13 @@ void s_DocBook_Listener::_handleHyperlink(PT_AttrPropIndex api)
 void s_DocBook_Listener::_handleBookmark(PT_AttrPropIndex api)
 {
 	UT_UTF8String buf(""), escaped("");
-	const XML_Char* szValue = 0;
+	const gchar* szValue = 0;
 	const PP_AttrProp * pAP = NULL;
 	bool bHaveProp = m_pDocument->getAttrProp(api,&pAP);
 
 	if(bHaveProp && pAP && pAP->getAttribute("type", szValue))
 	{
-		if (!UT_strcmp (szValue, "start") && pAP->getAttribute("name", szValue))
+		if (!strcmp (szValue, "start") && pAP->getAttribute("name", szValue))
 		{
 			buf = "anchor id=\"";
 			escaped = szValue;
@@ -2017,7 +2053,7 @@ void s_DocBook_Listener::_handleBookmark(PT_AttrPropIndex api)
 void s_DocBook_Listener::_handleHdrFtr(PT_AttrPropIndex api)
 {
 	UT_UTF8String buf("abi-");  //prefix the type to prevent confusion on import (e.g. if there's a 'footer' style)
-	const XML_Char* szValue = 0;
+	const gchar* szValue = 0;
 	const PP_AttrProp * pAP = NULL;
 	bool bHaveProp = m_pDocument->getAttrProp(api,&pAP);
 
@@ -2031,7 +2067,7 @@ void s_DocBook_Listener::_handleHdrFtr(PT_AttrPropIndex api)
 void s_DocBook_Listener::_handleFootnote(PT_AttrPropIndex api)
 {
 	UT_UTF8String buf("footnote id=\"footnote-id-");  //make it an NCNAME to prevent validation errors
-	const XML_Char* szValue = 0;
+	const gchar* szValue = 0;
 	const PP_AttrProp * pAP = NULL;
 	bool bHaveProp = m_pDocument->getAttrProp(api,&pAP);
 
@@ -2304,37 +2340,12 @@ UT_Error IE_Exp_DocBook::_writeDocument(void)
 /*****************************************************************/
 /*****************************************************************/
 
-/*!
-   removes the suffix from a string by searching backwards for the specified 
-   character delimiter. If the delimiter is not found, a copy of the original 
-   string is returned
-   
-   eg. _stripSuffix("/home/user/file.png, '.') returns "/home/user/file" 
-	   _stripSuffix("/home/user/foo_bar, '_') returns /home/user/foo 
-	   _stripSuffix("/home/user/file.png, '_') returns /home/user/file.png"
-   TODO: put this in UT_String somehow, it came from ie_exp_HTML.
-*/
-char *s_DocBook_Listener::_stripSuffix(const char* from, char delimiter)
-{
-	char * fremove_s = (char *)malloc(strlen(from)+1);
-	strcpy(fremove_s, from);   
-
-	char * p = fremove_s + strlen(fremove_s);
-	while ((p >= fremove_s) && (*p != delimiter))
-		p--;
-	
-	if (p >= fremove_s)
-	*p = '\0';
-	
-	return fremove_s;
-}
-
 const UT_UTF8String s_DocBook_Listener::_getProps(PT_AttrPropIndex api)
 {
 	UT_UTF8String buf("");
 	const PP_AttrProp * pAP = NULL;
 	bool bHaveProp = m_pDocument->getAttrProp (api, &pAP);
-	const XML_Char * szName = 0, * szValue = 0;
+	const gchar * szName = 0, * szValue = 0;
 
 	UT_return_val_if_fail(pAP && bHaveProp,"");
 
@@ -2363,7 +2374,7 @@ void s_DocBook_Listener::_handleDataItems(void)
 		UT_sint32 loc = -1;
 		for (UT_uint32 i = 0; i < m_utvDataIDs.getItemCount(); i++)
 		{
-			if(UT_strcmp(const_cast<char*>(reinterpret_cast<const char*>(m_utvDataIDs[i])), szName) == 0)
+			if(strcmp(const_cast<char*>(reinterpret_cast<const char*>(m_utvDataIDs[i])), szName) == 0)
 			{
 				loc = i;
 				break;
@@ -2372,43 +2383,34 @@ void s_DocBook_Listener::_handleDataItems(void)
 		
 		if(loc > -1)
 		{
-			FILE *fp;
-			UT_String fname; // EVIL EVIL bad hardcoded buffer size
+			UT_UTF8String fname;
 			
-			UT_String_sprintf(fname, "%s_data", m_pie->getFileName());
+			UT_UTF8String_sprintf(fname, "%s_data", m_pie->getFileName());
 			/* int result = */
-			m_pDocument->getApp()->makeDirectory(fname.c_str(), 0750);
+			UT_go_directory_create(fname.utf8_str(), 0750, NULL);
 			
-			if (!UT_strcmp(szMimeType, "image/svg+xml"))
-				UT_String_sprintf(fname, "%s/%s_%d.svg", fname.c_str(), szName, loc);
-			if (!UT_strcmp(szMimeType, "application/mathml+xml"))
-				UT_String_sprintf(fname, "%s/%s_%d.mathml", fname.c_str(), szName, loc);
+			if (!strcmp(szMimeType, "image/svg+xml"))
+				UT_UTF8String_sprintf(fname, "%s/%s_%d.svg", fname.utf8_str(), szName, loc);
+			if (!strcmp(szMimeType, "application/mathml+xml"))
+				UT_UTF8String_sprintf(fname, "%s/%s_%d.mathml", fname.utf8_str(), szName, loc);
 			else // PNG Image
 			{  
-				char * temp = _stripSuffix(UT_basename(szName), '_');
-				char * fstripped = _stripSuffix(temp, '.');
-				FREEP(temp);
-				UT_String_sprintf(fname, "%s/%s.png", fname.c_str(), fstripped);
-				FREEP(fstripped);
+			  char * temp = _stripSuffix(UT_go_basename(szName), '_');
+			  char * fstripped = _stripSuffix(temp, '.');
+			  FREEP(temp);
+			  UT_UTF8String_sprintf(fname, "%s/%s.png", fname.utf8_str(), fstripped);
+			  FREEP(fstripped);
 			}
 			
-			if (!UT_isRegularFile(fname.c_str()))
-			{
-				fp = fopen (fname.c_str(), "wb+");
 			
-				if(!fp)
-					continue;
+			GsfOutput *fp = UT_go_file_create (fname.utf8_str(), NULL);
 			
-				int cnt = 0, len = pByteBuf->getLength();
+			if(!fp)
+			  continue;
 			
-				while (cnt < len)
-				{
-					cnt += fwrite (pByteBuf->getPointer(cnt), 
-								 sizeof(UT_Byte), len-cnt, fp);
-				}
-			
-				fclose(fp);
-			}
+			gsf_output_write(fp, pByteBuf->getLength(), (const guint8*)pByteBuf->getPointer(0));			
+			gsf_output_close(fp);
+			g_object_unref (G_OBJECT (fp));
 		}
 	}
 }
