@@ -1,3 +1,5 @@
+/* -*- mode: C++; tab-width: 4; c-basic-offset: 4; -*- */
+
 /* AbiWord
  * Copyright (C) 1998 AbiSource, Inc.
  * 
@@ -42,19 +44,12 @@ int abi_plugin_register (XAP_ModuleInfo * mi)
 	{
 		m_impSniffer = new IE_Imp_DocBook_Sniffer (PLUGIN_NAME);
 	}
-	else
-	{
-		m_impSniffer->ref();
-	}
 
 	if (!m_expSniffer)
 	{
 		m_expSniffer = new IE_Exp_DocBook_Sniffer (PLUGIN_NAME);
 	}
-	else
-	{
-		m_expSniffer->ref();
-	}
+
 	IE_Imp::registerImporter (m_impSniffer);
 	IE_Exp::registerExporter (m_expSniffer);
 
@@ -80,16 +75,12 @@ int abi_plugin_unregister (XAP_ModuleInfo * mi)
 	UT_ASSERT (m_expSniffer);
 
 	IE_Imp::unregisterImporter (m_impSniffer);
-	if (!m_impSniffer->unref())
-	{
-		m_impSniffer = 0;
-	}
+	delete m_impSniffer;
+	m_impSniffer = 0;
 
 	IE_Exp::unregisterExporter (m_expSniffer);
-	if (!m_expSniffer->unref())
-	{
-		m_expSniffer = 0;
-	}
+	delete m_expSniffer;
+	m_expSniffer = 0;
 
 	return 1;
 }
