@@ -1,4 +1,5 @@
 /* -*- c-basic-offset: 4; tab-width: 4; indent-tabs-mode: t -*- */
+
 /* AbiWord SVG Graphic importer employing LibRSVG and GdkPixbuf
  * Copyright (C) 2002 Dom Lachowicz
  *
@@ -57,7 +58,7 @@ static void _write_flush(png_structp png_ptr) { } // Empty Fuction.
 
 //------------------------------------------------------------------------------------
 
-class ABI_EXPORT IE_RSVGGraphic : public IE_ImpGraphic
+class IE_RSVGGraphic : public IE_ImpGraphic
 {
 public:
 
@@ -94,25 +95,6 @@ public:
 		*ppfg = static_cast<FG_Graphic *>(pFGR);
 		return UT_OK;
 	}
-	
-	/*!
-	 * Convert the data contained in the file to into a PNG-based 
-	 * FG_Graphic type
-	 */
-	virtual UT_Error importGraphic(const char * szFilename, FG_Graphic ** ppfg)
-	{
-		UT_ByteBuf bb;
-
-		if ( bb.insertFromFile(0, szFilename) )
-			{
-				return importGraphic ( &bb, ppfg ) ;
-			}
-		else
-			{
-				*ppfg = 0;
-				return UT_ERROR;
-			}
-	}
   
   /*!
    * Convert an image byte buffer into a PNG byte buffer
@@ -281,7 +263,13 @@ private:
 
 //------------------------------------------------------------------------------------
 
-class ABI_EXPORT IE_RSVGGraphic_Sniffer : public IE_ImpGraphicSniffer
+// supported suffixes
+static IE_SuffixConfidence IE_RSVGGraphic_Sniffer__SuffixConfidence[] = {
+	{ "svg", 	UT_CONFIDENCE_PERFECT 	},
+	{ "", 	UT_CONFIDENCE_ZILCH 	}
+};
+
+class IE_RSVGGraphic_Sniffer : public IE_ImpGraphicSniffer
 {
 public:
 
@@ -294,6 +282,11 @@ public:
 	{
 	}
 
+  const IE_SuffixConfidence * getSuffixConfidence ()
+	{
+		return IE_RSVGGraphic_Sniffer__SuffixConfidence;
+	}
+
   virtual UT_Confidence_t recognizeContents(const char * szBuf, UT_uint32 iNum)
 	{
 		// todo: make me better
@@ -303,13 +296,18 @@ public:
 		return UT_CONFIDENCE_ZILCH;
 	}
 	
-	virtual UT_Confidence_t recognizeSuffix(const char * szSuffix)
+	virtual const IE_MimeConfidence * getMimeConfidence () 
 	{
-		if ( !UT_stricmp(szSuffix, ".svg") )
-			return UT_CONFIDENCE_PERFECT ;
-		return UT_CONFIDENCE_ZILCH;
+		/* Aliases once this plugin supports mime types (list courtesy of gdk-pixbuf)
+		image/svg+xml
+		image/svg
+		image/svg-xml
+		image/vnd.adobe.svg+xml
+		text/xml-svg
+		*/
+		return NULL;
 	}
-	
+
 	virtual bool getDlgLabels(const char ** pszDesc,
 							  const char ** pszSuffixList,
 							  IEGraphicFileType * ft)
@@ -351,11 +349,6 @@ int abi_plugin_register (XAP_ModuleInfo * mi)
       // create a new sniffer
       m_sniffer = new IE_RSVGGraphic_Sniffer ();
 	}
-    else
-	{
-		// else just reference the one we've got
-		m_sniffer->ref();
-	}
 
     IE_ImpGraphic::registerImporter ( m_sniffer );
 
@@ -374,9 +367,7 @@ int abi_plugin_unregister (XAP_ModuleInfo * mi)
     if ( m_sniffer )
 	{
 		IE_ImpGraphic::unregisterImporter ( m_sniffer );
-	}
-    if (!m_sniffer->unref())
-    {
+		delete m_sniffer;
 		m_sniffer = 0;
 	}
 
