@@ -1,3 +1,5 @@
+/* -*- mode: C++; tab-width: 4; c-basic-offset: 4; -*- */
+
 /* AbiWord
  * Copyright (C) 2001 AbiSource, Inc.
  * 
@@ -40,8 +42,8 @@
 #include "fl_AutoNum.h"
 #include "fp_PageSize.h"
 #include "ut_string_class.h"
-
 #include "ie_exp_KWord_1.h"
+#include "ie_impexp_KWord_1.h"
 #include "ut_locale.h"
 
 /*****************************************************************/
@@ -90,7 +92,7 @@ protected:
 	void				_openBlock(PT_AttrPropIndex api);
 	void				_openSection(PT_AttrPropIndex api);
 	void				_openSpan(PT_AttrPropIndex api, PT_BlockOffset pos, UT_uint32 len);
-	void                _writeMarginSize(PT_AttrPropIndex api, char * name);
+	void                _writeMarginSize(PT_AttrPropIndex api, const char * name);
 
 private:
 	PD_Document *		m_pDocument;
@@ -134,7 +136,7 @@ IE_Exp_KWord_1_Sniffer::IE_Exp_KWord_1_Sniffer (const char * name) :
 
 UT_Confidence_t IE_Exp_KWord_1_Sniffer::supportsMIME (const char * szMIME)
 {
-  if (UT_strcmp (IE_FileInfo::mapAlias (szMIME), IE_MIME_KWord) == 0)
+  if (strcmp (IE_FileInfo::mapAlias (szMIME), IE_MIMETYPE_KWord) == 0)
     {
       return UT_CONFIDENCE_GOOD;
     }
@@ -143,7 +145,7 @@ UT_Confidence_t IE_Exp_KWord_1_Sniffer::supportsMIME (const char * szMIME)
 
 bool IE_Exp_KWord_1_Sniffer::recognizeSuffix(const char * szSuffix)
 {
-	return (!UT_stricmp(szSuffix,".kwd"));
+	return (!g_ascii_strcasecmp(szSuffix,".kwd"));
 }
 
 UT_Error IE_Exp_KWord_1_Sniffer::constructExporter(PD_Document * pDocument,
@@ -301,11 +303,11 @@ bool s_KWord_1_Listener::populateStrux(PL_StruxDocHandle /*sdh*/,
 		const PP_AttrProp* pAP = NULL;
 		if (m_pDocument->getAttrProp(indexAP, &pAP) && pAP)
 		{
-			const XML_Char* pszSectionType = NULL;
+			const gchar* pszSectionType = NULL;
 			pAP->getAttribute("type", pszSectionType);
 			if (
 				!pszSectionType
-				|| (0 == UT_strcmp(pszSectionType, "doc"))
+				|| (0 == strcmp(pszSectionType, "doc"))
 				)
 			{
 				_openSection(pcr->getIndexAP());
@@ -476,10 +478,10 @@ abiPageSizeToKoPageFormat (fp_PageSize abi_page_size)
 	}
 }
 
-void s_KWord_1_Listener::_writeMarginSize(PT_AttrPropIndex api, char * name)
+void s_KWord_1_Listener::_writeMarginSize(PT_AttrPropIndex api, const char * name)
 {
-        UT_String buf;
-	const XML_Char * szValue;
+	UT_String buf;
+	const gchar * szValue;
 	const PP_AttrProp * pAP = NULL;
 	bool bHaveProp = m_pDocument->getAttrProp(api,&pAP);
 	UT_ASSERT((bHaveProp));
@@ -573,29 +575,23 @@ void s_KWord_1_Listener::_handleDataItems(void)
 
 	for (UT_uint32 k=0; (m_pDocument->enumDataItems(k,NULL,&szName,&pByteBuf,reinterpret_cast<const void**>(&szMimeType))); k++)
 	{	  	  
-	  FILE *fp;
-	  UT_String fname;	  
+	  UT_UTF8String fname;	  
 	  
-	  if (!UT_strcmp(szMimeType, "image/svg+xml"))
-	      UT_String_sprintf(fname, "%s-%d.svg", m_pie->getFileName(), k);
-	  if (!UT_strcmp(szMimeType, "application/mathml+xml"))
-	    UT_String_sprintf(fname, "%s-%d.mathml", m_pie->getFileName(), k);
+	  if (!strcmp(szMimeType, "image/svg+xml"))
+	      UT_UTF8String_sprintf(fname, "%s-%d.svg", m_pie->getFileName(), k);
+	  if (!strcmp(szMimeType, "application/mathml+xml"))
+	    UT_UTF8String_sprintf(fname, "%s-%d.mathml", m_pie->getFileName(), k);
 	  else // PNG Image
-	    UT_String_sprintf(fname, "%s-%d.png", m_pie->getFileName(), k);
+	    UT_UTF8String_sprintf(fname, "%s-%d.png", m_pie->getFileName(), k);
 	  
-	  fp = fopen (fname.c_str(), "wb+");
+	  GsfOutput *fp = UT_go_file_create(fname.utf8_str(), NULL);
 	  
 	  if(!fp)
 	    continue;
 	  
-	  int cnt = 0, len = pByteBuf->getLength();
-	  
-	  while (cnt < len)
-	    {
-	      cnt += fwrite (pByteBuf->getPointer(cnt), sizeof(UT_Byte), len-cnt, fp);
-	    }
-	  
-	  fclose(fp);
+	  gsf_output_write(fp, pByteBuf->getLength(), (const guint8*)pByteBuf->getPointer(0));
+	  gsf_output_close(fp);
+	  g_object_unref(G_OBJECT(fp));
 	}
 	
 	return;
@@ -704,7 +700,7 @@ void s_KWord_1_Listener::_openBlock(PT_AttrPropIndex api)
 	// FIXME: does AbiWord support this?
 	if (bHaveProp && pAP)
 	{
-		const XML_Char * szValue;
+		const gchar * szValue;
 	
 		// FLOW
 		// <FLOW align="left,right,center,justify"/>
@@ -839,10 +835,10 @@ void s_KWord_1_Listener::_openBlock(PT_AttrPropIndex api)
 	// e.g., <PAGEBREAKING linesTogether="true" keepWithNext="true"/>
 		bool bLinesTogether=false;
 		if (pAP->getProperty("keep-together", szValue))
-			bLinesTogether = (UT_stricmp(szValue,"true") == 0);
+			bLinesTogether = (g_ascii_strcasecmp(szValue,"true") == 0);
 		bool bKeepWithNext=false;
 		if (pAP->getProperty("keep-with-next", szValue))
-			bKeepWithNext = (UT_stricmp(szValue,"true") == 0);
+			bKeepWithNext = (g_ascii_strcasecmp(szValue,"true") == 0);
 	// FIXME: I should probably run this if and only if the above properties
 	// exist (dealing with styles)
 		if (bLinesTogether || bKeepWithNext)
@@ -974,7 +970,7 @@ void s_KWord_1_Listener::_openSpan(PT_AttrPropIndex api, PT_BlockOffset pos, UT_
 	if (bHaveProp && pAP)
 	{
 		// query and output properties
-		const XML_Char * szValue = 0;
+		const gchar * szValue = 0;
 
 		if (pAP->getProperty("color", szValue))
 		{
@@ -1039,7 +1035,7 @@ void s_KWord_1_Listener::_openSpan(PT_AttrPropIndex api, PT_BlockOffset pos, UT_
 		if (pAP->getProperty("font-weight", szValue))
 		{
 			m_sFormats += "<WEIGHT value=\"";
-			if (!UT_stricmp(static_cast<const char *>(szValue), "bold"))
+			if (!g_ascii_strcasecmp(static_cast<const char *>(szValue), "bold"))
 				m_sFormats += "75";
 			else
 				m_sFormats += "50";
@@ -1050,7 +1046,7 @@ void s_KWord_1_Listener::_openSpan(PT_AttrPropIndex api, PT_BlockOffset pos, UT_
 		if (pAP->getProperty("font-style", szValue))
 		{
 			m_sFormats += "<ITALIC value=\"";
-			if (!UT_stricmp(static_cast<const char *>(szValue), "italic"))
+			if (!g_ascii_strcasecmp(static_cast<const char *>(szValue), "italic"))
 				m_sFormats += "1";
 			else
 				m_sFormats += "0";
@@ -1074,9 +1070,9 @@ void s_KWord_1_Listener::_openSpan(PT_AttrPropIndex api, PT_BlockOffset pos, UT_
 		// <VERTALIGN value="0"/> 0=normal, 1=subscript, 2=superscript
 		if (pAP->getProperty("text-position", szValue))
 		{
-			if (!UT_stricmp (szValue, "subscript"))
+			if (!g_ascii_strcasecmp (szValue, "subscript"))
 				m_sFormats += "<VERTALIGN value=\"1\"/>\n";
-			else if (!UT_stricmp (szValue, "superscript"))
+			else if (!g_ascii_strcasecmp (szValue, "superscript"))
 				m_sFormats += "<VERTALIGN value=\"2\"/>\n";
 			else
 				m_sFormats += "<VERTALIGN value=\"0\"/>\n";
