@@ -1,8 +1,8 @@
-/* -*- mode: C++; tab-width: 4; c-basic-offset: 4; -*- */
+/* -*- Mode: C++; tab-width: 4; c-basic-offset: 4; -*- */
 
 /* AbiWord
  * Copyright (C) 1998 AbiSource, Inc.
- * Copyright (C) 2004 Marc Maurer (j.m.maurer@student.utwente.nl)
+ * Copyright (C) 2004 Marc Maurer (uwog@uwog.net)
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -29,6 +29,7 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include "ut_stack.h"
 #include "ut_debugmsg.h"
 #include "ut_string.h"
 #include "ut_bytebuf.h"
@@ -70,10 +71,6 @@ int abi_plugin_register (XAP_ModuleInfo * mi)
 	{
 		m_sniffer = new IE_Exp_LaTeX_Sniffer ();
 	}
-	else
-	{
-		m_sniffer->ref();
-	}
 
 	mi->name = "LaTeX Exporter";
 	mi->desc = "Export LaTeX Documents";
@@ -94,13 +91,11 @@ int abi_plugin_unregister (XAP_ModuleInfo * mi)
 	mi->author = 0;
 	mi->usage = 0;
 
-	UT_ASSERT (m_sniffer);
+	UT_return_val_if_fail (m_sniffer, 0);
 
 	IE_Exp::unregisterExporter (m_sniffer);
-	if (!m_sniffer->unref())
-	{
-		m_sniffer = 0;
-	}
+	delete m_sniffer;
+	m_sniffer = 0;
 
 	return 1;
 }
@@ -123,7 +118,7 @@ IE_Exp_LaTeX_Sniffer::IE_Exp_LaTeX_Sniffer () :
 
 bool IE_Exp_LaTeX_Sniffer::recognizeSuffix(const char * szSuffix)
 {
-	return (!UT_stricmp(szSuffix,".tex") || !UT_stricmp(szSuffix, ".latex"));
+	return (!g_ascii_strcasecmp(szSuffix,".tex") || !g_ascii_strcasecmp(szSuffix, ".latex"));
 }
 
 UT_Error IE_Exp_LaTeX_Sniffer::constructExporter(PD_Document * pDocument,
@@ -171,8 +166,8 @@ IE_Exp_LaTeX::~IE_Exp_LaTeX()
 /*****************************************************************/
 /*****************************************************************/
 typedef UT_UCSChar U16;
-static int wvConvertUnicodeToLaTeX(U16 char16,char*& out);
-static bool _convertLettersToSymbols(char c, char *& subst);
+static int wvConvertUnicodeToLaTeX(U16 char16, const char*& out);
+static bool _convertLettersToSymbols(char c, const char *& subst);
 
 #define BT_NORMAL		1
 #define BT_HEADING1		2
@@ -183,12 +178,82 @@ static bool _convertLettersToSymbols(char c, char *& subst);
 
 #define BULLET_LIST	1
 #define NUMBERED_LIST	2
+typedef int LIST_TYPE;
+
+class LaTeX_Analysis_Listener : public PL_Listener
+{
+public:
+	bool m_hasEndnotes;
+
+	LaTeX_Analysis_Listener(PD_Document * pDocument,
+							IE_Exp_LaTeX * pie)
+		: m_hasEndnotes(false)
+	{
+	}
+
+	virtual ~LaTeX_Analysis_Listener()
+	{
+	}
+
+	virtual bool		populate(PL_StruxFmtHandle sfh,
+								 const PX_ChangeRecord * pcr)
+	{
+		return true;	
+	}
+
+	virtual bool		populateStrux(PL_StruxDocHandle sdh,
+									  const PX_ChangeRecord * pcr,
+									  PL_StruxFmtHandle * psfh)
+	{
+		UT_ASSERT(pcr->getType() == PX_ChangeRecord::PXT_InsertStrux);
+		const PX_ChangeRecord_Strux * pcrx = static_cast<const PX_ChangeRecord_Strux *> (pcr);
+		*psfh = 0;							// we don't need it.
+
+		switch (pcrx->getStruxType())
+			{
+			case PTX_SectionEndnote:
+			case PTX_EndEndnote:
+				m_hasEndnotes = true;
+				break;
+			default:
+				break;
+			}
+
+		return true;
+	}
+
+	virtual bool		change(PL_StruxFmtHandle sfh,
+							   const PX_ChangeRecord * pcr)
+	{
+		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+		return false;
+	}
+
+	virtual bool		insertStrux(PL_StruxFmtHandle sfh,
+									const PX_ChangeRecord * pcr,
+									PL_StruxDocHandle sdh,
+									PL_ListenerId lid,
+									void (* pfnBindHandles)(PL_StruxDocHandle sdhNew,
+															PL_ListenerId lid,
+															PL_StruxFmtHandle sfhNew))
+	{
+		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+		return false;
+	}
+
+	virtual bool		signal(UT_uint32 iSignal)
+	{
+		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+		return false;
+	}
+};
 
 class s_LaTeX_Listener : public PL_Listener
 {
 public:
 	s_LaTeX_Listener(PD_Document * pDocument,
-						IE_Exp_LaTeX * pie);
+					 IE_Exp_LaTeX * pie,
+					 LaTeX_Analysis_Listener& analysis);
 	virtual ~s_LaTeX_Listener();
 
 	virtual bool		populate(PL_StruxFmtHandle sfh,
@@ -218,6 +283,8 @@ protected:
 	void				_closeSection(void);
 	void				_closeSpan(void);
 	void				_closeTable(void);
+	void                _closeList(void);
+	void                _closeLists(void);
 	void				_openCell(PT_AttrPropIndex api);
 	void				_openParagraph(PT_AttrPropIndex api);
 	void				_openSection(PT_AttrPropIndex api);
@@ -244,12 +311,17 @@ protected:
 	bool           m_bInSymbol;
  	bool	m_bInCourier;
  	bool	m_bInSansSerif;
+	bool				m_bInEndnote;
+	bool				m_bHaveEndnote;
   
 	JustificationTypes  m_eJustification;
 	bool				m_bLineHeight;
 	bool				m_bFirstSection;
 	int 				ChapterNumber;
-	int				list_type;
+	int                 m_Indent;
+	
+// Type for the last-processed list  
+	LIST_TYPE			list_type;
 
 	// Need to look up proper type, and place to stick #defines...
 
@@ -259,14 +331,40 @@ protected:
 	// Table utility members
 	
 	ie_Table *			m_pTableHelper;
+
+//	std::stack<LIST_TYPE>   list_stack;
+	UT_NumberStack          list_stack;
 };
 
 void s_LaTeX_Listener::_closeParagraph(void)
 {
-	if ((!m_bInCell) && (!m_bInFootnote)) m_pie->write("\n");
+	if ((!m_bInCell) && (!m_bInFootnote) && (!m_bInEndnote)) m_pie->write("\n");
 	m_bInHeading = false;
 	return;
 }
+        
+void s_LaTeX_Listener::_closeList(void)
+{
+	switch (list_type) {
+		case NUMBERED_LIST:
+			m_pie->write("\\end{enumerate}\n");
+			break;
+		case BULLET_LIST:
+			m_pie->write("\\end{itemize}\n");
+			break;
+		default:
+			;
+	}
+	list_stack.pop();
+}
+
+void s_LaTeX_Listener::_closeLists()
+{
+	do{
+		_closeList();
+	} while(list_stack.getDepth());
+	m_bInList = false;
+}
 
 void s_LaTeX_Listener::_closeSection(void)
 {
@@ -277,12 +375,7 @@ void s_LaTeX_Listener::_closeSection(void)
 
 	if (m_bInList)
 	{
-		m_bInList = false;
-		if (list_type == BULLET_LIST)
-			m_pie->write("\\end{itemize}");
-		else if (list_type == NUMBERED_LIST)
-			m_pie->write("\\end{enumerate}");
-
+		this->_closeLists();
 	}
  
 	if (m_bMultiCols)
@@ -297,7 +390,7 @@ void s_LaTeX_Listener::_closeSection(void)
 
 void s_LaTeX_Listener::_closeBlock(void)
 { 
-	if(m_bInFootnote)
+	if(m_bInFootnote || m_bInEndnote)
 		return;
 	if (!m_bInBlock)
 		return;
@@ -349,16 +442,16 @@ void s_LaTeX_Listener::_openCell(PT_AttrPropIndex api)
 {
 	const PP_AttrProp * pAP = NULL;
 	bool bHaveProp = m_pDocument->getAttrProp(api,&pAP);
-	const XML_Char * szValue;
+	const gchar * szValue;
 	
 	m_bInCell = true;
 	if (bHaveProp && pAP)
 	{
 		pAP->getProperty("left-attach", szValue);
-		if (!UT_strcmp("0",szValue))
+		if (!strcmp("0",szValue))
 		{
 			pAP->getProperty("top-attach", szValue);
-			if (!UT_strcmp("0",szValue))
+			if (!strcmp("0",szValue))
 				m_pie->write("\n\\hline\n");
 			else
 				m_pie->write("\\\\\n\\hline\n");
@@ -383,82 +476,116 @@ void s_LaTeX_Listener::_openParagraph(PT_AttrPropIndex api)
 	
 	if (bHaveProp && pAP)
 	{
-		const XML_Char * szValue;
+		const gchar * szValue;
 
 		if ((pAP->getAttribute(PT_LISTID_ATTRIBUTE_NAME, szValue))
 			&& (pAP->getAttribute(PT_STYLE_ATTRIBUTE_NAME, szValue))
-			&& (0 == UT_strcmp(szValue, "Normal")))
+			&& (0 == strcmp(szValue, "Normal")))
 		{
-			if (m_bInList)
-				m_pie->write("\\item ");
-			else
+			int indent = 0;
+			bool bNewList = false;
+			const gchar * szIndent, * szLeft, * szListStyle;
+			
+			pAP->getProperty("list-style", szListStyle);
+			
+			if (pAP->getProperty("text-indent", szIndent) && pAP->getProperty("margin-left", szLeft))
+			{
+				indent = UT_convertToDimension(szIndent, DIM_MM) + UT_convertToDimension(szLeft, DIM_MM);
+				if (m_bInList)
+				{
+					UT_DEBUGMSG(("      indent = %d, m_Indent = %d\n", indent, m_Indent));
+					if(indent > this->m_Indent) //nested list
+						bNewList = true;
+					else if (indent < this->m_Indent)
+					{
+						this->_closeList();
+					}
+					else
+						/*
+						 * now we have indent == this->m_Indent,
+						 * but it is possible that the current list item is
+						 * of different style with the last one.
+						 */                                    
+					{
+						if(szListStyle)
+						{
+							LIST_TYPE temp = BULLET_LIST;
+							if (0 == strcmp(szListStyle, "Numbered List") )
+								temp = NUMBERED_LIST;
+							if (temp != list_type)
+							{
+								this->_closeList();
+								bNewList = true;
+							}
+						}
+					}
+				}
+			}
+			
+			if (bNewList || !m_bInList) 
+				//necessary to build a new (possibly nested) list
 			{
 				if (pAP->getProperty("list-style", szValue))
 				{
-					if (0 == UT_strcmp(szValue, "Numbered List"))
+					//if(NULL == list_stack)
+					//  this->list_stack = new std::stack<LIST_TYPE>();
+					if (0 == strcmp(szValue, "Numbered List"))
 					{
 						list_type = NUMBERED_LIST;
-						m_pie->write("\\begin{enumerate}\n\\item ");
+						m_pie->write("\\begin{enumerate}\n");
 					}
-					else if (0 == UT_strcmp(szValue, "Bullet List"))
+					else if (0 == strcmp(szValue, "Bullet List"))
 					{
 						list_type = BULLET_LIST;
-						m_pie->write("\\begin{itemize}\n\\item ");
+						m_pie->write("\\begin{itemize}\n");
 					}
+					list_stack.push(list_type);
 				}
 				m_bInList = true;
 			}
+			
+			if (szIndent && szLeft)
+				this->m_Indent = indent;
+			
+			m_pie->write("\\item ");
 		} else if (m_bInList) {
-			m_bInList = false;
-			switch (list_type) {
-			case NUMBERED_LIST:
-				m_pie->write("\\end{enumerate}\n");
-				break;
-			case BULLET_LIST:
-				m_pie->write("\\end{itemize}\n");
-				break;
-			default:
-				;
-			}
+			this->_closeLists();
 		}
-
-
-
 		if (pAP->getAttribute(PT_STYLE_ATTRIBUTE_NAME, szValue))
 		{
 			if (strstr(szValue, "Heading"))
 				m_bInHeading = true;
-			if(0 == UT_strcmp(szValue, "Heading 1")) 
+			if(0 == strcmp(szValue, "Heading 1")) 
 			{
 				m_iBlockType = BT_HEADING1;
 				m_pie->write("\\section*{");
 			}
-			else if(0 == UT_strcmp(szValue, "Heading 2")) 
+			else if(0 == strcmp(szValue, "Heading 2")) 
 			{
 				m_iBlockType = BT_HEADING2;
 				m_pie->write("\\subsection*{");
 			}
-			else if(0 == UT_strcmp(szValue, "Heading 3")) 
+			else if(0 == strcmp(szValue, "Heading 3")) 
 			{
 				m_iBlockType = BT_HEADING3;
 				m_pie->write("\\subsubsection*{");
 			}
-			else if(0 == UT_strcmp(szValue, "Numbered Heading 1")) 
+			else if(0 == strcmp(szValue, "Numbered Heading 1")) 
 			{
 				m_iBlockType = BT_HEADING1;
 				m_pie->write("\\section{");
 			}
-			else if(0 == UT_strcmp(szValue, "Numbered Heading 2")) 
+			else if(0 == strcmp(szValue, "Numbered Heading 2")) 
 			{
 				m_iBlockType = BT_HEADING2;
 				m_pie->write("\\subsection{");
 			}
-			else if(0 == UT_strcmp(szValue, "Numbered Heading 3")) 
+			else if(0 == strcmp(szValue, "Numbered Heading 3")) 
 			{
 				m_iBlockType = BT_HEADING3;
 				m_pie->write("\\subsubsection{");
 			}
-			else if (0 == UT_strcmp(szValue, "Chapter Heading")) {
+			else if (0 == strcmp(szValue, "Chapter Heading")) {
 				// TODO: Clean this...
 				char			szChapterNumber[6];
 				m_iBlockType = BT_HEADING1;
@@ -467,7 +594,7 @@ void s_LaTeX_Listener::_openParagraph(PT_AttrPropIndex api)
 				m_pie->write(szChapterNumber);
 				m_pie->write(" ");	  // \\newline");
         		}
-			else if (0 == UT_strcmp(szValue, "Chapter Heading")) {
+			else if (0 == strcmp(szValue, "Chapter Heading")) {
 				// TODO: Clean this...
 				char			szChapterNumber[6];
 				m_iBlockType = BT_HEADING1;
@@ -476,12 +603,12 @@ void s_LaTeX_Listener::_openParagraph(PT_AttrPropIndex api)
 				m_pie->write(szChapterNumber);
 				m_pie->write(" ");	  // \\newline");
         		}
-			else if(0 == UT_strcmp(szValue, "Block Text"))
+			else if(0 == strcmp(szValue, "Block Text"))
 			{
 				m_iBlockType = BT_BLOCKTEXT;
 				m_pie->write("\\begin{quote}\n");
 			}
-			else if(0 == UT_strcmp(szValue, "Plain Text"))
+			else if(0 == strcmp(szValue, "Plain Text"))
 			{
 				m_iBlockType = BT_PLAINTEXT;
 				m_pie->write("\\texttt{");
@@ -502,17 +629,17 @@ void s_LaTeX_Listener::_openParagraph(PT_AttrPropIndex api)
 		{
 			if (pAP->getProperty("text-align", szValue))
 			{
-				if (0 == UT_strcmp(szValue, "center"))
+				if (0 == strcmp(szValue, "center"))
 				{
 					m_pie->write("\\begin{center}\n");
 					m_eJustification = CENTER;
 				}
-				if (0 == UT_strcmp(szValue, "right"))
+				if (0 == strcmp(szValue, "right"))
 				{
 					m_pie->write("\\begin{flushright}\n");
 					m_eJustification = RIGHT;
 				}
-				if (0 == UT_strcmp(szValue, "left"))
+				if (0 == strcmp(szValue, "left"))
 				{
 					m_pie->write("\\begin{flushleft}\n");
 					m_eJustification = LEFT;
@@ -550,7 +677,7 @@ void s_LaTeX_Listener::_openSection(PT_AttrPropIndex api)
 {
 	const PP_AttrProp* pAP = NULL;
 	bool bMustEmitMulticol = false;
-	const XML_Char* pszNbCols = NULL;
+	const gchar* pszNbCols = NULL;
 
 	m_bBetweenQuotes = false;
 	m_bInList = false;
@@ -559,15 +686,15 @@ void s_LaTeX_Listener::_openSection(PT_AttrPropIndex api)
 
 	if (m_pDocument->getAttrProp(api, &pAP) && pAP)
 	{
-		const XML_Char* pszPageMarginLeft = NULL;
-		const XML_Char* pszPageMarginRight = NULL;
+		const gchar* pszPageMarginLeft = NULL;
+		const gchar* pszPageMarginRight = NULL;
 
 		pAP->getProperty("columns", pszNbCols);
 		pAP->getProperty("page-margin-right", pszPageMarginLeft);
 		pAP->getProperty("page-margin-left", pszPageMarginRight);
 
-		if (pszNbCols != NULL && ((0 == UT_strcmp(pszNbCols, "2"))
-								  || (0 == UT_strcmp(pszNbCols, "3"))))
+		if (pszNbCols != NULL && ((0 == strcmp(pszNbCols, "2"))
+								  || (0 == strcmp(pszNbCols, "3"))))
 		{
 			bMustEmitMulticol = true;
 			m_bMultiCols = true;
@@ -681,11 +808,11 @@ void s_LaTeX_Listener::_openSpan(PT_AttrPropIndex api)
 	
 	if (bHaveProp && pAP)
 	{
-		const XML_Char * szValue;
+		const gchar * szValue;
 
 		if (
 			(pAP->getProperty("font-weight", szValue) && !m_bInHeading)
-			&& !UT_strcmp(szValue, "bold")
+			&& !strcmp(szValue, "bold")
 			)
 		{
 			m_pie->write("\\textbf{");
@@ -693,7 +820,7 @@ void s_LaTeX_Listener::_openSpan(PT_AttrPropIndex api)
 		
 		if (
 			(pAP->getProperty("font-style", szValue) && !m_bInHeading)
-			&& !UT_strcmp(szValue, "italic")
+			&& !strcmp(szValue, "italic")
 			)
 		{
 			m_pie->write("\\emph{");
@@ -701,33 +828,33 @@ void s_LaTeX_Listener::_openSpan(PT_AttrPropIndex api)
 		
 		if (pAP->getProperty("text-decoration", szValue))
 		{
-			const XML_Char* pszDecor = szValue;
+			const gchar* pszDecor = szValue;
 
-			XML_Char* p;
-			if (!UT_cloneString((char *&)p, pszDecor))
+			gchar* p;
+			if (!(p = g_strdup(pszDecor)))
 			{
 				// TODO outofmem
 			}
 			
 			UT_ASSERT(p || !pszDecor);
-			XML_Char*	q = strtok(p, " ");
+			gchar*	q = strtok(p, " ");
 
 			// See the ulem.sty documentation (available at www.ctan.org)
 			// if you wish to include other kinds of underlines, such as
 			// double underlines or wavy underlines
 			while (q)
 			{
-			  if (0 == UT_strcmp(q, "underline")) // TODO: \def\undertext#1{$\underline{\vphantom{y}\smash{\hbox{#1}}}$}
+			  if (0 == strcmp(q, "underline")) // TODO: \def\undertext#1{$\underline{\vphantom{y}\smash{\hbox{#1}}}$}
 				{
 					m_pie->write("\\uline{");
 				}
 
-				if (0 == UT_strcmp(q, "overline"))
+				if (0 == strcmp(q, "overline"))
 				{
 					m_pie->write("$\\overline{\\textrm{");
 				}
 
-				if (0 == UT_strcmp(q, "line-through"))
+				if (0 == strcmp(q, "line-through"))
 				{
 					m_pie->write("\\sout{");
 				}
@@ -740,13 +867,13 @@ void s_LaTeX_Listener::_openSpan(PT_AttrPropIndex api)
 
 		if (pAP->getProperty("text-position", szValue))
 		{
-			if (!UT_strcmp("superscript", szValue))
+			if (!strcmp("superscript", szValue))
 			{
 				m_bInScript = true;
 				//m_pie->write("$^{\\mathrm{");
 				m_pie->write("\\textsuperscript{");
 			}
-			else if (!UT_strcmp("subscript", szValue))
+			else if (!strcmp("subscript", szValue))
 			{
 				m_bInScript = true;
 				//m_pie->write("$_{\\mathrm{");
@@ -754,12 +881,12 @@ void s_LaTeX_Listener::_openSpan(PT_AttrPropIndex api)
 			}
 		}
 		
-		const XML_Char* pszColor = NULL;
+		const gchar* pszColor = NULL;
 		pAP->getProperty("color", pszColor);
 		if (pszColor)
 		  {
-		    if ((0 != UT_strcmp("000000", pszColor)) &&
-			(0 != UT_strcmp("000000", pszColor)))
+		    if ((0 != strcmp("000000", pszColor)) &&
+			(0 != strcmp("transparent", pszColor)))
 		      {
  		 	UT_String szColor;
 			_convertColor(szColor,(const char*)pszColor);
@@ -771,13 +898,13 @@ void s_LaTeX_Listener::_openSpan(PT_AttrPropIndex api)
 		      m_pie->write("{");
 		  }
 		
-		const XML_Char* pszBgColor = NULL;
+		const gchar* pszBgColor = NULL;
 		pAP->getProperty("bgcolor", pszBgColor);
 
 		if (pszBgColor)
 		{
-		  if ((0 != UT_strcmp("000000", pszBgColor)) &&
-		      (0 != UT_strcmp("transparent", pszBgColor)))
+		  if ((0 != strcmp("000000", pszBgColor)) &&
+		      (0 != strcmp("transparent", pszBgColor)))
 		    {
 		      UT_String szColor;
 		      _convertColor(szColor,(const char*)pszBgColor);
@@ -791,7 +918,7 @@ void s_LaTeX_Listener::_openSpan(PT_AttrPropIndex api)
 
  		if (pAP->getProperty("font-size", szValue) && !m_bInHeading)
 		{
-			if (UT_strcmp (DEFAULT_SIZE, szValue) != 0)
+			if (strcmp (DEFAULT_SIZE, szValue) != 0)
 			{
 				m_pie->write("{\\");
 				UT_String szSize;
@@ -804,18 +931,18 @@ void s_LaTeX_Listener::_openSpan(PT_AttrPropIndex api)
 		if (pAP->getProperty("font-family", szValue) && !m_bInHeading)
 		{
 			// TODO: Use a dynamic substitution table
-			if (!UT_strcmp("Symbol", szValue) ||
-				!UT_strcmp("Standard Symbols", szValue))
+			if (!strcmp("Symbol", szValue) ||
+				!strcmp("Standard Symbols", szValue))
 				m_bInSymbol = true;
-			if (!UT_strcmp("Courier", szValue) ||
-				!UT_strcmp("Courier New", szValue) ||
-				!UT_strcmp("Luxi Mono",szValue)) {
+			if (!strcmp("Courier", szValue) ||
+				!strcmp("Courier New", szValue) ||
+				!strcmp("Luxi Mono",szValue)) {
 				m_bInCourier = true;
 				m_pie->write("\\texttt{");
 			}
-			if (!UT_strcmp("Arial", szValue) ||
-				!UT_strcmp("Helvetic", szValue) ||
-				!UT_strcmp("Luxi Sans",szValue)) {
+			if (!strcmp("Arial", szValue) ||
+				!strcmp("Helvetic", szValue) ||
+				!strcmp("Luxi Sans",szValue)) {
 				m_bInSansSerif = true;
 				m_pie->write("\\textsf{");
 			}
@@ -830,9 +957,8 @@ void s_LaTeX_Listener::_openSpan(PT_AttrPropIndex api)
 void s_LaTeX_Listener::_openTable(PT_AttrPropIndex api)
 {
 	UT_sint32 i = 0;
-	
-	m_pie->write("\n\\\\");
-	m_pie->write("\n%");
+
+	m_pie->write("\n\n%");
 	m_pie->write("\n% Table begins");
 	m_pie->write("\n% ");
 	m_pie->write("\n\\begin{table}[h]\\begin{tabular}{|");
@@ -854,7 +980,7 @@ void s_LaTeX_Listener::_closeSpan(void)
 	
 	if (pAP)
 	{
-		const XML_Char * szValue;
+		const gchar * szValue;
 		
 		if (pAP->getProperty("color", szValue))
 		{
@@ -870,19 +996,19 @@ void s_LaTeX_Listener::_closeSpan(void)
 //		    || (pAP->getProperty("font-family", szValue))  // TODO
 			)
 		{
-			if (UT_strcmp (szValue, DEFAULT_SIZE) != 0)
+			if (strcmp (szValue, DEFAULT_SIZE) != 0)
 				m_pie->write("}");
 		}
 
 		if (pAP->getProperty("text-position", szValue))
 		{
-			if (!UT_strcmp("superscript", szValue))
+			if (!strcmp("superscript", szValue))
 			{
 				m_bInScript = false;
 				//m_pie->write("}}$");
 				m_pie->write("}\n");
 			}
-			else if (!UT_strcmp("subscript", szValue))
+			else if (!strcmp("subscript", szValue))
 			{
 				m_bInScript = false;
 				//m_pie->write("}}$");
@@ -894,20 +1020,20 @@ void s_LaTeX_Listener::_closeSpan(void)
 			(pAP->getProperty("text-decoration", szValue))
 			)
 		{
-			const XML_Char* pszDecor = szValue;
+			const gchar* pszDecor = szValue;
 			
-			XML_Char* p;
-			if (!UT_cloneString((char *&)p, pszDecor))
+			gchar* p;
+			if (!(p = g_strdup(pszDecor)))
 			{
 				// TODO outofmem
 			}
 			
 			UT_ASSERT(p || !pszDecor);
-			XML_Char*	q = strtok(p, " ");
+			gchar*	q = strtok(p, " ");
 
 			while (q)
 			{
-				if (0 == UT_strcmp(q, "line-through"))
+				if (0 == strcmp(q, "line-through"))
 				{
 					m_pie->write("}");
 				}
@@ -922,25 +1048,25 @@ void s_LaTeX_Listener::_closeSpan(void)
 			(pAP->getProperty("text-decoration", szValue))
 			)
 		{
-			const XML_Char* pszDecor = szValue;
+			const gchar* pszDecor = szValue;
 			
-			XML_Char* p;
-			if (!UT_cloneString((char *&)p, pszDecor))
+			gchar* p;
+			if (!(p = g_strdup(pszDecor)))
 			{
 				// TODO outofmem
 			}
 			
 			UT_ASSERT(p || !pszDecor);
-			XML_Char*	q = strtok(p, " ");
+			gchar*	q = strtok(p, " ");
 
 			while (q)
 			{
-				if (0 == UT_strcmp(q, "underline"))
+				if (0 == strcmp(q, "underline"))
 				{
 					m_pie->write("}");
 				}
 
-				if (0 == UT_strcmp(q, "overline"))
+				if (0 == strcmp(q, "overline"))
 				{
 					m_pie->write("}}$");
 				}
@@ -953,7 +1079,7 @@ void s_LaTeX_Listener::_closeSpan(void)
 
 		if (
 			(pAP->getProperty("font-style", szValue) && !m_bInHeading)
-			&& !UT_strcmp(szValue, "italic")
+			&& !strcmp(szValue, "italic")
 			)
 		{
 			m_pie->write("}");
@@ -961,7 +1087,7 @@ void s_LaTeX_Listener::_closeSpan(void)
 		
 		if (
 			(pAP->getProperty("font-weight", szValue) && !m_bInHeading)
-			&& !UT_strcmp(szValue, "bold")
+			&& !strcmp(szValue, "bold")
 			)
 		{
 			m_pie->write("}");
@@ -969,18 +1095,18 @@ void s_LaTeX_Listener::_closeSpan(void)
 
 		if (pAP->getProperty("font-family", szValue) && !m_bInHeading)
 		{
-			if (!UT_strcmp ("Symbol", szValue) ||
-				!UT_strcmp("Standard Symbols", szValue))
+			if (!strcmp ("Symbol", szValue) ||
+				!strcmp("Standard Symbols", szValue))
 				m_bInSymbol = false;
-			if (!UT_strcmp("Courier", szValue) ||
-				!UT_strcmp("Courier New", szValue))
+			if (!strcmp("Courier", szValue) ||
+				!strcmp("Courier New", szValue))
 			{
 				m_pie->write("}");
 				m_bInCourier = false;
 			}
-			if (!UT_strcmp("Helvetic", szValue) ||
-				!UT_strcmp("Arial", szValue) ||
-				!UT_strcmp("Luxi Sans", szValue))
+			if (!strcmp("Helvetic", szValue) ||
+				!strcmp("Arial", szValue) ||
+				!strcmp("Luxi Sans", szValue))
 			{
 				m_pie->write("}");
 				m_bInSansSerif = false;
@@ -1013,7 +1139,7 @@ void s_LaTeX_Listener::_outputData(const UT_UCSChar * data, UT_uint32 length)
 
 	for (pData = data; (pData < data + length); /**/)
 	{
-		char* subst = "";
+		const char* subst = "";
 
 		if (m_bInSymbol)
 		{
@@ -1158,7 +1284,7 @@ void s_LaTeX_Listener::_outputData(const UT_UCSChar * data, UT_uint32 length)
 
 #define SUB(a,who) case a: subst = "\\(\\"who"\\)"; return true;
 #define SUBd(a,who) case a: subst = who; return true;
-static bool _convertLettersToSymbols(char c, char *& subst)
+static bool _convertLettersToSymbols(char c, const char *& subst)
 {
 	switch (c)
 	{
@@ -1200,18 +1326,20 @@ static bool _convertLettersToSymbols(char c, char *& subst)
 	}
 }
 
-s_LaTeX_Listener::s_LaTeX_Listener(PD_Document * pDocument, IE_Exp_LaTeX * pie)
+s_LaTeX_Listener::s_LaTeX_Listener(PD_Document * pDocument, IE_Exp_LaTeX * pie, LaTeX_Analysis_Listener& analysis)
   : m_pDocument(pDocument),
 	m_pie(pie),
-	m_bInSection(false),
 	m_bInBlock(false),
 	m_bInCell(false),
+	m_bInSection(false),
 	m_bInSpan(false),
 	m_bInFootnote(false),
-	m_bFirstSection(true),
 	m_bInSymbol(0),
 	m_bInCourier(0),
-	m_bInSansSerif(0)
+	m_bInSansSerif(0),
+	m_bFirstSection(true),
+	m_bInEndnote(false),
+	m_bHaveEndnote(analysis.m_hasEndnotes)
 {
 	m_pie->write("%% ================================================================================\n");
 	m_pie->write("%% This LaTeX file was created by AbiWord.                                         \n");
@@ -1240,6 +1368,10 @@ s_LaTeX_Listener::s_LaTeX_Listener(PD_Document * pDocument, IE_Exp_LaTeX * pie)
 	m_pie->write("%% Please set your language here\n");
 	m_pie->write("\\usepackage[english]{babel}\n");
 	m_pie->write("\\usepackage{color}\n");
+
+	if (m_bHaveEndnote)
+		m_pie->write("\\usepackage{endnotes}\n");
+
 	// Must be as late as possible.
 	m_pie->write("\\usepackage{hyperref}\n");
 
@@ -1265,7 +1397,8 @@ s_LaTeX_Listener::~s_LaTeX_Listener()
 	_closeSection();
 	_handleDataItems();
 	DELETEP(m_pTableHelper);
-
+	if (m_bHaveEndnote)
+		m_pie->write("\n\\theendnotes");
 	m_pie->write("\n\\end{document}\n");
 }
 
@@ -1302,14 +1435,12 @@ bool s_LaTeX_Listener::populate(PL_StruxFmtHandle /*sfh*/,
 			const PP_AttrProp * pAP = NULL;
 			bool bHaveProp = m_pDocument->getAttrProp(api,&pAP);
 
-			const XML_Char* szValue = NULL;
+			const gchar* szValue = NULL;
 
 			fd_Field* field = NULL;
 
 			switch (pcro->getObjectType())
 			{
-			int i;
-			
 			case PTO_Image:
 				// TODO we *will* create separate image files.
 				// (we will borrow the code from HTML export)
@@ -1355,7 +1486,7 @@ bool s_LaTeX_Listener::populate(PL_StruxFmtHandle /*sfh*/,
 				if(m_bInHeading) return true;
 				if(bHaveProp && pAP && pAP->getAttribute("type", szValue))
 				{
-					if(0 == UT_strcmp("start",szValue))
+					if(0 == strcmp("start",szValue))
 					{
 						if(pAP->getAttribute("name", szValue))
 						{
@@ -1364,7 +1495,7 @@ bool s_LaTeX_Listener::populate(PL_StruxFmtHandle /*sfh*/,
 							m_pie->write("}{");
 						}
 					}
-					else if(0 == UT_strcmp("end",szValue)) m_pie->write("}");
+					else if(0 == strcmp("end",szValue)) m_pie->write("}");
 				}
 				else
 				{
@@ -1373,7 +1504,7 @@ bool s_LaTeX_Listener::populate(PL_StruxFmtHandle /*sfh*/,
 				return true;
 
 			default:
-				UT_ASSERT(0);
+				UT_ASSERT_HARMLESS(0);
 				return true;
 			}
 		}
@@ -1382,7 +1513,7 @@ bool s_LaTeX_Listener::populate(PL_StruxFmtHandle /*sfh*/,
 		return true;
 		
 	default:
-		UT_ASSERT(0);
+		UT_ASSERT_HARMLESS(0);
 		return false;
 	}
 }
@@ -1397,7 +1528,6 @@ bool s_LaTeX_Listener::populateStrux(PL_StruxDocHandle sdh,
 
 	switch (pcrx->getStruxType())
 	{
-	case PTX_SectionEndnote:
 	case PTX_Section:
 	{
 		_closeSpan();
@@ -1408,11 +1538,11 @@ bool s_LaTeX_Listener::populateStrux(PL_StruxDocHandle sdh,
 		const PP_AttrProp* pAP = NULL;
 		if (m_pDocument->getAttrProp(indexAP, &pAP) && pAP)
 		{
-			const XML_Char* pszSectionType = NULL;
+			const gchar* pszSectionType = NULL;
 			pAP->getAttribute("type", pszSectionType);
 			if (
 				!pszSectionType
-				|| (0 == UT_strcmp(pszSectionType, "doc"))
+				|| (0 == strcmp(pszSectionType, "doc"))
 				)
 			{
 				_openSection(pcr->getIndexAP());
@@ -1441,11 +1571,11 @@ bool s_LaTeX_Listener::populateStrux(PL_StruxDocHandle sdh,
 		const PP_AttrProp* pAP = NULL;
 		if (m_pDocument->getAttrProp(indexAP, &pAP) && pAP)
 		{
-			const XML_Char* pszSectionType = NULL;
+			const gchar* pszSectionType = NULL;
 			pAP->getAttribute("type", pszSectionType);
 			if (
 				!pszSectionType
-				|| (0 == UT_strcmp(pszSectionType, "doc"))
+				|| (0 == strcmp(pszSectionType, "doc"))
 				)
 			{
 				_openSection(pcr->getIndexAP());
@@ -1516,8 +1646,31 @@ bool s_LaTeX_Listener::populateStrux(PL_StruxDocHandle sdh,
 		m_pie->write("\\footnote{");
 		return true;
 	}
-
+	
+	case PTX_SectionTOC:
+	{
+		_closeSpan();	
+		_closeBlock();
+		/*
+		_closeSection();
+		 */
+		m_pie->write("\\tableofcontents \n");
+	}
+	case PTX_EndTOC:
+		return true;
+	
+	case PTX_SectionEndnote:
+	{
+		m_bInEndnote = true;
+		m_pie->write("\\endnote{");
+		return true;
+	}
 	case PTX_EndEndnote:
+	{
+		m_bInEndnote = false;
+		m_pie->write("} ");
+		return true;
+	}
 	default:
 		UT_ASSERT(UT_TODO);
 		return true;
@@ -1555,7 +1708,11 @@ bool s_LaTeX_Listener::signal(UT_uint32 /* iSignal */)
 
 UT_Error IE_Exp_LaTeX::_writeDocument(void)
 {
-	m_pListener = new s_LaTeX_Listener(getDoc(),this);
+	LaTeX_Analysis_Listener analysis(getDoc(), this);
+	if (!getDoc()->tellListener(&analysis))
+		return UT_ERROR;
+
+	m_pListener = new s_LaTeX_Listener(getDoc(),this, analysis);
 	if (!m_pListener)
 		return UT_IE_NOMEMORY;
 	if (!getDoc()->tellListener(static_cast<PL_Listener *>(m_pListener)))
@@ -1582,7 +1739,7 @@ void s_LaTeX_Listener::_handleDataItems(void)
 #undef printf
 #define printf(x) out = (x); 
 
-static int wvConvertUnicodeToLaTeX(U16 char16,char*& out)
+static int wvConvertUnicodeToLaTeX(U16 char16,const char*& out)
 	{
 	out = ""; //this is needed
 
@@ -2337,22 +2494,22 @@ static int wvConvertUnicodeToLaTeX(U16 char16,char*& out)
 			return(1);
 
 		case 0x01FA:
-			printf("\\'{\\AA}");  /*  with acute */
+			printf("\\'{\\AA}");  /* ? with acute */
 			return(1);
 		case 0x01FB:
-			printf("\\'{\\aa}");  /*  with acute */
+			printf("\\'{\\aa}");  /* ? with acute */
 			return(1);
 		case 0x01FC:
-			printf("\\'{\\AE}");  /*  with acute */
+			printf("\\'{\\AE}");  /* ? with acute */
 			return(1);
 		case 0x01FD:
-			printf("\\'{\\ae}");  /*  with acute */
+			printf("\\'{\\ae}");  /* ? with acute */
 			return(1);
 		case 0x01FE:
-			printf("\\'{\\O}");  /*  with acute */
+			printf("\\'{\\O}");  /* ? with acute */
 			return(1);
 		case 0x01FF:
-			printf("\\'{\\o}");  /*  with acute */
+			printf("\\'{\\o}");  /* ? with acute */
 			return(1);
 
 		case 0x2010:
