@@ -113,66 +113,76 @@ static inline void do_SetMetadata(PD_Document* aDoc, const UT_String& aKey, UT_U
 	aDoc->setMetaDataProp(aKey, str);
 }
 
-void SDWDocInfo::load(GsfInfile* aDoc, PD_Document* aPDDoc) UT_THROWS((UT_Error)) {
-	UT_DEBUGMSG(("SDW: Loading Docinfo...\n"));
-	// firstly, set StarOffice as generator
-	aPDDoc->setMetaDataProp(PD_META_KEY_GENERATOR, UT_UTF8String("StarOffice"));
-	AutoGsfInput docInfo =  gsf_infile_child_by_name(aDoc, "SfxDocumentInfo");
-	if (!(GsfInput*)docInfo)
-		UT_THROW(UT_IE_BOGUSDOCUMENT);
+void SDWDocInfo::load(GsfInfile* aDoc, PD_Document* aPDDoc) 
+	UT_THROWS((UT_Error)) 
+{
+	char* headStr = NULL;
 
-	char* headStr;
-	readByteString(docInfo, headStr);
-	if (strcmp(headStr, "SfxDocumentInfo") != 0)
-		UT_THROW(UT_IE_BOGUSDOCUMENT);
+	try {
+		UT_DEBUGMSG(("SDW: Loading Docinfo...\n"));
+		// firstly, set StarOffice as generator
+		aPDDoc->setMetaDataProp(PD_META_KEY_GENERATOR, UT_UTF8String("StarOffice"));
+		AutoGsfInput docInfo =  gsf_infile_child_by_name(aDoc, "SfxDocumentInfo");
+		if (!(GsfInput*)docInfo)
+			UT_THROW(UT_IE_BOGUSDOCUMENT);
 
-	UT_uint16 version;
-	streamRead(docInfo, version);
-	bool pwProtect;
-	streamRead(docInfo, pwProtect);
-	UT_uint16 charset;
-	streamRead(docInfo, charset);
-	auto_iconv converter(findConverter((UT_uint8)charset));
-	if (!UT_iconv_isValid(converter))
-		UT_THROW(UT_IE_BOGUSDOCUMENT);
+		readByteString(docInfo, headStr);
+		if (strcmp(headStr, "SfxDocumentInfo") != 0)
+			UT_THROW(UT_IE_BOGUSDOCUMENT);
 
-	bool graphPortable, queryTemplateReload;
-	streamRead(docInfo, graphPortable);
-	streamRead(docInfo, queryTemplateReload);
+		UT_uint16 version;
+		streamRead(docInfo, version);
+		bool pwProtect;
+		streamRead(docInfo, pwProtect);
+		UT_uint16 charset;
+		streamRead(docInfo, charset);
+		auto_iconv converter(findConverter((UT_uint8)charset));
+		if (!UT_iconv_isValid(converter))
+			UT_THROW(UT_IE_BOGUSDOCUMENT);
 
-	TimeStamp stamp(converter);
-	stamp.load(docInfo); // creator
-	do_SetMetadata(aPDDoc, PD_META_KEY_CREATOR, stamp.mString);
-	aPDDoc->setMetaDataProp(PD_META_KEY_DATE, stamp.ToString());
-	stamp.load(docInfo); // modifier
-	// (ab-)using Contributor as the person who last modified the doc...
-	do_SetMetadata(aPDDoc, PD_META_KEY_CONTRIBUTOR, stamp.mString);
-	aPDDoc->setMetaDataProp(PD_META_KEY_DATE_LAST_CHANGED, stamp.ToString());
-	stamp.load(docInfo); // printer (person, not device)
+		bool graphPortable, queryTemplateReload;
+		streamRead(docInfo, graphPortable);
+		streamRead(docInfo, queryTemplateReload);
 
-	UT_UCS4String data;
-	readPaddedByteString(docInfo, data, converter, 63);
-	do_SetMetadata(aPDDoc, PD_META_KEY_TITLE, data);
-	readPaddedByteString(docInfo, data, converter, 63);
-	do_SetMetadata(aPDDoc, PD_META_KEY_SUBJECT, data);
-	readPaddedByteString(docInfo, data, converter, 255);
-	do_SetMetadata(aPDDoc, PD_META_KEY_DESCRIPTION, data);
-	readPaddedByteString(docInfo, data, converter, 127);
-	do_SetMetadata(aPDDoc, PD_META_KEY_KEYWORDS, data);
+		TimeStamp stamp(converter);
+		stamp.load(docInfo); // creator
+		do_SetMetadata(aPDDoc, PD_META_KEY_CREATOR, stamp.mString);
+		aPDDoc->setMetaDataProp(PD_META_KEY_DATE, stamp.ToString());
+		stamp.load(docInfo); // modifier
+		// (ab-)using Contributor as the person who last modified the doc...
+		do_SetMetadata(aPDDoc, PD_META_KEY_CONTRIBUTOR, stamp.mString);
+		aPDDoc->setMetaDataProp(PD_META_KEY_DATE_LAST_CHANGED, stamp.ToString());
+		stamp.load(docInfo); // printer (person, not device)
 
-	// Read user-defined data
-	for (int i = 0; i < 4; i++) {
-		UT_UCS4String key, value;
-		readPaddedByteString(docInfo, key, converter, 19);
-		readPaddedByteString(docInfo, value, converter, 19);
-		UT_String prefixedKey = CUSTOM_META_PREFIX + UT_String(UT_UTF8String(key).utf8_str());
-		do_SetMetadata(aPDDoc, prefixedKey, value);
+		UT_UCS4String data;
+		readPaddedByteString(docInfo, data, converter, 63);
+		do_SetMetadata(aPDDoc, PD_META_KEY_TITLE, data);
+		readPaddedByteString(docInfo, data, converter, 63);
+		do_SetMetadata(aPDDoc, PD_META_KEY_SUBJECT, data);
+		readPaddedByteString(docInfo, data, converter, 255);
+		do_SetMetadata(aPDDoc, PD_META_KEY_DESCRIPTION, data);
+		readPaddedByteString(docInfo, data, converter, 127);
+		do_SetMetadata(aPDDoc, PD_META_KEY_KEYWORDS, data);
+
+		// Read user-defined data
+		for (int i = 0; i < 4; i++) {
+			UT_UCS4String key, value;
+			readPaddedByteString(docInfo, key, converter, 19);
+			readPaddedByteString(docInfo, value, converter, 19);
+			UT_String prefixedKey = CUSTOM_META_PREFIX + UT_String(UT_UTF8String(key).utf8_str());
+			do_SetMetadata(aPDDoc, prefixedKey, value);
+		}
+
+		// rest of the stream contains no useful information
+
+		delete [] headStr;
+		UT_DEBUGMSG(("SDW: Docinfo done loading\n"));
+	}
+	catch(UT_Error & e) {
+		if(headStr) {
+			delete [] headStr;
+		}
+		throw e;
 	}
-
-	// rest of the stream contains no useful information
-
-	// FIXME make sure the exception cleanup go here...
-	delete [] headStr;
-	UT_DEBUGMSG(("SDW: Docinfo done loading\n"));
 }
 
