@@ -239,7 +239,7 @@ void streamRead(GsfInput* aStream, TextAttr& aAttr, gsf_off_t aEoa) UT_THROWS((U
 			UT_uint16 fontLen = GSF_LE_GET_GUINT16(aAttr.data + 3);
 			UT_String_sprintf(aAttr.attrVal, "%.*s", fontLen, (aAttr.data + 5));
 			
-			break; }
+		break; }
     case 0x1007: // font height
       // structure: | height (2 byte, twips) | prop (?) (2 byte) (if version >= 2, if ver=1 1 byte) | unit (if version>=2) |
       // XXX we ignore "prop" and unit for now, they seem not used much
@@ -327,8 +327,8 @@ void streamRead(GsfInput* aStream, TextAttr& aAttr, gsf_off_t aEoa) UT_THROWS((U
       // Data structure:
       // Count(8) | Position (in twips) (32) | Adjustment (8) | Decimal Separator (?) (8) | Fill character (8)
       // (total size per tab = 7)
-      UT_sint8 count = aAttr.data[0];
-      for (int i = 1; (i + 6) < aAttr.dataLen; i += 7) {
+      // UT_sint8 count = aAttr.data[0];
+      for (UT_uint32 i = 1; (i + 6) < aAttr.dataLen; i += 7) {
         // Abiword wants: 12.3cm/L0, where 0 indicates what to fill with
         UT_uint16 posInTwips = GSF_LE_GET_GUINT32(aAttr.data + i);
         UT_String pos = twipsToSizeString(posInTwips);
@@ -377,7 +377,7 @@ void streamRead(GsfInput* aStream, TextAttr& aAttr, gsf_off_t aEoa) UT_THROWS((U
 static void hexdump(void* aPtr, UT_uint32 aLen) {
 	unsigned char* ptr = (unsigned char*)aPtr;
 	for (UT_uint32 i = 0; i < aLen; i++) {
-		fprintf(stderr, "%02x ", ptr[i], ptr[i]);
+		fprintf(stderr, "%02x ", ptr[i]);
 	}
 }
 #endif
@@ -396,7 +396,7 @@ static UT_String _getPassword (XAP_Frame * pFrame)
 	= static_cast<XAP_DialogFactory *>(pFrame->getDialogFactory());
       
       XAP_Dialog_Password * pDlg = static_cast<XAP_Dialog_Password*>(pDialogFactory->requestDialog(XAP_DIALOG_ID_PASSWORD));
-      UT_ASSERT(pDlg);
+      UT_return_val_if_fail(pDlg,password);
       
       pDlg->runModal (pFrame);
       
@@ -457,26 +457,51 @@ IE_Imp_StarOffice_Sniffer::IE_Imp_StarOffice_Sniffer () :
   // 
 }
 
-UT_Confidence_t IE_Imp_StarOffice_Sniffer::supportsMIME (const char * szMIME)
+// supported suffixes
+static IE_SuffixConfidence IE_Imp_StarOffice_Sniffer__SuffixConfidence[] = {
+	{ "sdw", 	UT_CONFIDENCE_PERFECT 	},
+	{ "", 	UT_CONFIDENCE_ZILCH 	}
+};
+
+const IE_SuffixConfidence * IE_Imp_StarOffice_Sniffer::getSuffixConfidence ()
 {
-  if (UT_strcmp (IE_FileInfo::mapAlias (szMIME), IE_MIME_SDW) == 0)
-    {
-      return UT_CONFIDENCE_GOOD;
-    }
-  return UT_CONFIDENCE_ZILCH;
+	return IE_Imp_StarOffice_Sniffer__SuffixConfidence;
+}
+
+// supported mimetypes
+static IE_MimeConfidence IE_Imp_StarOffice_Sniffer__MimeConfidence[] = {
+	{ IE_MIME_MATCH_FULL, 	"application/vnd.stardivision.writer",	UT_CONFIDENCE_GOOD 	},	/* seems to be the official mimetype */
+	{ IE_MIME_MATCH_FULL, 	"application/x-staroffice-word", 		UT_CONFIDENCE_GOOD 	},	/* aliases */
+	{ IE_MIME_MATCH_FULL, 	"application/x-staroffice-words", 		UT_CONFIDENCE_GOOD 	},
+	{ IE_MIME_MATCH_FULL, 	"application/x-starwriter", 			UT_CONFIDENCE_GOOD 	},
+	{ IE_MIME_MATCH_BOGUS, 	"", 									UT_CONFIDENCE_ZILCH }
+};
+
+const IE_MimeConfidence * IE_Imp_StarOffice_Sniffer::getMimeConfidence ()
+{
+	return IE_Imp_StarOffice_Sniffer__MimeConfidence;
 }
 
 static const UT_Byte sdwSignature[] = {0xD0, 0xCF, 0x11, 0xE0, 0xA1, 0xB1, 0x1A, 0xE1};
 
-UT_Confidence_t IE_Imp_StarOffice_Sniffer::recognizeContents(const char* szBuf, UT_uint32 iNumBytes) {
-	if (iNumBytes >= sizeof(sdwSignature)) {
-		return (memcmp(szBuf, sdwSignature, sizeof(sdwSignature)) == 0) ? UT_CONFIDENCE_GOOD : UT_CONFIDENCE_ZILCH;
-	}
-	return UT_CONFIDENCE_ZILCH;
-}
+UT_Confidence_t IE_Imp_StarOffice_Sniffer::recognizeContents(GsfInput * input) {
+	GsfInfile * ole;
+	UT_Confidence_t confidence = UT_CONFIDENCE_ZILCH;
 
-UT_Confidence_t IE_Imp_StarOffice_Sniffer::recognizeSuffix(const char* szSuffix) {
-	return (UT_stricmp(szSuffix, ".sdw") == 0) ? UT_CONFIDENCE_PERFECT : UT_CONFIDENCE_ZILCH;
+	ole = gsf_infile_msole_new (input, NULL);
+	if (ole)
+		{
+			GsfInput * starWriterDocument = gsf_infile_child_by_name (ole, "StarWriterDocument");
+			if (starWriterDocument != NULL)
+				{
+					confidence = UT_CONFIDENCE_PERFECT;
+					g_object_unref (G_OBJECT (starWriterDocument));
+				}
+
+			g_object_unref (G_OBJECT (ole));
+		}
+
+	return confidence;
 }
 
 UT_Error IE_Imp_StarOffice_Sniffer::constructImporter(PD_Document *pDocument, IE_Imp **ppie) {
@@ -550,7 +575,7 @@ void DocHdr::load(GsfInput* stream) UT_THROWS((UT_Error)) {
 	if (nRecSzPos != 0 && nVersion >= SWG_RECSIZES) {
 		// Read the Recsizes
 		// XXX to be done see sw/source/core/sw3io/sw3imp.cxx#L1070
-		UT_ASSERT(UT_NOT_IMPLEMENTED);
+		UT_ASSERT_HARMLESS(UT_NOT_IMPLEMENTED);
 	}
 
 	if (nFileFlags & SWGF_BAD_FILE)
@@ -587,7 +612,7 @@ void IE_Imp_StarOffice::readRecSize(GsfInput* aStream, UT_uint32& aSize, gsf_off
 	aSize -= 4; // Substract 4 for the rec type + size
 	if (aSize == 0xFFFFFF && mDocHdr.nVersion >= SWG_LONGRECS) {
 		// XXX need recsizes from header, see above
-		UT_ASSERT(UT_NOT_IMPLEMENTED);
+		UT_ASSERT_HARMLESS(UT_NOT_IMPLEMENTED);
 	}
 	if (aEOR)
 		*aEOR = gsf_input_tell(aStream) + aSize;
@@ -600,14 +625,10 @@ void readFlagRec(GsfInput* stream, UT_uint8& flags, gsf_off_t* newPos) UT_THROWS
 }
 
 
-UT_Error IE_Imp_StarOffice::importFile(const char* szFilename) UT_THROWS(()) {
+UT_Error IE_Imp_StarOffice::_loadFile(GsfInput * input) UT_THROWS(()) {
 	UT_TRY {
 		UT_DEBUGMSG(("SDW: Starting import\n"));
-		GsfInput *input = GSF_INPUT (gsf_input_stdio_new(szFilename, NULL));
-		if (!input)
-			return UT_IE_BOGUSDOCUMENT;
 		mOle = GSF_INFILE (gsf_infile_msole_new(input, NULL));
-		g_object_unref (G_OBJECT (input));
 		if (!mOle)
 			return UT_IE_BOGUSDOCUMENT;
 
@@ -752,23 +773,23 @@ UT_Error IE_Imp_StarOffice::importFile(const char* szFilename) UT_THROWS(()) {
 										return UT_IE_BOGUSDOCUMENT;
 								}
 
-								static XML_Char* attributes[3] = {
+								static const gchar* attributes[3] = {
 									"props",
 									NULL,
 									NULL
 								};
-								attributes[1] = (XML_Char*)pAttrs.c_str();
+								attributes[1] = const_cast<gchar*>(reinterpret_cast<const gchar*>(pAttrs.c_str()));
 								// first, insert the paragraph
-								if (!appendStrux(PTX_Block, (const XML_Char**)attributes))
+								if (!appendStrux(PTX_Block, (const gchar**)attributes))
 									return UT_IE_NOMEMORY;
 
 								UT_String pca(attrs); // character attributes for the whole paragraph
 								// now insert the spans of text
 								UT_uint32 len = textNode.length();
 								UT_uint32 lastInsPos = 0;
-								for (int i = 1; i < len; i++) {
+								for (UT_uint32 i = 1; i < len; i++) {
 									bool doInsert = false; // whether there was an attribute change
-									for (int j = 0; j < charAttributes.getItemCount(); j++) {
+									for (UT_uint32 j = 0; j < charAttributes.getItemCount(); j++) {
 										const TextAttr* a = reinterpret_cast<const TextAttr*>(charAttributes[j]);
 										// clear the last attribute, if set
 										if (a->endSet && a->end == (i - 1)) {
@@ -795,9 +816,9 @@ UT_Error IE_Imp_StarOffice::importFile(const char* szFilename) UT_THROWS(()) {
 											doInsert = true;
 									}
 									if (doInsert || i == (len - 1)) {
-										attributes[1] = (XML_Char*)attrs.c_str();
+										attributes[1] = const_cast<gchar*>(reinterpret_cast<const gchar*>(attrs.c_str()));
 										UT_DEBUGMSG(("SDW: Going to appendFmt with %s\n", attributes[1]));
-										if (!appendFmt((const XML_Char **) attributes))
+										if (!appendFmt((const gchar **) attributes))
 											return UT_IE_NOMEMORY; /* leave cast alone! */
 										UT_DEBUGMSG(("SDW: About to insert %lu-%lu\n", lastInsPos, i));
 										size_t spanLen = i - lastInsPos;
@@ -906,7 +927,7 @@ UT_Error IE_Imp_StarOffice::importFile(const char* szFilename) UT_THROWS(()) {
 				case SWG_STRINGPOOL:
 				{
 					if (mDocHdr.nVersion <= SWG_POOLIDS) {
-						UT_ASSERT(UT_NOT_IMPLEMENTED);
+						UT_ASSERT_HARMLESS(UT_NOT_IMPLEMENTED);
 						break;
 					}
 					UT_uint8 encoding;
@@ -923,12 +944,17 @@ UT_Error IE_Imp_StarOffice::importFile(const char* szFilename) UT_THROWS(()) {
 						UT_uint16 len;
 						::readByteString(mDocStream, str, &len);
 						if (id == IDX_NOCONV_FF) {
-							UT_ASSERT(UT_NOT_IMPLEMENTED);
+							UT_ASSERT_HARMLESS(UT_NOT_IMPLEMENTED);
 						}
+						// FIXME: find a way to not have to copy and free 
+						// the result of UT_convert_cd.... --hub
 						UT_DEBUGMSG(("SDW: StringPool: found 0x%04x <-> %.*s\n", id, len, str));
 						UT_UCS4Char* convertedString = reinterpret_cast<UT_UCS4Char*>(UT_convert_cd(str, len, cd, NULL, NULL));
-						mStringPool.insert(id, convertedString);
+						mStringPool.insert(stringpool_map::value_type(id, convertedString));
+						FREEP(convertedString);
+                        delete [] str;
 					}
+                    UT_iconv_close(cd);
 					break;
 				}
 				case SWG_COMMENT: // skip over comments
@@ -974,10 +1000,6 @@ int abi_plugin_register (XAP_ModuleInfo * mi)
     {
     	m_impSniffer = new IE_Imp_StarOffice_Sniffer ();
     }
-    else
-    {
-	m_impSniffer->ref();
-    }
 
     mi->name    = "StarOffice .sdw file importer";
     mi->desc    = "Imports StarWriter binary (OLE) documents";
@@ -987,8 +1009,6 @@ int abi_plugin_register (XAP_ModuleInfo * mi)
   
     IE_Imp::registerImporter (m_impSniffer);
 
-		// finally, initialize libgsf
-		gsf_init();
     return 1;
 }
 
@@ -1004,17 +1024,15 @@ int abi_plugin_unregister (XAP_ModuleInfo * mi)
     UT_ASSERT (m_impSniffer);
 
     IE_Imp::unregisterImporter (m_impSniffer);
-    if(!m_impSniffer->unref())
-    {
+	delete m_impSniffer;
 	m_impSniffer = 0;
-    }
-		gsf_shutdown();
+
     return 1;
 }
 
 ABI_FAR_CALL
-int abi_plugin_supports_version (UT_uint32 major, UT_uint32 minor, 
-                                 UT_uint32 release)
+int abi_plugin_supports_version (UT_uint32 /*major*/, UT_uint32 /*minor*/, 
+                                 UT_uint32 /*release*/)
 {
     return 1;
 }
