@@ -30,7 +30,7 @@ class PD_Document;
 
 // The importer/reader for WML files.
 
-class ABI_EXPORT IE_Imp_WML_Sniffer : public IE_ImpSniffer
+class IE_Imp_WML_Sniffer : public IE_ImpSniffer
 {
 	friend class IE_Imp;
 
@@ -38,9 +38,10 @@ public:
 	IE_Imp_WML_Sniffer(const char * name);
 	virtual ~IE_Imp_WML_Sniffer() {}
 
+	virtual const IE_SuffixConfidence * getSuffixConfidence ();
+	virtual const IE_MimeConfidence * getMimeConfidence ();
 	virtual UT_Confidence_t recognizeContents (const char * szBuf, 
 									UT_uint32 iNumbytes);
-	virtual UT_Confidence_t recognizeSuffix (const char * szSuffix);
 	virtual bool getDlgLabels (const char ** szDesc,
 							   const char ** szSuffixList,
 							   IEFileType * ft);
@@ -49,30 +50,29 @@ public:
 
 };
 
-class ABI_EXPORT IE_Imp_WML : public IE_Imp_XML
+class IE_Imp_WML : public IE_Imp_XML
 {
 public:
 	IE_Imp_WML(PD_Document * pDocument);
 	virtual ~IE_Imp_WML();
 
-	void			startElement(const XML_Char *name, 
-					      const XML_Char **atts);
-	void			endElement(const XML_Char *name);
-	void			charData(const XML_Char *s, int len);
+	void			startElement(const gchar *name, 
+					      const gchar **atts);
+	void			endElement(const gchar *name);
+	void			charData(const gchar *s, int len);
 
  private:
-	void openTable(const XML_Char **atts);
+	void openTable(const gchar **atts);
 	void closeTable(void);
-	void openRow(const XML_Char **atts);
+	void openRow(const gchar **atts);
 	void closeRow(void);
-	void openCell(const XML_Char **atts);
+	void openCell(const gchar **atts);
 	void closeCell(void);
-	void createImage(const char *name, const XML_Char **atts);
-	char* getPath(const char *file);
+	void createImage(const char *name, const gchar **atts);
 
-	UT_uint32					m_iColumns;
+	UT_sint32					m_iColumns;
 	UT_uint32					m_iImages;
-	UT_uint32					m_iOpenedColumns;
+	UT_sint32					m_iOpenedColumns;
 	IE_Imp_TableHelperStack *	m_TableHelperStack;
 };
 
