@@ -21,8 +21,6 @@
 #include "ut_bytebuf.h"
 #include "ut_base64.h"
 #include "ut_debugmsg.h"
-#include "ut_map.h"
-#include "ut_pair.h"
 #include "ut_path.h"
 #include "ut_locale.h"
 #include "pt_Types.h"
@@ -67,7 +65,7 @@ IE_Exp_XSL_FO_Sniffer::IE_Exp_XSL_FO_Sniffer (const char * name) :
 
 bool IE_Exp_XSL_FO_Sniffer::recognizeSuffix(const char * szSuffix)
 {
-	return (!UT_stricmp(szSuffix, ".fo"));
+	return (!g_ascii_strcasecmp(szSuffix, ".fo"));
 }
 
 UT_Error IE_Exp_XSL_FO_Sniffer::constructExporter(PD_Document * pDocument,
@@ -106,6 +104,30 @@ UT_Error IE_Exp_XSL_FO::_writeDocument()
 /*****************************************************************/
 /*****************************************************************/
 
+static char *_stripSuffix(const char* from, char delimiter)
+{
+	// taken from DocBook exporter
+	char * fremove_s = (char *)malloc(strlen(from)+1);
+	strcpy(fremove_s, from);   
+
+	char * p = fremove_s + strlen(fremove_s);
+	while ((p >= fremove_s) && (*p != delimiter))
+		p--;
+	
+	if (p >= fremove_s)
+	*p = '\0';
+	
+	return fremove_s;
+}
+
+static char *_stripSuffix(const UT_UTF8String &from, char delimiter)
+{
+  return _stripSuffix(from.utf8_str(), delimiter);
+}
+
+/*****************************************************************/
+/*****************************************************************/
+
 void s_XSL_FO_Listener::_tagClose(UT_uint32 tagID, const UT_UTF8String & content, bool newline)
 {
 	UT_uint32 i = 0;
@@ -604,11 +626,11 @@ void s_XSL_FO_Listener::_handleBookmark(PT_AttrPropIndex api)
 {
 	const PP_AttrProp * pAP = NULL;
 	bool bHaveProp = m_pDocument->getAttrProp(api,&pAP);
-	const XML_Char* szValue = NULL;
+	const gchar* szValue = NULL;
 
 	if(bHaveProp && pAP && pAP->getAttribute("type", szValue) && szValue)
 	{
-		if(!UT_strcmp(szValue, "start") && pAP->getAttribute("name", szValue) && szValue)
+		if(!strcmp(szValue, "start") && pAP->getAttribute("name", szValue) && szValue)
 		{
 			UT_UTF8String buf, anchor = szValue;
 			anchor.escapeXML();
@@ -628,7 +650,7 @@ void s_XSL_FO_Listener::_handleEmbedded(PT_AttrPropIndex api)
 {
 	const PP_AttrProp * pAP = NULL;
 	bool bHaveProp = m_pDocument->getAttrProp(api,&pAP);
-	const XML_Char* szValue = NULL;
+	const gchar* szValue = NULL;
 
 	UT_return_if_fail(bHaveProp && pAP && pAP->getAttribute("dataid", szValue) && szValue);
 
@@ -637,10 +659,10 @@ void s_XSL_FO_Listener::_handleEmbedded(PT_AttrPropIndex api)
 	buf = "snapshot-png-";
 	buf += szValue;
 	buf.escapeXML();
-	char* dataid = UT_strdup(buf.utf8_str());
+	char* dataid = g_strdup(buf.utf8_str());
 	m_utvDataIDs.push_back(dataid);
 
-	url = UT_basename(m_pie->getFileName());
+	url = UT_go_basename(m_pie->getFileName());
 	url.escapeXML();
 
 	img = "external-graphic src=\"url('";
@@ -678,12 +700,14 @@ void s_XSL_FO_Listener::_handleField(const PX_ChangeRecord_Object * pcro, PT_Att
 	
 	if (bHaveProp && pAP)
 	{
-		const XML_Char* szValue = NULL;
+		const gchar* szValue = NULL;
 		if (pAP->getAttribute("type", szValue) && szValue)
 		{
 			fd_Field * field = pcro->getField();
 
-			if((_tagTop() == TT_LISTBLOCK) && !UT_strcmp(static_cast<const char*>(szValue), "list_label"))
+			m_pie->populateFields ();
+
+			if((_tagTop() == TT_LISTBLOCK) && !strcmp(static_cast<const char*>(szValue), "list_label"))
 			{
 				m_pie->write("\n");
 
@@ -692,7 +716,7 @@ void s_XSL_FO_Listener::_handleField(const PX_ChangeRecord_Object * pcro, PT_Att
 				_tagOpen(TT_BLOCK, "block", false);
 
 				UT_UTF8String label = "";
-				for(int i = 0; i < m_Lists.getItemCount(); i++)
+				for(UT_uint32 i = 0; i < m_Lists.getItemCount(); i++)
 				{
 					ListHelper * lh = m_Lists[i];
 					if(lh && ((*lh).retrieveID() == m_iListID))
@@ -713,7 +737,7 @@ void s_XSL_FO_Listener::_handleField(const PX_ChangeRecord_Object * pcro, PT_Att
 				m_iBlockDepth++;
 				m_bWroteListField = true;
 			}
-			else if(!UT_strcmp(szValue, "footnote_ref"))
+			else if(!strcmp(szValue, "footnote_ref"))
 			{
 				UT_UTF8String buf = field->getValue();
 				buf.escapeXML();
@@ -742,13 +766,10 @@ void s_XSL_FO_Listener::_handleFrame(PT_AttrPropIndex api)
 {
 	const PP_AttrProp * pAP = NULL;
 	bool bHaveProp = m_pDocument->getAttrProp(api,&pAP);
-	const XML_Char * szValue = NULL;
+	const gchar * szValue = NULL;
 
 	if(pAP && bHaveProp && (pAP->getAttribute("strux-image-dataid", szValue)) && szValue)
 	{
-		if(!UT_strcmp("",szValue))  // occurs with word documents
-			return;
-
 		_handlePositionedImage(api);
 		return;
 	}
@@ -760,7 +781,7 @@ void s_XSL_FO_Listener::_handleHyperlink(PT_AttrPropIndex api)
 {
 	const PP_AttrProp * pAP = NULL;
 	bool bHaveProp = m_pDocument->getAttrProp(api,&pAP);
-	const XML_Char* szValue = NULL;
+	const gchar* szValue = NULL;
 
 	if(bHaveProp && pAP && pAP->getAttribute("xlink:href", szValue) && szValue)
 	{
@@ -801,16 +822,16 @@ void s_XSL_FO_Listener::_handleImage(PT_AttrPropIndex api)
 {
 	const PP_AttrProp * pAP = NULL;
 	bool bHaveProp = m_pDocument->getAttrProp(api,&pAP);
-	const XML_Char* szValue = NULL;
+	const gchar* szValue = NULL;
 
 	UT_return_if_fail(bHaveProp && pAP && pAP->getAttribute("dataid", szValue) && szValue);
 
-	char * dataid = UT_strdup(const_cast<char*>(szValue));
+	char * dataid = g_strdup(const_cast<char*>(szValue));
 	m_utvDataIDs.push_back(dataid);
 
 	UT_UTF8String buf, img, url;
 
-	url = UT_basename(m_pie->getFileName());
+	url = UT_go_basename(m_pie->getFileName());
 	url.escapeXML();
 	buf = szValue;
 	buf.escapeXML();
@@ -826,17 +847,14 @@ void s_XSL_FO_Listener::_handleImage(PT_AttrPropIndex api)
 
 	if(pAP->getProperty("width", szValue) && szValue)
 	{
-		UT_UTF8String_sprintf(buf, "%f", UT_convertToDimension(szValue, DIM_PX));
 		img += " content-width=\"";
-		img += buf;
+		img += szValue;
 		img += "\"";
-		buf.clear();
 	}
 	if(pAP->getProperty("height", szValue) && szValue)
 	{
-		UT_UTF8String_sprintf(buf, "%f", UT_convertToDimension(szValue, DIM_PX));
 		img += " content-height=\"";
-		img += buf;
+		img += szValue;
 		img += "\"";
 	}
 	//note: language isn't a valid attribute
@@ -850,16 +868,16 @@ void s_XSL_FO_Listener::_handlePositionedImage(PT_AttrPropIndex api)
 
 	const PP_AttrProp * pAP = NULL;
 	bool bHaveProp = m_pDocument->getAttrProp(api,&pAP);
-	const XML_Char* szValue = NULL;
+	const gchar* szValue = NULL;
 
 	UT_return_if_fail(bHaveProp && pAP && pAP->getAttribute("strux-image-dataid", szValue) && szValue);
 
-	char * dataid = UT_strdup(const_cast<char*>(szValue));
+	char * dataid = g_strdup(const_cast<char*>(szValue));
 	m_utvDataIDs.push_back(dataid);
 
 	UT_UTF8String buf, img, url;
 
-	url = UT_basename(m_pie->getFileName());
+	url = UT_go_basename(m_pie->getFileName());
 	url.escapeXML();
 	buf = szValue;
 	buf.escapeXML();
@@ -875,17 +893,14 @@ void s_XSL_FO_Listener::_handlePositionedImage(PT_AttrPropIndex api)
 
 	if(pAP->getProperty("width", szValue) && szValue)
 	{
-		UT_UTF8String_sprintf(buf, "%f", UT_convertToDimension(szValue, DIM_PX));
 		img += " content-width=\"";
-		img += buf;
+		img += szValue;
 		img += "\"";
-		buf.clear();
 	}
 	if(pAP->getProperty("height", szValue) && szValue)
 	{
-		UT_UTF8String_sprintf(buf, "%f", UT_convertToDimension(szValue, DIM_PX));
 		img += " content-height=\"";
-		img += buf;
+		img += szValue;
 		img += "\"";
 	}
 	//note: language isn't a valid attribute
@@ -897,7 +912,7 @@ void s_XSL_FO_Listener::_handleMath(PT_AttrPropIndex api)
 {
 	const PP_AttrProp * pAP = NULL;
 	bool bHaveProp = m_pDocument->getAttrProp(api,&pAP);
-	const XML_Char* szValue = NULL;
+	const gchar* szValue = NULL;
 
 	UT_return_if_fail(bHaveProp && pAP && pAP->getAttribute("dataid", szValue) && szValue);
 
@@ -908,10 +923,10 @@ void s_XSL_FO_Listener::_handleMath(PT_AttrPropIndex api)
 	buf = "snapshot-png-";
 	buf += szValue;
 	buf.escapeXML();
-	char * dataid = UT_strdup(buf.utf8_str());
+	char * dataid = g_strdup(buf.utf8_str());
 	m_utvDataIDs.push_back(dataid);
 
-	url = UT_basename(m_pie->getFileName());
+	url = UT_go_basename(m_pie->getFileName());
 	url.escapeXML();
 
 	img = "external-graphic src=\"url('";
@@ -1052,11 +1067,11 @@ bool s_XSL_FO_Listener::populateStrux(PL_StruxDocHandle sdh,
 		const PP_AttrProp* pAP = NULL;
 		if (m_pDocument->getAttrProp(api, &pAP) && pAP)
 		{
-			const XML_Char* pszSectionType = NULL;
+			const gchar* pszSectionType = NULL;
 			pAP->getAttribute("type", pszSectionType);
 			if (
 				!pszSectionType
-				|| (0 == UT_strcmp(pszSectionType, "doc"))
+				|| (0 == strcmp(pszSectionType, "doc"))
 				)
 			{
 				_openSection(pcr->getIndexAP());
@@ -1279,7 +1294,7 @@ void s_XSL_FO_Listener::_handlePageSize(PT_AttrPropIndex api)
 	// todo - validate these and make sure they all make sense
 	if (bHaveProp && pAP)
 	{
-		const XML_Char * szValue = NULL;
+		const gchar * szValue = NULL;
 
 		szValue = PP_evalProperty("page-margin-top",
 								  NULL, NULL, pAP, m_pDocument, true);
@@ -1320,14 +1335,14 @@ void s_XSL_FO_Listener::_handlePageSize(PT_AttrPropIndex api)
 		UT_Dimension docUnit = m_pDocument->m_docPageSize.getDims(); 
 		UT_UTF8String buf;
 
+		// page-width, page-height
 		UT_UTF8String_sprintf(buf, " page-width=\"%f%s\"", m_pDocument->m_docPageSize.Width(docUnit), UT_dimensionName(docUnit));
 		master += buf;
 		buf.clear();
 
-		UT_UTF8String_sprintf(" page-height=\"%f%s\"", m_pDocument->m_docPageSize.Height(docUnit), UT_dimensionName(docUnit));
+		UT_UTF8String_sprintf(buf, " page-height=\"%f%s\"", m_pDocument->m_docPageSize.Height(docUnit), UT_dimensionName(docUnit));
 		master += buf;
 	}
-	// page-width, page-height
 
 	master += " master-name=\"first\"";
 
@@ -1352,7 +1367,7 @@ void s_XSL_FO_Listener::_handleDataItems(void)
 		UT_sint32 loc = -1;
 		for (UT_uint32 i = 0; i < m_utvDataIDs.getItemCount(); i++)
 		{
-			if(UT_strcmp(const_cast<char*>(reinterpret_cast<const char*>(m_utvDataIDs[i])), szName) == 0)
+			if(strcmp(const_cast<char*>(reinterpret_cast<const char*>(m_utvDataIDs[i])), szName) == 0)
 			{
 				loc = i;
 				break;
@@ -1361,19 +1376,18 @@ void s_XSL_FO_Listener::_handleDataItems(void)
 
 		if(loc > -1)
 		{
-			FILE *fp;
 			UT_UTF8String fname;
 
 			UT_UTF8String_sprintf(fname, "%s_data", m_pie->getFileName());
-			m_pDocument->getApp()->makeDirectory(fname.utf8_str(), 0750);
+			UT_go_directory_create(fname.utf8_str(), 0750, NULL);
 
-			if (!UT_strcmp(szMimeType, "image/svg+xml"))
+			if (!strcmp(szMimeType, "image/svg+xml"))
 				UT_UTF8String_sprintf(fname, "%s/%d.svg", fname.utf8_str(), loc);
-			if (!UT_strcmp(szMimeType, "application/mathml+xml"))
+			if (!strcmp(szMimeType, "application/mathml+xml"))
 				UT_UTF8String_sprintf(fname, "%s/%d.mathml", fname.utf8_str(), loc);
 			else // PNG Image
 			{  
-				char * temp = _stripSuffix(UT_basename(szName), '_');
+				char * temp = _stripSuffix(UT_go_basename(szName), '_');
 				char * fstripped = _stripSuffix(temp, '.');
 				UT_UTF8String_sprintf(fname, "%s/%s.png", fname.utf8_str(), fstripped);
 
@@ -1381,20 +1395,14 @@ void s_XSL_FO_Listener::_handleDataItems(void)
 				FREEP(fstripped);
 			}
 	  
-			fp = fopen (fname.utf8_str(), "wb+");
+			GsfOutput *fp = UT_go_file_create (fname.utf8_str(), NULL);
 	  
 			if(!fp)
 				continue;
 	  
-			int cnt = 0, len = pByteBuf->getLength();
-	  
-			while (cnt < len)
-			{
-				xxx_UT_DEBUGMSG(("DOM: len: %d cnt: %d\n", len, cnt));
-				cnt += fwrite (pByteBuf->getPointer(cnt), sizeof(UT_Byte), len-cnt, fp);
-			}
-
-			fclose(fp);
+			gsf_output_write(fp, pByteBuf->getLength(), (const guint8 *)pByteBuf->getPointer(0));
+			gsf_output_close(fp);
+			g_object_unref(fp);
 		}
 	}
 
@@ -1431,7 +1439,7 @@ void s_XSL_FO_Listener::_openBlock(PT_AttrPropIndex api)
 	UT_UTF8String buf;
 	const PP_AttrProp* pAP = 0;
 	bool bHaveProp = m_pDocument->getAttrProp(api, &pAP), bList = false;
-	const XML_Char* szValue = NULL;
+	const gchar* szValue = NULL;
 
 	if(bHaveProp && pAP && pAP->getAttribute("level", szValue) && szValue)
 	{
@@ -1536,7 +1544,7 @@ void s_XSL_FO_Listener::_openSpan(PT_AttrPropIndex api)
 	// query and output properties
 	if (bHaveProp && pAP)
 	{
-		const XML_Char * szValue = NULL;
+		const gchar * szValue = NULL;
 
 		if (pAP->getProperty("bgcolor", szValue) && szValue)
 		{
@@ -1784,19 +1792,3 @@ void s_XSL_FO_Listener::_outputData(const UT_UCSChar * data, UT_uint32 length)
 
 	m_pie->write(sBuf.utf8_str(), sBuf.byteLength());
 }
-
-char *s_XSL_FO_Listener::_stripSuffix(const char* from, char delimiter)
-{
-	// taken from DocBook exporter
-	char * fremove_s = (char *)malloc(strlen(from)+1);
-	strcpy(fremove_s, from);   
-
-	char * p = fremove_s + strlen(fremove_s);
-	while ((p >= fremove_s) && (*p != delimiter))
-		p--;
-	
-	if (p >= fremove_s)
-	*p = '\0';
-	
-	return fremove_s;
-}
