@@ -33,7 +33,7 @@
 #include "ut_misc.h"
 #include "pt_Types.h"
 #include "ut_assert.h"
-#include "ap_Strings.h"
+#include "xap_Strings.h"
 #include "fl_BlockLayout.h"
 #include "pp_Revision.h"
 #include "ut_string_class.h"
@@ -339,6 +339,10 @@ public:
 #ifdef FMT_TEST
 	virtual void		__dump(FILE * fp) const;
 #endif
+	void               setMustClearScreen(void)
+	{ m_bMustClearScreen = true;}
+	bool               getMustClearScreen(void)
+	{return m_bMustClearScreen;}
 
 protected:
 	virtual bool        _deleteFollowingIfAtInsPoint() const;
@@ -376,8 +380,8 @@ protected:
 	void				_setDirection(UT_BidiCharType c) { m_iDirection = c; }
 	UT_BidiCharType		_getDirection(void) const { return m_iDirection; }
 	UT_BidiCharType		_getVisDirection(void) const { return m_iVisDirection; }
-	GR_Font *			_getFont(void) const;
-	void  				_setFont(GR_Font * f);
+	const GR_Font *			_getFont(void) const;
+	void  				_setFont(const GR_Font * f);
 	unsigned char		_getDecorations(void) const { return m_fDecorations; }
 	void				_setDecorations(unsigned char d) {m_fDecorations = d;}
 	
@@ -449,7 +453,7 @@ private:
 	// the run highlight color. If the property is transparent use the page color
 	UT_RGBColor             m_pColorHL;
 
-	GR_Font * 				m_pFont;
+	const GR_Font * 			m_pFont;
 
 	bool					m_bRecalcWidth;
 
@@ -480,6 +484,7 @@ private:
     bool                    m_bDrawSelection;
     PT_DocPosition          m_iSelLow;
     PT_DocPosition          m_iSelHigh;
+	bool                    m_bMustClearScreen;
 #ifdef DEBUG
 	UT_uint32               m_iFontAllocNo;
 #endif
